// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// This file is auto-generated. All changes will be lost. See tools/update-emojis.py
// clang-format off
{u8"waving hand", u8"\U0001F44B\U0001F3FB", u8"wave_tone1", u8"waving hand: light skin tone"},
{u8"waving hand", u8"\U0001F44B\U0001F3FC", u8"wave_tone2", u8"waving hand: medium-light skin tone"},
{u8"waving hand", u8"\U0001F44B\U0001F3FD", u8"wave_tone3", u8"waving hand: medium skin tone"},
{u8"waving hand", u8"\U0001F44B\U0001F3FE", u8"wave_tone4", u8"waving hand: medium-dark skin tone"},
{u8"waving hand", u8"\U0001F44B\U0001F3FF", u8"wave_tone5", u8"waving hand: dark skin tone"},
{u8"raised back of hand", u8"\U0001F91A\U0001F3FB", u8"raised_back_of_hand_tone1", u8"raised back of hand: light skin tone"},
{u8"raised back of hand", u8"\U0001F91A\U0001F3FC", u8"raised_back_of_hand_tone2", u8"raised back of hand: medium-light skin tone"},
{u8"raised back of hand", u8"\U0001F91A\U0001F3FD", u8"raised_back_of_hand_tone3", u8"raised back of hand: medium skin tone"},
{u8"raised back of hand", u8"\U0001F91A\U0001F3FE", u8"raised_back_of_hand_tone4", u8"raised back of hand: medium-dark skin tone"},
{u8"raised back of hand", u8"\U0001F91A\U0001F3FF", u8"raised_back_of_hand_tone5", u8"raised back of hand: dark skin tone"},
{u8"hand with fingers splayed", u8"\U0001F590\U0001F3FB", u8"hand_splayed_tone1", u8"hand with fingers splayed: light skin tone"},
{u8"hand with fingers splayed", u8"\U0001F590\U0001F3FC", u8"hand_splayed_tone2", u8"hand with fingers splayed: medium-light skin tone"},
{u8"hand with fingers splayed", u8"\U0001F590\U0001F3FD", u8"hand_splayed_tone3", u8"hand with fingers splayed: medium skin tone"},
{u8"hand with fingers splayed", u8"\U0001F590\U0001F3FE", u8"hand_splayed_tone4", u8"hand with fingers splayed: medium-dark skin tone"},
{u8"hand with fingers splayed", u8"\U0001F590\U0001F3FF", u8"hand_splayed_tone5", u8"hand with fingers splayed: dark skin tone"},
{u8"raised hand", u8"\U0000270B\U0001F3FB", u8"raised_hand_tone1", u8"raised hand: light skin tone"},
{u8"raised hand", u8"\U0000270B\U0001F3FC", u8"raised_hand_tone2", u8"raised hand: medium-light skin tone"},
{u8"raised hand", u8"\U0000270B\U0001F3FD", u8"raised_hand_tone3", u8"raised hand: medium skin tone"},
{u8"raised hand", u8"\U0000270B\U0001F3FE", u8"raised_hand_tone4", u8"raised hand: medium-dark skin tone"},
{u8"raised hand", u8"\U0000270B\U0001F3FF", u8"raised_hand_tone5", u8"raised hand: dark skin tone"},
{u8"vulcan salute", u8"\U0001F596\U0001F3FB", u8"vulcan_tone1", u8"vulcan salute: light skin tone"},
{u8"vulcan salute", u8"\U0001F596\U0001F3FC", u8"vulcan_tone2", u8"vulcan salute: medium-light skin tone"},
{u8"vulcan salute", u8"\U0001F596\U0001F3FD", u8"vulcan_tone3", u8"vulcan salute: medium skin tone"},
{u8"vulcan salute", u8"\U0001F596\U0001F3FE", u8"vulcan_tone4", u8"vulcan salute: medium-dark skin tone"},
{u8"vulcan salute", u8"\U0001F596\U0001F3FF", u8"vulcan_tone5", u8"vulcan salute: dark skin tone"},
{u8"rightwards hand", u8"\U0001FAF1\U0001F3FB", u8"rightwards hand: light skin tone", u8"rightwards hand: light skin tone"},
{u8"rightwards hand", u8"\U0001FAF1\U0001F3FC", u8"rightwards hand: medium-light skin tone", u8"rightwards hand: medium-light skin tone"},
{u8"rightwards hand", u8"\U0001FAF1\U0001F3FD", u8"rightwards hand: medium skin tone", u8"rightwards hand: medium skin tone"},
{u8"rightwards hand", u8"\U0001FAF1\U0001F3FE", u8"rightwards hand: medium-dark skin tone", u8"rightwards hand: medium-dark skin tone"},
{u8"rightwards hand", u8"\U0001FAF1\U0001F3FF", u8"rightwards hand: dark skin tone", u8"rightwards hand: dark skin tone"},
{u8"leftwards hand", u8"\U0001FAF2\U0001F3FB", u8"leftwards hand: light skin tone", u8"leftwards hand: light skin tone"},
{u8"leftwards hand", u8"\U0001FAF2\U0001F3FC", u8"leftwards hand: medium-light skin tone", u8"leftwards hand: medium-light skin tone"},
{u8"leftwards hand", u8"\U0001FAF2\U0001F3FD", u8"leftwards hand: medium skin tone", u8"leftwards hand: medium skin tone"},
{u8"leftwards hand", u8"\U0001FAF2\U0001F3FE", u8"leftwards hand: medium-dark skin tone", u8"leftwards hand: medium-dark skin tone"},
{u8"leftwards hand", u8"\U0001FAF2\U0001F3FF", u8"leftwards hand: dark skin tone", u8"leftwards hand: dark skin tone"},
{u8"palm down hand", u8"\U0001FAF3\U0001F3FB", u8"palm down hand: light skin tone", u8"palm down hand: light skin tone"},
{u8"palm down hand", u8"\U0001FAF3\U0001F3FC", u8"palm down hand: medium-light skin tone", u8"palm down hand: medium-light skin tone"},
{u8"palm down hand", u8"\U0001FAF3\U0001F3FD", u8"palm down hand: medium skin tone", u8"palm down hand: medium skin tone"},
{u8"palm down hand", u8"\U0001FAF3\U0001F3FE", u8"palm down hand: medium-dark skin tone", u8"palm down hand: medium-dark skin tone"},
{u8"palm down hand", u8"\U0001FAF3\U0001F3FF", u8"palm down hand: dark skin tone", u8"palm down hand: dark skin tone"},
{u8"palm up hand", u8"\U0001FAF4\U0001F3FB", u8"palm up hand: light skin tone", u8"palm up hand: light skin tone"},
{u8"palm up hand", u8"\U0001FAF4\U0001F3FC", u8"palm up hand: medium-light skin tone", u8"palm up hand: medium-light skin tone"},
{u8"palm up hand", u8"\U0001FAF4\U0001F3FD", u8"palm up hand: medium skin tone", u8"palm up hand: medium skin tone"},
{u8"palm up hand", u8"\U0001FAF4\U0001F3FE", u8"palm up hand: medium-dark skin tone", u8"palm up hand: medium-dark skin tone"},
{u8"palm up hand", u8"\U0001FAF4\U0001F3FF", u8"palm up hand: dark skin tone", u8"palm up hand: dark skin tone"},
{u8"OK hand", u8"\U0001F44C\U0001F3FB", u8"ok_hand_tone1", u8"OK hand: light skin tone"},
{u8"OK hand", u8"\U0001F44C\U0001F3FC", u8"ok_hand_tone2", u8"OK hand: medium-light skin tone"},
{u8"OK hand", u8"\U0001F44C\U0001F3FD", u8"ok_hand_tone3", u8"OK hand: medium skin tone"},
{u8"OK hand", u8"\U0001F44C\U0001F3FE", u8"ok_hand_tone4", u8"OK hand: medium-dark skin tone"},
{u8"OK hand", u8"\U0001F44C\U0001F3FF", u8"ok_hand_tone5", u8"OK hand: dark skin tone"},
{u8"pinched fingers", u8"\U0001F90C\U0001F3FB", u8"pinched fingers: light skin tone", u8"pinched fingers: light skin tone"},
{u8"pinched fingers", u8"\U0001F90C\U0001F3FC", u8"pinched fingers: medium-light skin tone", u8"pinched fingers: medium-light skin tone"},
{u8"pinched fingers", u8"\U0001F90C\U0001F3FD", u8"pinched fingers: medium skin tone", u8"pinched fingers: medium skin tone"},
{u8"pinched fingers", u8"\U0001F90C\U0001F3FE", u8"pinched fingers: medium-dark skin tone", u8"pinched fingers: medium-dark skin tone"},
{u8"pinched fingers", u8"\U0001F90C\U0001F3FF", u8"pinched fingers: dark skin tone", u8"pinched fingers: dark skin tone"},
{u8"pinching hand", u8"\U0001F90F\U0001F3FB", u8"pinching hand: light skin tone", u8"pinching hand: light skin tone"},
{u8"pinching hand", u8"\U0001F90F\U0001F3FC", u8"pinching hand: medium-light skin tone", u8"pinching hand: medium-light skin tone"},
{u8"pinching hand", u8"\U0001F90F\U0001F3FD", u8"pinching hand: medium skin tone", u8"pinching hand: medium skin tone"},
{u8"pinching hand", u8"\U0001F90F\U0001F3FE", u8"pinching hand: medium-dark skin tone", u8"pinching hand: medium-dark skin tone"},
{u8"pinching hand", u8"\U0001F90F\U0001F3FF", u8"pinching hand: dark skin tone", u8"pinching hand: dark skin tone"},
{u8"victory hand", u8"\U0000270C\U0001F3FB", u8"v_tone1", u8"victory hand: light skin tone"},
{u8"victory hand", u8"\U0000270C\U0001F3FC", u8"v_tone2", u8"victory hand: medium-light skin tone"},
{u8"victory hand", u8"\U0000270C\U0001F3FD", u8"v_tone3", u8"victory hand: medium skin tone"},
{u8"victory hand", u8"\U0000270C\U0001F3FE", u8"v_tone4", u8"victory hand: medium-dark skin tone"},
{u8"victory hand", u8"\U0000270C\U0001F3FF", u8"v_tone5", u8"victory hand: dark skin tone"},
{u8"crossed fingers", u8"\U0001F91E\U0001F3FB", u8"fingers_crossed_tone1", u8"crossed fingers: light skin tone"},
{u8"crossed fingers", u8"\U0001F91E\U0001F3FC", u8"fingers_crossed_tone2", u8"crossed fingers: medium-light skin tone"},
{u8"crossed fingers", u8"\U0001F91E\U0001F3FD", u8"fingers_crossed_tone3", u8"crossed fingers: medium skin tone"},
{u8"crossed fingers", u8"\U0001F91E\U0001F3FE", u8"fingers_crossed_tone4", u8"crossed fingers: medium-dark skin tone"},
{u8"crossed fingers", u8"\U0001F91E\U0001F3FF", u8"fingers_crossed_tone5", u8"crossed fingers: dark skin tone"},
{u8"hand with index finger and thumb crossed", u8"\U0001FAF0\U0001F3FB", u8"hand with index finger and thumb crossed: light skin tone", u8"hand with index finger and thumb crossed: light skin tone"},
{u8"hand with index finger and thumb crossed", u8"\U0001FAF0\U0001F3FC", u8"hand with index finger and thumb crossed: medium-light skin tone", u8"hand with index finger and thumb crossed: medium-light skin tone"},
{u8"hand with index finger and thumb crossed", u8"\U0001FAF0\U0001F3FD", u8"hand with index finger and thumb crossed: medium skin tone", u8"hand with index finger and thumb crossed: medium skin tone"},
{u8"hand with index finger and thumb crossed", u8"\U0001FAF0\U0001F3FE", u8"hand with index finger and thumb crossed: medium-dark skin tone", u8"hand with index finger and thumb crossed: medium-dark skin tone"},
{u8"hand with index finger and thumb crossed", u8"\U0001FAF0\U0001F3FF", u8"hand with index finger and thumb crossed: dark skin tone", u8"hand with index finger and thumb crossed: dark skin tone"},
{u8"love-you gesture", u8"\U0001F91F\U0001F3FB", u8"love_you_gesture_tone1", u8"love-you gesture: light skin tone"},
{u8"love-you gesture", u8"\U0001F91F\U0001F3FC", u8"love_you_gesture_tone2", u8"love-you gesture: medium-light skin tone"},
{u8"love-you gesture", u8"\U0001F91F\U0001F3FD", u8"love_you_gesture_tone3", u8"love-you gesture: medium skin tone"},
{u8"love-you gesture", u8"\U0001F91F\U0001F3FE", u8"love_you_gesture_tone4", u8"love-you gesture: medium-dark skin tone"},
{u8"love-you gesture", u8"\U0001F91F\U0001F3FF", u8"love_you_gesture_tone5", u8"love-you gesture: dark skin tone"},
{u8"sign of the horns", u8"\U0001F918\U0001F3FB", u8"metal_tone1", u8"sign of the horns: light skin tone"},
{u8"sign of the horns", u8"\U0001F918\U0001F3FC", u8"metal_tone2", u8"sign of the horns: medium-light skin tone"},
{u8"sign of the horns", u8"\U0001F918\U0001F3FD", u8"metal_tone3", u8"sign of the horns: medium skin tone"},
{u8"sign of the horns", u8"\U0001F918\U0001F3FE", u8"metal_tone4", u8"sign of the horns: medium-dark skin tone"},
{u8"sign of the horns", u8"\U0001F918\U0001F3FF", u8"metal_tone5", u8"sign of the horns: dark skin tone"},
{u8"call me hand", u8"\U0001F919\U0001F3FB", u8"call_me_tone1", u8"call me hand: light skin tone"},
{u8"call me hand", u8"\U0001F919\U0001F3FC", u8"call_me_tone2", u8"call me hand: medium-light skin tone"},
{u8"call me hand", u8"\U0001F919\U0001F3FD", u8"call_me_tone3", u8"call me hand: medium skin tone"},
{u8"call me hand", u8"\U0001F919\U0001F3FE", u8"call_me_tone4", u8"call me hand: medium-dark skin tone"},
{u8"call me hand", u8"\U0001F919\U0001F3FF", u8"call_me_tone5", u8"call me hand: dark skin tone"},
{u8"backhand index pointing left", u8"\U0001F448\U0001F3FB", u8"point_left_tone1", u8"backhand index pointing left: light skin tone"},
{u8"backhand index pointing left", u8"\U0001F448\U0001F3FC", u8"point_left_tone2", u8"backhand index pointing left: medium-light skin tone"},
{u8"backhand index pointing left", u8"\U0001F448\U0001F3FD", u8"point_left_tone3", u8"backhand index pointing left: medium skin tone"},
{u8"backhand index pointing left", u8"\U0001F448\U0001F3FE", u8"point_left_tone4", u8"backhand index pointing left: medium-dark skin tone"},
{u8"backhand index pointing left", u8"\U0001F448\U0001F3FF", u8"point_left_tone5", u8"backhand index pointing left: dark skin tone"},
{u8"backhand index pointing right", u8"\U0001F449\U0001F3FB", u8"point_right_tone1", u8"backhand index pointing right: light skin tone"},
{u8"backhand index pointing right", u8"\U0001F449\U0001F3FC", u8"point_right_tone2", u8"backhand index pointing right: medium-light skin tone"},
{u8"backhand index pointing right", u8"\U0001F449\U0001F3FD", u8"point_right_tone3", u8"backhand index pointing right: medium skin tone"},
{u8"backhand index pointing right", u8"\U0001F449\U0001F3FE", u8"point_right_tone4", u8"backhand index pointing right: medium-dark skin tone"},
{u8"backhand index pointing right", u8"\U0001F449\U0001F3FF", u8"point_right_tone5", u8"backhand index pointing right: dark skin tone"},
{u8"backhand index pointing up", u8"\U0001F446\U0001F3FB", u8"point_up_2_tone1", u8"backhand index pointing up: light skin tone"},
{u8"backhand index pointing up", u8"\U0001F446\U0001F3FC", u8"point_up_2_tone2", u8"backhand index pointing up: medium-light skin tone"},
{u8"backhand index pointing up", u8"\U0001F446\U0001F3FD", u8"point_up_2_tone3", u8"backhand index pointing up: medium skin tone"},
{u8"backhand index pointing up", u8"\U0001F446\U0001F3FE", u8"point_up_2_tone4", u8"backhand index pointing up: medium-dark skin tone"},
{u8"backhand index pointing up", u8"\U0001F446\U0001F3FF", u8"point_up_2_tone5", u8"backhand index pointing up: dark skin tone"},
{u8"middle finger", u8"\U0001F595\U0001F3FB", u8"middle_finger_tone1", u8"middle finger: light skin tone"},
{u8"middle finger", u8"\U0001F595\U0001F3FC", u8"middle_finger_tone2", u8"middle finger: medium-light skin tone"},
{u8"middle finger", u8"\U0001F595\U0001F3FD", u8"middle_finger_tone3", u8"middle finger: medium skin tone"},
{u8"middle finger", u8"\U0001F595\U0001F3FE", u8"middle_finger_tone4", u8"middle finger: medium-dark skin tone"},
{u8"middle finger", u8"\U0001F595\U0001F3FF", u8"middle_finger_tone5", u8"middle finger: dark skin tone"},
{u8"backhand index pointing down", u8"\U0001F447\U0001F3FB", u8"point_down_tone1", u8"backhand index pointing down: light skin tone"},
{u8"backhand index pointing down", u8"\U0001F447\U0001F3FC", u8"point_down_tone2", u8"backhand index pointing down: medium-light skin tone"},
{u8"backhand index pointing down", u8"\U0001F447\U0001F3FD", u8"point_down_tone3", u8"backhand index pointing down: medium skin tone"},
{u8"backhand index pointing down", u8"\U0001F447\U0001F3FE", u8"point_down_tone4", u8"backhand index pointing down: medium-dark skin tone"},
{u8"backhand index pointing down", u8"\U0001F447\U0001F3FF", u8"point_down_tone5", u8"backhand index pointing down: dark skin tone"},
{u8"index pointing up", u8"\U0000261D\U0001F3FB", u8"point_up_tone1", u8"index pointing up: light skin tone"},
{u8"index pointing up", u8"\U0000261D\U0001F3FC", u8"point_up_tone2", u8"index pointing up: medium-light skin tone"},
{u8"index pointing up", u8"\U0000261D\U0001F3FD", u8"point_up_tone3", u8"index pointing up: medium skin tone"},
{u8"index pointing up", u8"\U0000261D\U0001F3FE", u8"point_up_tone4", u8"index pointing up: medium-dark skin tone"},
{u8"index pointing up", u8"\U0000261D\U0001F3FF", u8"point_up_tone5", u8"index pointing up: dark skin tone"},
{u8"index pointing at the viewer", u8"\U0001FAF5\U0001F3FB", u8"index pointing at the viewer: light skin tone", u8"index pointing at the viewer: light skin tone"},
{u8"index pointing at the viewer", u8"\U0001FAF5\U0001F3FC", u8"index pointing at the viewer: medium-light skin tone", u8"index pointing at the viewer: medium-light skin tone"},
{u8"index pointing at the viewer", u8"\U0001FAF5\U0001F3FD", u8"index pointing at the viewer: medium skin tone", u8"index pointing at the viewer: medium skin tone"},
{u8"index pointing at the viewer", u8"\U0001FAF5\U0001F3FE", u8"index pointing at the viewer: medium-dark skin tone", u8"index pointing at the viewer: medium-dark skin tone"},
{u8"index pointing at the viewer", u8"\U0001FAF5\U0001F3FF", u8"index pointing at the viewer: dark skin tone", u8"index pointing at the viewer: dark skin tone"},
{u8"thumbs up", u8"\U0001F44D\U0001F3FB", u8"thumbsup_tone1", u8"thumbs up: light skin tone"},
{u8"thumbs up", u8"\U0001F44D\U0001F3FC", u8"thumbsup_tone2", u8"thumbs up: medium-light skin tone"},
{u8"thumbs up", u8"\U0001F44D\U0001F3FD", u8"thumbsup_tone3", u8"thumbs up: medium skin tone"},
{u8"thumbs up", u8"\U0001F44D\U0001F3FE", u8"thumbsup_tone4", u8"thumbs up: medium-dark skin tone"},
{u8"thumbs up", u8"\U0001F44D\U0001F3FF", u8"thumbsup_tone5", u8"thumbs up: dark skin tone"},
{u8"thumbs down", u8"\U0001F44E\U0001F3FB", u8"thumbsdown_tone1", u8"thumbs down: light skin tone"},
{u8"thumbs down", u8"\U0001F44E\U0001F3FC", u8"thumbsdown_tone2", u8"thumbs down: medium-light skin tone"},
{u8"thumbs down", u8"\U0001F44E\U0001F3FD", u8"thumbsdown_tone3", u8"thumbs down: medium skin tone"},
{u8"thumbs down", u8"\U0001F44E\U0001F3FE", u8"thumbsdown_tone4", u8"thumbs down: medium-dark skin tone"},
{u8"thumbs down", u8"\U0001F44E\U0001F3FF", u8"thumbsdown_tone5", u8"thumbs down: dark skin tone"},
{u8"raised fist", u8"\U0000270A\U0001F3FB", u8"fist_tone1", u8"raised fist: light skin tone"},
{u8"raised fist", u8"\U0000270A\U0001F3FC", u8"fist_tone2", u8"raised fist: medium-light skin tone"},
{u8"raised fist", u8"\U0000270A\U0001F3FD", u8"fist_tone3", u8"raised fist: medium skin tone"},
{u8"raised fist", u8"\U0000270A\U0001F3FE", u8"fist_tone4", u8"raised fist: medium-dark skin tone"},
{u8"raised fist", u8"\U0000270A\U0001F3FF", u8"fist_tone5", u8"raised fist: dark skin tone"},
{u8"oncoming fist", u8"\U0001F44A\U0001F3FB", u8"punch_tone1", u8"oncoming fist: light skin tone"},
{u8"oncoming fist", u8"\U0001F44A\U0001F3FC", u8"punch_tone2", u8"oncoming fist: medium-light skin tone"},
{u8"oncoming fist", u8"\U0001F44A\U0001F3FD", u8"punch_tone3", u8"oncoming fist: medium skin tone"},
{u8"oncoming fist", u8"\U0001F44A\U0001F3FE", u8"punch_tone4", u8"oncoming fist: medium-dark skin tone"},
{u8"oncoming fist", u8"\U0001F44A\U0001F3FF", u8"punch_tone5", u8"oncoming fist: dark skin tone"},
{u8"left-facing fist", u8"\U0001F91B\U0001F3FB", u8"left_facing_fist_tone1", u8"left-facing fist: light skin tone"},
{u8"left-facing fist", u8"\U0001F91B\U0001F3FC", u8"left_facing_fist_tone2", u8"left-facing fist: medium-light skin tone"},
{u8"left-facing fist", u8"\U0001F91B\U0001F3FD", u8"left_facing_fist_tone3", u8"left-facing fist: medium skin tone"},
{u8"left-facing fist", u8"\U0001F91B\U0001F3FE", u8"left_facing_fist_tone4", u8"left-facing fist: medium-dark skin tone"},
{u8"left-facing fist", u8"\U0001F91B\U0001F3FF", u8"left_facing_fist_tone5", u8"left-facing fist: dark skin tone"},
{u8"right-facing fist", u8"\U0001F91C\U0001F3FB", u8"right_facing_fist_tone1", u8"right-facing fist: light skin tone"},
{u8"right-facing fist", u8"\U0001F91C\U0001F3FC", u8"right_facing_fist_tone2", u8"right-facing fist: medium-light skin tone"},
{u8"right-facing fist", u8"\U0001F91C\U0001F3FD", u8"right_facing_fist_tone3", u8"right-facing fist: medium skin tone"},
{u8"right-facing fist", u8"\U0001F91C\U0001F3FE", u8"right_facing_fist_tone4", u8"right-facing fist: medium-dark skin tone"},
{u8"right-facing fist", u8"\U0001F91C\U0001F3FF", u8"right_facing_fist_tone5", u8"right-facing fist: dark skin tone"},
{u8"clapping hands", u8"\U0001F44F\U0001F3FB", u8"clap_tone1", u8"clapping hands: light skin tone"},
{u8"clapping hands", u8"\U0001F44F\U0001F3FC", u8"clap_tone2", u8"clapping hands: medium-light skin tone"},
{u8"clapping hands", u8"\U0001F44F\U0001F3FD", u8"clap_tone3", u8"clapping hands: medium skin tone"},
{u8"clapping hands", u8"\U0001F44F\U0001F3FE", u8"clap_tone4", u8"clapping hands: medium-dark skin tone"},
{u8"clapping hands", u8"\U0001F44F\U0001F3FF", u8"clap_tone5", u8"clapping hands: dark skin tone"},
{u8"raising hands", u8"\U0001F64C\U0001F3FB", u8"raised_hands_tone1", u8"raising hands: light skin tone"},
{u8"raising hands", u8"\U0001F64C\U0001F3FC", u8"raised_hands_tone2", u8"raising hands: medium-light skin tone"},
{u8"raising hands", u8"\U0001F64C\U0001F3FD", u8"raised_hands_tone3", u8"raising hands: medium skin tone"},
{u8"raising hands", u8"\U0001F64C\U0001F3FE", u8"raised_hands_tone4", u8"raising hands: medium-dark skin tone"},
{u8"raising hands", u8"\U0001F64C\U0001F3FF", u8"raised_hands_tone5", u8"raising hands: dark skin tone"},
{u8"heart hands", u8"\U0001FAF6\U0001F3FB", u8"heart hands: light skin tone", u8"heart hands: light skin tone"},
{u8"heart hands", u8"\U0001FAF6\U0001F3FC", u8"heart hands: medium-light skin tone", u8"heart hands: medium-light skin tone"},
{u8"heart hands", u8"\U0001FAF6\U0001F3FD", u8"heart hands: medium skin tone", u8"heart hands: medium skin tone"},
{u8"heart hands", u8"\U0001FAF6\U0001F3FE", u8"heart hands: medium-dark skin tone", u8"heart hands: medium-dark skin tone"},
{u8"heart hands", u8"\U0001FAF6\U0001F3FF", u8"heart hands: dark skin tone", u8"heart hands: dark skin tone"},
{u8"open hands", u8"\U0001F450\U0001F3FB", u8"open_hands_tone1", u8"open hands: light skin tone"},
{u8"open hands", u8"\U0001F450\U0001F3FC", u8"open_hands_tone2", u8"open hands: medium-light skin tone"},
{u8"open hands", u8"\U0001F450\U0001F3FD", u8"open_hands_tone3", u8"open hands: medium skin tone"},
{u8"open hands", u8"\U0001F450\U0001F3FE", u8"open_hands_tone4", u8"open hands: medium-dark skin tone"},
{u8"open hands", u8"\U0001F450\U0001F3FF", u8"open_hands_tone5", u8"open hands: dark skin tone"},
{u8"palms up together", u8"\U0001F932\U0001F3FB", u8"palms_up_together_tone1", u8"palms up together: light skin tone"},
{u8"palms up together", u8"\U0001F932\U0001F3FC", u8"palms_up_together_tone2", u8"palms up together: medium-light skin tone"},
{u8"palms up together", u8"\U0001F932\U0001F3FD", u8"palms_up_together_tone3", u8"palms up together: medium skin tone"},
{u8"palms up together", u8"\U0001F932\U0001F3FE", u8"palms_up_together_tone4", u8"palms up together: medium-dark skin tone"},
{u8"palms up together", u8"\U0001F932\U0001F3FF", u8"palms_up_together_tone5", u8"palms up together: dark skin tone"},
{u8"handshake", u8"\U0001F91D\U0001F3FB", u8"handshake: light skin tone", u8"handshake: light skin tone"},
{u8"handshake", u8"\U0001F91D\U0001F3FC", u8"handshake: medium-light skin tone", u8"handshake: medium-light skin tone"},
{u8"handshake", u8"\U0001F91D\U0001F3FD", u8"handshake: medium skin tone", u8"handshake: medium skin tone"},
{u8"handshake", u8"\U0001F91D\U0001F3FE", u8"handshake: medium-dark skin tone", u8"handshake: medium-dark skin tone"},
{u8"handshake", u8"\U0001F91D\U0001F3FF", u8"handshake: dark skin tone", u8"handshake: dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FC", u8"handshake: light skin tone, medium-light skin tone", u8"handshake: light skin tone, medium-light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FD", u8"handshake: light skin tone, medium skin tone", u8"handshake: light skin tone, medium skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FE", u8"handshake: light skin tone, medium-dark skin tone", u8"handshake: light skin tone, medium-dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FF", u8"handshake: light skin tone, dark skin tone", u8"handshake: light skin tone, dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FB", u8"handshake: medium-light skin tone, light skin tone", u8"handshake: medium-light skin tone, light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FD", u8"handshake: medium-light skin tone, medium skin tone", u8"handshake: medium-light skin tone, medium skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FE", u8"handshake: medium-light skin tone, medium-dark skin tone", u8"handshake: medium-light skin tone, medium-dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FF", u8"handshake: medium-light skin tone, dark skin tone", u8"handshake: medium-light skin tone, dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FB", u8"handshake: medium skin tone, light skin tone", u8"handshake: medium skin tone, light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FC", u8"handshake: medium skin tone, medium-light skin tone", u8"handshake: medium skin tone, medium-light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FE", u8"handshake: medium skin tone, medium-dark skin tone", u8"handshake: medium skin tone, medium-dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FF", u8"handshake: medium skin tone, dark skin tone", u8"handshake: medium skin tone, dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FB", u8"handshake: medium-dark skin tone, light skin tone", u8"handshake: medium-dark skin tone, light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FC", u8"handshake: medium-dark skin tone, medium-light skin tone", u8"handshake: medium-dark skin tone, medium-light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FD", u8"handshake: medium-dark skin tone, medium skin tone", u8"handshake: medium-dark skin tone, medium skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FF", u8"handshake: medium-dark skin tone, dark skin tone", u8"handshake: medium-dark skin tone, dark skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FB", u8"handshake: dark skin tone, light skin tone", u8"handshake: dark skin tone, light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FC", u8"handshake: dark skin tone, medium-light skin tone", u8"handshake: dark skin tone, medium-light skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FD", u8"handshake: dark skin tone, medium skin tone", u8"handshake: dark skin tone, medium skin tone"},
{u8"handshake", u8"\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FE", u8"handshake: dark skin tone, medium-dark skin tone", u8"handshake: dark skin tone, medium-dark skin tone"},
{u8"folded hands", u8"\U0001F64F\U0001F3FB", u8"pray_tone1", u8"folded hands: light skin tone"},
{u8"folded hands", u8"\U0001F64F\U0001F3FC", u8"pray_tone2", u8"folded hands: medium-light skin tone"},
{u8"folded hands", u8"\U0001F64F\U0001F3FD", u8"pray_tone3", u8"folded hands: medium skin tone"},
{u8"folded hands", u8"\U0001F64F\U0001F3FE", u8"pray_tone4", u8"folded hands: medium-dark skin tone"},
{u8"folded hands", u8"\U0001F64F\U0001F3FF", u8"pray_tone5", u8"folded hands: dark skin tone"},
{u8"writing hand", u8"\U0000270D\U0001F3FB", u8"writing_hand_tone1", u8"writing hand: light skin tone"},
{u8"writing hand", u8"\U0000270D\U0001F3FC", u8"writing_hand_tone2", u8"writing hand: medium-light skin tone"},
{u8"writing hand", u8"\U0000270D\U0001F3FD", u8"writing_hand_tone3", u8"writing hand: medium skin tone"},
{u8"writing hand", u8"\U0000270D\U0001F3FE", u8"writing_hand_tone4", u8"writing hand: medium-dark skin tone"},
{u8"writing hand", u8"\U0000270D\U0001F3FF", u8"writing_hand_tone5", u8"writing hand: dark skin tone"},
{u8"nail polish", u8"\U0001F485\U0001F3FB", u8"nail_care_tone1", u8"nail polish: light skin tone"},
{u8"nail polish", u8"\U0001F485\U0001F3FC", u8"nail_care_tone2", u8"nail polish: medium-light skin tone"},
{u8"nail polish", u8"\U0001F485\U0001F3FD", u8"nail_care_tone3", u8"nail polish: medium skin tone"},
{u8"nail polish", u8"\U0001F485\U0001F3FE", u8"nail_care_tone4", u8"nail polish: medium-dark skin tone"},
{u8"nail polish", u8"\U0001F485\U0001F3FF", u8"nail_care_tone5", u8"nail polish: dark skin tone"},
{u8"selfie", u8"\U0001F933\U0001F3FB", u8"selfie_tone1", u8"selfie: light skin tone"},
{u8"selfie", u8"\U0001F933\U0001F3FC", u8"selfie_tone2", u8"selfie: medium-light skin tone"},
{u8"selfie", u8"\U0001F933\U0001F3FD", u8"selfie_tone3", u8"selfie: medium skin tone"},
{u8"selfie", u8"\U0001F933\U0001F3FE", u8"selfie_tone4", u8"selfie: medium-dark skin tone"},
{u8"selfie", u8"\U0001F933\U0001F3FF", u8"selfie_tone5", u8"selfie: dark skin tone"},
{u8"flexed biceps", u8"\U0001F4AA\U0001F3FB", u8"muscle_tone1", u8"flexed biceps: light skin tone"},
{u8"flexed biceps", u8"\U0001F4AA\U0001F3FC", u8"muscle_tone2", u8"flexed biceps: medium-light skin tone"},
{u8"flexed biceps", u8"\U0001F4AA\U0001F3FD", u8"muscle_tone3", u8"flexed biceps: medium skin tone"},
{u8"flexed biceps", u8"\U0001F4AA\U0001F3FE", u8"muscle_tone4", u8"flexed biceps: medium-dark skin tone"},
{u8"flexed biceps", u8"\U0001F4AA\U0001F3FF", u8"muscle_tone5", u8"flexed biceps: dark skin tone"},
{u8"leg", u8"\U0001F9B5\U0001F3FB", u8"leg_tone1", u8"leg: light skin tone"},
{u8"leg", u8"\U0001F9B5\U0001F3FC", u8"leg_tone2", u8"leg: medium-light skin tone"},
{u8"leg", u8"\U0001F9B5\U0001F3FD", u8"leg_tone3", u8"leg: medium skin tone"},
{u8"leg", u8"\U0001F9B5\U0001F3FE", u8"leg_tone4", u8"leg: medium-dark skin tone"},
{u8"leg", u8"\U0001F9B5\U0001F3FF", u8"leg_tone5", u8"leg: dark skin tone"},
{u8"foot", u8"\U0001F9B6\U0001F3FB", u8"foot_tone1", u8"foot: light skin tone"},
{u8"foot", u8"\U0001F9B6\U0001F3FC", u8"foot_tone2", u8"foot: medium-light skin tone"},
{u8"foot", u8"\U0001F9B6\U0001F3FD", u8"foot_tone3", u8"foot: medium skin tone"},
{u8"foot", u8"\U0001F9B6\U0001F3FE", u8"foot_tone4", u8"foot: medium-dark skin tone"},
{u8"foot", u8"\U0001F9B6\U0001F3FF", u8"foot_tone5", u8"foot: dark skin tone"},
{u8"ear", u8"\U0001F442\U0001F3FB", u8"ear_tone1", u8"ear: light skin tone"},
{u8"ear", u8"\U0001F442\U0001F3FC", u8"ear_tone2", u8"ear: medium-light skin tone"},
{u8"ear", u8"\U0001F442\U0001F3FD", u8"ear_tone3", u8"ear: medium skin tone"},
{u8"ear", u8"\U0001F442\U0001F3FE", u8"ear_tone4", u8"ear: medium-dark skin tone"},
{u8"ear", u8"\U0001F442\U0001F3FF", u8"ear_tone5", u8"ear: dark skin tone"},
{u8"ear with hearing aid", u8"\U0001F9BB\U0001F3FB", u8"ear with hearing aid: light skin tone", u8"ear with hearing aid: light skin tone"},
{u8"ear with hearing aid", u8"\U0001F9BB\U0001F3FC", u8"ear with hearing aid: medium-light skin tone", u8"ear with hearing aid: medium-light skin tone"},
{u8"ear with hearing aid", u8"\U0001F9BB\U0001F3FD", u8"ear with hearing aid: medium skin tone", u8"ear with hearing aid: medium skin tone"},
{u8"ear with hearing aid", u8"\U0001F9BB\U0001F3FE", u8"ear with hearing aid: medium-dark skin tone", u8"ear with hearing aid: medium-dark skin tone"},
{u8"ear with hearing aid", u8"\U0001F9BB\U0001F3FF", u8"ear with hearing aid: dark skin tone", u8"ear with hearing aid: dark skin tone"},
{u8"nose", u8"\U0001F443\U0001F3FB", u8"nose_tone1", u8"nose: light skin tone"},
{u8"nose", u8"\U0001F443\U0001F3FC", u8"nose_tone2", u8"nose: medium-light skin tone"},
{u8"nose", u8"\U0001F443\U0001F3FD", u8"nose_tone3", u8"nose: medium skin tone"},
{u8"nose", u8"\U0001F443\U0001F3FE", u8"nose_tone4", u8"nose: medium-dark skin tone"},
{u8"nose", u8"\U0001F443\U0001F3FF", u8"nose_tone5", u8"nose: dark skin tone"},
{u8"baby", u8"\U0001F476\U0001F3FB", u8"baby_tone1", u8"baby: light skin tone"},
{u8"baby", u8"\U0001F476\U0001F3FC", u8"baby_tone2", u8"baby: medium-light skin tone"},
{u8"baby", u8"\U0001F476\U0001F3FD", u8"baby_tone3", u8"baby: medium skin tone"},
{u8"baby", u8"\U0001F476\U0001F3FE", u8"baby_tone4", u8"baby: medium-dark skin tone"},
{u8"baby", u8"\U0001F476\U0001F3FF", u8"baby_tone5", u8"baby: dark skin tone"},
{u8"child", u8"\U0001F9D2\U0001F3FB", u8"child_tone1", u8"child: light skin tone"},
{u8"child", u8"\U0001F9D2\U0001F3FC", u8"child_tone2", u8"child: medium-light skin tone"},
{u8"child", u8"\U0001F9D2\U0001F3FD", u8"child_tone3", u8"child: medium skin tone"},
{u8"child", u8"\U0001F9D2\U0001F3FE", u8"child_tone4", u8"child: medium-dark skin tone"},
{u8"child", u8"\U0001F9D2\U0001F3FF", u8"child_tone5", u8"child: dark skin tone"},
{u8"boy", u8"\U0001F466\U0001F3FB", u8"boy_tone1", u8"boy: light skin tone"},
{u8"boy", u8"\U0001F466\U0001F3FC", u8"boy_tone2", u8"boy: medium-light skin tone"},
{u8"boy", u8"\U0001F466\U0001F3FD", u8"boy_tone3", u8"boy: medium skin tone"},
{u8"boy", u8"\U0001F466\U0001F3FE", u8"boy_tone4", u8"boy: medium-dark skin tone"},
{u8"boy", u8"\U0001F466\U0001F3FF", u8"boy_tone5", u8"boy: dark skin tone"},
{u8"girl", u8"\U0001F467\U0001F3FB", u8"girl_tone1", u8"girl: light skin tone"},
{u8"girl", u8"\U0001F467\U0001F3FC", u8"girl_tone2", u8"girl: medium-light skin tone"},
{u8"girl", u8"\U0001F467\U0001F3FD", u8"girl_tone3", u8"girl: medium skin tone"},
{u8"girl", u8"\U0001F467\U0001F3FE", u8"girl_tone4", u8"girl: medium-dark skin tone"},
{u8"girl", u8"\U0001F467\U0001F3FF", u8"girl_tone5", u8"girl: dark skin tone"},
{u8"person", u8"\U0001F9D1\U0001F3FB", u8"adult_tone1", u8"person: light skin tone"},
{u8"person", u8"\U0001F9D1\U0001F3FC", u8"adult_tone2", u8"person: medium-light skin tone"},
{u8"person", u8"\U0001F9D1\U0001F3FD", u8"adult_tone3", u8"person: medium skin tone"},
{u8"person", u8"\U0001F9D1\U0001F3FE", u8"adult_tone4", u8"person: medium-dark skin tone"},
{u8"person", u8"\U0001F9D1\U0001F3FF", u8"adult_tone5", u8"person: dark skin tone"},
{u8"person", u8"\U0001F471\U0001F3FB", u8"person_with_blond_hair_tone1", u8"person: light skin tone, blond hair"},
{u8"person", u8"\U0001F471\U0001F3FC", u8"person_with_blond_hair_tone2", u8"person: medium-light skin tone, blond hair"},
{u8"person", u8"\U0001F471\U0001F3FD", u8"person_with_blond_hair_tone3", u8"person: medium skin tone, blond hair"},
{u8"person", u8"\U0001F471\U0001F3FE", u8"person_with_blond_hair_tone4", u8"person: medium-dark skin tone, blond hair"},
{u8"person", u8"\U0001F471\U0001F3FF", u8"person_with_blond_hair_tone5", u8"person: dark skin tone, blond hair"},
{u8"man", u8"\U0001F468\U0001F3FB", u8"man_tone1", u8"man: light skin tone"},
{u8"man", u8"\U0001F468\U0001F3FC", u8"man_tone2", u8"man: medium-light skin tone"},
{u8"man", u8"\U0001F468\U0001F3FD", u8"man_tone3", u8"man: medium skin tone"},
{u8"man", u8"\U0001F468\U0001F3FE", u8"man_tone4", u8"man: medium-dark skin tone"},
{u8"man", u8"\U0001F468\U0001F3FF", u8"man_tone5", u8"man: dark skin tone"},
{u8"person", u8"\U0001F9D4\U0001F3FB", u8"bearded_person_tone1", u8"person: light skin tone, beard"},
{u8"person", u8"\U0001F9D4\U0001F3FC", u8"bearded_person_tone2", u8"person: medium-light skin tone, beard"},
{u8"person", u8"\U0001F9D4\U0001F3FD", u8"bearded_person_tone3", u8"person: medium skin tone, beard"},
{u8"person", u8"\U0001F9D4\U0001F3FE", u8"bearded_person_tone4", u8"person: medium-dark skin tone, beard"},
{u8"person", u8"\U0001F9D4\U0001F3FF", u8"bearded_person_tone5", u8"person: dark skin tone, beard"},
{u8"man", u8"\U0001F9D4\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man: light skin tone, beard", u8"man: light skin tone, beard"},
{u8"man", u8"\U0001F9D4\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man: medium-light skin tone, beard", u8"man: medium-light skin tone, beard"},
{u8"man", u8"\U0001F9D4\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man: medium skin tone, beard", u8"man: medium skin tone, beard"},
{u8"man", u8"\U0001F9D4\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man: medium-dark skin tone, beard", u8"man: medium-dark skin tone, beard"},
{u8"man", u8"\U0001F9D4\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man: dark skin tone, beard", u8"man: dark skin tone, beard"},
{u8"woman", u8"\U0001F9D4\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman: light skin tone, beard", u8"woman: light skin tone, beard"},
{u8"woman", u8"\U0001F9D4\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman: medium-light skin tone, beard", u8"woman: medium-light skin tone, beard"},
{u8"woman", u8"\U0001F9D4\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman: medium skin tone, beard", u8"woman: medium skin tone, beard"},
{u8"woman", u8"\U0001F9D4\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman: medium-dark skin tone, beard", u8"woman: medium-dark skin tone, beard"},
{u8"woman", u8"\U0001F9D4\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman: dark skin tone, beard", u8"woman: dark skin tone, beard"},
{u8"man", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9B0", u8"man: light skin tone, red hair", u8"man: light skin tone, red hair"},
{u8"man", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9B0", u8"man: medium-light skin tone, red hair", u8"man: medium-light skin tone, red hair"},
{u8"man", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9B0", u8"man: medium skin tone, red hair", u8"man: medium skin tone, red hair"},
{u8"man", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9B0", u8"man: medium-dark skin tone, red hair", u8"man: medium-dark skin tone, red hair"},
{u8"man", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9B0", u8"man: dark skin tone, red hair", u8"man: dark skin tone, red hair"},
{u8"man", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9B1", u8"man: light skin tone, curly hair", u8"man: light skin tone, curly hair"},
{u8"man", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9B1", u8"man: medium-light skin tone, curly hair", u8"man: medium-light skin tone, curly hair"},
{u8"man", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9B1", u8"man: medium skin tone, curly hair", u8"man: medium skin tone, curly hair"},
{u8"man", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9B1", u8"man: medium-dark skin tone, curly hair", u8"man: medium-dark skin tone, curly hair"},
{u8"man", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9B1", u8"man: dark skin tone, curly hair", u8"man: dark skin tone, curly hair"},
{u8"man", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9B3", u8"man: light skin tone, white hair", u8"man: light skin tone, white hair"},
{u8"man", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9B3", u8"man: medium-light skin tone, white hair", u8"man: medium-light skin tone, white hair"},
{u8"man", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9B3", u8"man: medium skin tone, white hair", u8"man: medium skin tone, white hair"},
{u8"man", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9B3", u8"man: medium-dark skin tone, white hair", u8"man: medium-dark skin tone, white hair"},
{u8"man", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9B3", u8"man: dark skin tone, white hair", u8"man: dark skin tone, white hair"},
{u8"man", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9B2", u8"man: light skin tone, bald", u8"man: light skin tone, bald"},
{u8"man", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9B2", u8"man: medium-light skin tone, bald", u8"man: medium-light skin tone, bald"},
{u8"man", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9B2", u8"man: medium skin tone, bald", u8"man: medium skin tone, bald"},
{u8"man", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9B2", u8"man: medium-dark skin tone, bald", u8"man: medium-dark skin tone, bald"},
{u8"man", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9B2", u8"man: dark skin tone, bald", u8"man: dark skin tone, bald"},
{u8"woman", u8"\U0001F469\U0001F3FB", u8"woman_tone1", u8"woman: light skin tone"},
{u8"woman", u8"\U0001F469\U0001F3FC", u8"woman_tone2", u8"woman: medium-light skin tone"},
{u8"woman", u8"\U0001F469\U0001F3FD", u8"woman_tone3", u8"woman: medium skin tone"},
{u8"woman", u8"\U0001F469\U0001F3FE", u8"woman_tone4", u8"woman: medium-dark skin tone"},
{u8"woman", u8"\U0001F469\U0001F3FF", u8"woman_tone5", u8"woman: dark skin tone"},
{u8"woman", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9B0", u8"woman: light skin tone, red hair", u8"woman: light skin tone, red hair"},
{u8"woman", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9B0", u8"woman: medium-light skin tone, red hair", u8"woman: medium-light skin tone, red hair"},
{u8"woman", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9B0", u8"woman: medium skin tone, red hair", u8"woman: medium skin tone, red hair"},
{u8"woman", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9B0", u8"woman: medium-dark skin tone, red hair", u8"woman: medium-dark skin tone, red hair"},
{u8"woman", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9B0", u8"woman: dark skin tone, red hair", u8"woman: dark skin tone, red hair"},
{u8"person", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9B0", u8"person: light skin tone, red hair", u8"person: light skin tone, red hair"},
{u8"person", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9B0", u8"person: medium-light skin tone, red hair", u8"person: medium-light skin tone, red hair"},
{u8"person", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9B0", u8"person: medium skin tone, red hair", u8"person: medium skin tone, red hair"},
{u8"person", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9B0", u8"person: medium-dark skin tone, red hair", u8"person: medium-dark skin tone, red hair"},
{u8"person", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9B0", u8"person: dark skin tone, red hair", u8"person: dark skin tone, red hair"},
{u8"woman", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9B1", u8"woman: light skin tone, curly hair", u8"woman: light skin tone, curly hair"},
{u8"woman", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9B1", u8"woman: medium-light skin tone, curly hair", u8"woman: medium-light skin tone, curly hair"},
{u8"woman", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9B1", u8"woman: medium skin tone, curly hair", u8"woman: medium skin tone, curly hair"},
{u8"woman", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9B1", u8"woman: medium-dark skin tone, curly hair", u8"woman: medium-dark skin tone, curly hair"},
{u8"woman", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9B1", u8"woman: dark skin tone, curly hair", u8"woman: dark skin tone, curly hair"},
{u8"person", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9B1", u8"person: light skin tone, curly hair", u8"person: light skin tone, curly hair"},
{u8"person", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9B1", u8"person: medium-light skin tone, curly hair", u8"person: medium-light skin tone, curly hair"},
{u8"person", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9B1", u8"person: medium skin tone, curly hair", u8"person: medium skin tone, curly hair"},
{u8"person", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9B1", u8"person: medium-dark skin tone, curly hair", u8"person: medium-dark skin tone, curly hair"},
{u8"person", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9B1", u8"person: dark skin tone, curly hair", u8"person: dark skin tone, curly hair"},
{u8"woman", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9B3", u8"woman: light skin tone, white hair", u8"woman: light skin tone, white hair"},
{u8"woman", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9B3", u8"woman: medium-light skin tone, white hair", u8"woman: medium-light skin tone, white hair"},
{u8"woman", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9B3", u8"woman: medium skin tone, white hair", u8"woman: medium skin tone, white hair"},
{u8"woman", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9B3", u8"woman: medium-dark skin tone, white hair", u8"woman: medium-dark skin tone, white hair"},
{u8"woman", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9B3", u8"woman: dark skin tone, white hair", u8"woman: dark skin tone, white hair"},
{u8"person", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9B3", u8"person: light skin tone, white hair", u8"person: light skin tone, white hair"},
{u8"person", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9B3", u8"person: medium-light skin tone, white hair", u8"person: medium-light skin tone, white hair"},
{u8"person", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9B3", u8"person: medium skin tone, white hair", u8"person: medium skin tone, white hair"},
{u8"person", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9B3", u8"person: medium-dark skin tone, white hair", u8"person: medium-dark skin tone, white hair"},
{u8"person", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9B3", u8"person: dark skin tone, white hair", u8"person: dark skin tone, white hair"},
{u8"woman", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9B2", u8"woman: light skin tone, bald", u8"woman: light skin tone, bald"},
{u8"woman", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9B2", u8"woman: medium-light skin tone, bald", u8"woman: medium-light skin tone, bald"},
{u8"woman", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9B2", u8"woman: medium skin tone, bald", u8"woman: medium skin tone, bald"},
{u8"woman", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9B2", u8"woman: medium-dark skin tone, bald", u8"woman: medium-dark skin tone, bald"},
{u8"woman", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9B2", u8"woman: dark skin tone, bald", u8"woman: dark skin tone, bald"},
{u8"person", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9B2", u8"person: light skin tone, bald", u8"person: light skin tone, bald"},
{u8"person", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9B2", u8"person: medium-light skin tone, bald", u8"person: medium-light skin tone, bald"},
{u8"person", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9B2", u8"person: medium skin tone, bald", u8"person: medium skin tone, bald"},
{u8"person", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9B2", u8"person: medium-dark skin tone, bald", u8"person: medium-dark skin tone, bald"},
{u8"person", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9B2", u8"person: dark skin tone, bald", u8"person: dark skin tone, bald"},
{u8"woman", u8"\U0001F471\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman: light skin tone, blond hair", u8"woman: light skin tone, blond hair"},
{u8"woman", u8"\U0001F471\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman: medium-light skin tone, blond hair", u8"woman: medium-light skin tone, blond hair"},
{u8"woman", u8"\U0001F471\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman: medium skin tone, blond hair", u8"woman: medium skin tone, blond hair"},
{u8"woman", u8"\U0001F471\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman: medium-dark skin tone, blond hair", u8"woman: medium-dark skin tone, blond hair"},
{u8"woman", u8"\U0001F471\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman: dark skin tone, blond hair", u8"woman: dark skin tone, blond hair"},
{u8"man", u8"\U0001F471\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man: light skin tone, blond hair", u8"man: light skin tone, blond hair"},
{u8"man", u8"\U0001F471\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man: medium-light skin tone, blond hair", u8"man: medium-light skin tone, blond hair"},
{u8"man", u8"\U0001F471\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man: medium skin tone, blond hair", u8"man: medium skin tone, blond hair"},
{u8"man", u8"\U0001F471\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man: medium-dark skin tone, blond hair", u8"man: medium-dark skin tone, blond hair"},
{u8"man", u8"\U0001F471\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man: dark skin tone, blond hair", u8"man: dark skin tone, blond hair"},
{u8"older person", u8"\U0001F9D3\U0001F3FB", u8"older_adult_tone1", u8"older person: light skin tone"},
{u8"older person", u8"\U0001F9D3\U0001F3FC", u8"older_adult_tone2", u8"older person: medium-light skin tone"},
{u8"older person", u8"\U0001F9D3\U0001F3FD", u8"older_adult_tone3", u8"older person: medium skin tone"},
{u8"older person", u8"\U0001F9D3\U0001F3FE", u8"older_adult_tone4", u8"older person: medium-dark skin tone"},
{u8"older person", u8"\U0001F9D3\U0001F3FF", u8"older_adult_tone5", u8"older person: dark skin tone"},
{u8"old man", u8"\U0001F474\U0001F3FB", u8"older_man_tone1", u8"old man: light skin tone"},
{u8"old man", u8"\U0001F474\U0001F3FC", u8"older_man_tone2", u8"old man: medium-light skin tone"},
{u8"old man", u8"\U0001F474\U0001F3FD", u8"older_man_tone3", u8"old man: medium skin tone"},
{u8"old man", u8"\U0001F474\U0001F3FE", u8"older_man_tone4", u8"old man: medium-dark skin tone"},
{u8"old man", u8"\U0001F474\U0001F3FF", u8"older_man_tone5", u8"old man: dark skin tone"},
{u8"old woman", u8"\U0001F475\U0001F3FB", u8"older_woman_tone1", u8"old woman: light skin tone"},
{u8"old woman", u8"\U0001F475\U0001F3FC", u8"older_woman_tone2", u8"old woman: medium-light skin tone"},
{u8"old woman", u8"\U0001F475\U0001F3FD", u8"older_woman_tone3", u8"old woman: medium skin tone"},
{u8"old woman", u8"\U0001F475\U0001F3FE", u8"older_woman_tone4", u8"old woman: medium-dark skin tone"},
{u8"old woman", u8"\U0001F475\U0001F3FF", u8"older_woman_tone5", u8"old woman: dark skin tone"},
{u8"person frowning", u8"\U0001F64D\U0001F3FB", u8"person_frowning_tone1", u8"person frowning: light skin tone"},
{u8"person frowning", u8"\U0001F64D\U0001F3FC", u8"person_frowning_tone2", u8"person frowning: medium-light skin tone"},
{u8"person frowning", u8"\U0001F64D\U0001F3FD", u8"person_frowning_tone3", u8"person frowning: medium skin tone"},
{u8"person frowning", u8"\U0001F64D\U0001F3FE", u8"person_frowning_tone4", u8"person frowning: medium-dark skin tone"},
{u8"person frowning", u8"\U0001F64D\U0001F3FF", u8"person_frowning_tone5", u8"person frowning: dark skin tone"},
{u8"man frowning", u8"\U0001F64D\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man frowning: light skin tone", u8"man frowning: light skin tone"},
{u8"man frowning", u8"\U0001F64D\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man frowning: medium-light skin tone", u8"man frowning: medium-light skin tone"},
{u8"man frowning", u8"\U0001F64D\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man frowning: medium skin tone", u8"man frowning: medium skin tone"},
{u8"man frowning", u8"\U0001F64D\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man frowning: medium-dark skin tone", u8"man frowning: medium-dark skin tone"},
{u8"man frowning", u8"\U0001F64D\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man frowning: dark skin tone", u8"man frowning: dark skin tone"},
{u8"woman frowning", u8"\U0001F64D\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman frowning: light skin tone", u8"woman frowning: light skin tone"},
{u8"woman frowning", u8"\U0001F64D\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman frowning: medium-light skin tone", u8"woman frowning: medium-light skin tone"},
{u8"woman frowning", u8"\U0001F64D\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman frowning: medium skin tone", u8"woman frowning: medium skin tone"},
{u8"woman frowning", u8"\U0001F64D\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman frowning: medium-dark skin tone", u8"woman frowning: medium-dark skin tone"},
{u8"woman frowning", u8"\U0001F64D\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman frowning: dark skin tone", u8"woman frowning: dark skin tone"},
{u8"person pouting", u8"\U0001F64E\U0001F3FB", u8"person_with_pouting_face_tone1", u8"person pouting: light skin tone"},
{u8"person pouting", u8"\U0001F64E\U0001F3FC", u8"person_with_pouting_face_tone2", u8"person pouting: medium-light skin tone"},
{u8"person pouting", u8"\U0001F64E\U0001F3FD", u8"person_with_pouting_face_tone3", u8"person pouting: medium skin tone"},
{u8"person pouting", u8"\U0001F64E\U0001F3FE", u8"person_with_pouting_face_tone4", u8"person pouting: medium-dark skin tone"},
{u8"person pouting", u8"\U0001F64E\U0001F3FF", u8"person_with_pouting_face_tone5", u8"person pouting: dark skin tone"},
{u8"man pouting", u8"\U0001F64E\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man pouting: light skin tone", u8"man pouting: light skin tone"},
{u8"man pouting", u8"\U0001F64E\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man pouting: medium-light skin tone", u8"man pouting: medium-light skin tone"},
{u8"man pouting", u8"\U0001F64E\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man pouting: medium skin tone", u8"man pouting: medium skin tone"},
{u8"man pouting", u8"\U0001F64E\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man pouting: medium-dark skin tone", u8"man pouting: medium-dark skin tone"},
{u8"man pouting", u8"\U0001F64E\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man pouting: dark skin tone", u8"man pouting: dark skin tone"},
{u8"woman pouting", u8"\U0001F64E\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman pouting: light skin tone", u8"woman pouting: light skin tone"},
{u8"woman pouting", u8"\U0001F64E\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman pouting: medium-light skin tone", u8"woman pouting: medium-light skin tone"},
{u8"woman pouting", u8"\U0001F64E\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman pouting: medium skin tone", u8"woman pouting: medium skin tone"},
{u8"woman pouting", u8"\U0001F64E\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman pouting: medium-dark skin tone", u8"woman pouting: medium-dark skin tone"},
{u8"woman pouting", u8"\U0001F64E\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman pouting: dark skin tone", u8"woman pouting: dark skin tone"},
{u8"person gesturing NO", u8"\U0001F645\U0001F3FB", u8"no_good_tone1", u8"person gesturing NO: light skin tone"},
{u8"person gesturing NO", u8"\U0001F645\U0001F3FC", u8"no_good_tone2", u8"person gesturing NO: medium-light skin tone"},
{u8"person gesturing NO", u8"\U0001F645\U0001F3FD", u8"no_good_tone3", u8"person gesturing NO: medium skin tone"},
{u8"person gesturing NO", u8"\U0001F645\U0001F3FE", u8"no_good_tone4", u8"person gesturing NO: medium-dark skin tone"},
{u8"person gesturing NO", u8"\U0001F645\U0001F3FF", u8"no_good_tone5", u8"person gesturing NO: dark skin tone"},
{u8"man gesturing NO", u8"\U0001F645\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man gesturing NO: light skin tone", u8"man gesturing NO: light skin tone"},
{u8"man gesturing NO", u8"\U0001F645\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man gesturing NO: medium-light skin tone", u8"man gesturing NO: medium-light skin tone"},
{u8"man gesturing NO", u8"\U0001F645\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man gesturing NO: medium skin tone", u8"man gesturing NO: medium skin tone"},
{u8"man gesturing NO", u8"\U0001F645\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man gesturing NO: medium-dark skin tone", u8"man gesturing NO: medium-dark skin tone"},
{u8"man gesturing NO", u8"\U0001F645\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man gesturing NO: dark skin tone", u8"man gesturing NO: dark skin tone"},
{u8"woman gesturing NO", u8"\U0001F645\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman gesturing NO: light skin tone", u8"woman gesturing NO: light skin tone"},
{u8"woman gesturing NO", u8"\U0001F645\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman gesturing NO: medium-light skin tone", u8"woman gesturing NO: medium-light skin tone"},
{u8"woman gesturing NO", u8"\U0001F645\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman gesturing NO: medium skin tone", u8"woman gesturing NO: medium skin tone"},
{u8"woman gesturing NO", u8"\U0001F645\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman gesturing NO: medium-dark skin tone", u8"woman gesturing NO: medium-dark skin tone"},
{u8"woman gesturing NO", u8"\U0001F645\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman gesturing NO: dark skin tone", u8"woman gesturing NO: dark skin tone"},
{u8"person gesturing OK", u8"\U0001F646\U0001F3FB", u8"ok_woman_tone1", u8"person gesturing OK: light skin tone"},
{u8"person gesturing OK", u8"\U0001F646\U0001F3FC", u8"ok_woman_tone2", u8"person gesturing OK: medium-light skin tone"},
{u8"person gesturing OK", u8"\U0001F646\U0001F3FD", u8"ok_woman_tone3", u8"person gesturing OK: medium skin tone"},
{u8"person gesturing OK", u8"\U0001F646\U0001F3FE", u8"ok_woman_tone4", u8"person gesturing OK: medium-dark skin tone"},
{u8"person gesturing OK", u8"\U0001F646\U0001F3FF", u8"ok_woman_tone5", u8"person gesturing OK: dark skin tone"},
{u8"man gesturing OK", u8"\U0001F646\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man gesturing OK: light skin tone", u8"man gesturing OK: light skin tone"},
{u8"man gesturing OK", u8"\U0001F646\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man gesturing OK: medium-light skin tone", u8"man gesturing OK: medium-light skin tone"},
{u8"man gesturing OK", u8"\U0001F646\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man gesturing OK: medium skin tone", u8"man gesturing OK: medium skin tone"},
{u8"man gesturing OK", u8"\U0001F646\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man gesturing OK: medium-dark skin tone", u8"man gesturing OK: medium-dark skin tone"},
{u8"man gesturing OK", u8"\U0001F646\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man gesturing OK: dark skin tone", u8"man gesturing OK: dark skin tone"},
{u8"woman gesturing OK", u8"\U0001F646\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman gesturing OK: light skin tone", u8"woman gesturing OK: light skin tone"},
{u8"woman gesturing OK", u8"\U0001F646\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman gesturing OK: medium-light skin tone", u8"woman gesturing OK: medium-light skin tone"},
{u8"woman gesturing OK", u8"\U0001F646\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman gesturing OK: medium skin tone", u8"woman gesturing OK: medium skin tone"},
{u8"woman gesturing OK", u8"\U0001F646\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman gesturing OK: medium-dark skin tone", u8"woman gesturing OK: medium-dark skin tone"},
{u8"woman gesturing OK", u8"\U0001F646\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman gesturing OK: dark skin tone", u8"woman gesturing OK: dark skin tone"},
{u8"person tipping hand", u8"\U0001F481\U0001F3FB", u8"information_desk_person_tone1", u8"person tipping hand: light skin tone"},
{u8"person tipping hand", u8"\U0001F481\U0001F3FC", u8"information_desk_person_tone2", u8"person tipping hand: medium-light skin tone"},
{u8"person tipping hand", u8"\U0001F481\U0001F3FD", u8"information_desk_person_tone3", u8"person tipping hand: medium skin tone"},
{u8"person tipping hand", u8"\U0001F481\U0001F3FE", u8"information_desk_person_tone4", u8"person tipping hand: medium-dark skin tone"},
{u8"person tipping hand", u8"\U0001F481\U0001F3FF", u8"information_desk_person_tone5", u8"person tipping hand: dark skin tone"},
{u8"man tipping hand", u8"\U0001F481\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man tipping hand: light skin tone", u8"man tipping hand: light skin tone"},
{u8"man tipping hand", u8"\U0001F481\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man tipping hand: medium-light skin tone", u8"man tipping hand: medium-light skin tone"},
{u8"man tipping hand", u8"\U0001F481\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man tipping hand: medium skin tone", u8"man tipping hand: medium skin tone"},
{u8"man tipping hand", u8"\U0001F481\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man tipping hand: medium-dark skin tone", u8"man tipping hand: medium-dark skin tone"},
{u8"man tipping hand", u8"\U0001F481\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man tipping hand: dark skin tone", u8"man tipping hand: dark skin tone"},
{u8"woman tipping hand", u8"\U0001F481\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman tipping hand: light skin tone", u8"woman tipping hand: light skin tone"},
{u8"woman tipping hand", u8"\U0001F481\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman tipping hand: medium-light skin tone", u8"woman tipping hand: medium-light skin tone"},
{u8"woman tipping hand", u8"\U0001F481\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman tipping hand: medium skin tone", u8"woman tipping hand: medium skin tone"},
{u8"woman tipping hand", u8"\U0001F481\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman tipping hand: medium-dark skin tone", u8"woman tipping hand: medium-dark skin tone"},
{u8"woman tipping hand", u8"\U0001F481\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman tipping hand: dark skin tone", u8"woman tipping hand: dark skin tone"},
{u8"person raising hand", u8"\U0001F64B\U0001F3FB", u8"raising_hand_tone1", u8"person raising hand: light skin tone"},
{u8"person raising hand", u8"\U0001F64B\U0001F3FC", u8"raising_hand_tone2", u8"person raising hand: medium-light skin tone"},
{u8"person raising hand", u8"\U0001F64B\U0001F3FD", u8"raising_hand_tone3", u8"person raising hand: medium skin tone"},
{u8"person raising hand", u8"\U0001F64B\U0001F3FE", u8"raising_hand_tone4", u8"person raising hand: medium-dark skin tone"},
{u8"person raising hand", u8"\U0001F64B\U0001F3FF", u8"raising_hand_tone5", u8"person raising hand: dark skin tone"},
{u8"man raising hand", u8"\U0001F64B\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man raising hand: light skin tone", u8"man raising hand: light skin tone"},
{u8"man raising hand", u8"\U0001F64B\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man raising hand: medium-light skin tone", u8"man raising hand: medium-light skin tone"},
{u8"man raising hand", u8"\U0001F64B\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man raising hand: medium skin tone", u8"man raising hand: medium skin tone"},
{u8"man raising hand", u8"\U0001F64B\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man raising hand: medium-dark skin tone", u8"man raising hand: medium-dark skin tone"},
{u8"man raising hand", u8"\U0001F64B\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man raising hand: dark skin tone", u8"man raising hand: dark skin tone"},
{u8"woman raising hand", u8"\U0001F64B\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman raising hand: light skin tone", u8"woman raising hand: light skin tone"},
{u8"woman raising hand", u8"\U0001F64B\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman raising hand: medium-light skin tone", u8"woman raising hand: medium-light skin tone"},
{u8"woman raising hand", u8"\U0001F64B\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman raising hand: medium skin tone", u8"woman raising hand: medium skin tone"},
{u8"woman raising hand", u8"\U0001F64B\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman raising hand: medium-dark skin tone", u8"woman raising hand: medium-dark skin tone"},
{u8"woman raising hand", u8"\U0001F64B\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman raising hand: dark skin tone", u8"woman raising hand: dark skin tone"},
{u8"deaf person", u8"\U0001F9CF\U0001F3FB", u8"deaf person: light skin tone", u8"deaf person: light skin tone"},
{u8"deaf person", u8"\U0001F9CF\U0001F3FC", u8"deaf person: medium-light skin tone", u8"deaf person: medium-light skin tone"},
{u8"deaf person", u8"\U0001F9CF\U0001F3FD", u8"deaf person: medium skin tone", u8"deaf person: medium skin tone"},
{u8"deaf person", u8"\U0001F9CF\U0001F3FE", u8"deaf person: medium-dark skin tone", u8"deaf person: medium-dark skin tone"},
{u8"deaf person", u8"\U0001F9CF\U0001F3FF", u8"deaf person: dark skin tone", u8"deaf person: dark skin tone"},
{u8"deaf man", u8"\U0001F9CF\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"deaf man: light skin tone", u8"deaf man: light skin tone"},
{u8"deaf man", u8"\U0001F9CF\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"deaf man: medium-light skin tone", u8"deaf man: medium-light skin tone"},
{u8"deaf man", u8"\U0001F9CF\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"deaf man: medium skin tone", u8"deaf man: medium skin tone"},
{u8"deaf man", u8"\U0001F9CF\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"deaf man: medium-dark skin tone", u8"deaf man: medium-dark skin tone"},
{u8"deaf man", u8"\U0001F9CF\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"deaf man: dark skin tone", u8"deaf man: dark skin tone"},
{u8"deaf woman", u8"\U0001F9CF\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"deaf woman: light skin tone", u8"deaf woman: light skin tone"},
{u8"deaf woman", u8"\U0001F9CF\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"deaf woman: medium-light skin tone", u8"deaf woman: medium-light skin tone"},
{u8"deaf woman", u8"\U0001F9CF\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"deaf woman: medium skin tone", u8"deaf woman: medium skin tone"},
{u8"deaf woman", u8"\U0001F9CF\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"deaf woman: medium-dark skin tone", u8"deaf woman: medium-dark skin tone"},
{u8"deaf woman", u8"\U0001F9CF\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"deaf woman: dark skin tone", u8"deaf woman: dark skin tone"},
{u8"person bowing", u8"\U0001F647\U0001F3FB", u8"bow_tone1", u8"person bowing: light skin tone"},
{u8"person bowing", u8"\U0001F647\U0001F3FC", u8"bow_tone2", u8"person bowing: medium-light skin tone"},
{u8"person bowing", u8"\U0001F647\U0001F3FD", u8"bow_tone3", u8"person bowing: medium skin tone"},
{u8"person bowing", u8"\U0001F647\U0001F3FE", u8"bow_tone4", u8"person bowing: medium-dark skin tone"},
{u8"person bowing", u8"\U0001F647\U0001F3FF", u8"bow_tone5", u8"person bowing: dark skin tone"},
{u8"man bowing", u8"\U0001F647\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man bowing: light skin tone", u8"man bowing: light skin tone"},
{u8"man bowing", u8"\U0001F647\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man bowing: medium-light skin tone", u8"man bowing: medium-light skin tone"},
{u8"man bowing", u8"\U0001F647\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man bowing: medium skin tone", u8"man bowing: medium skin tone"},
{u8"man bowing", u8"\U0001F647\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man bowing: medium-dark skin tone", u8"man bowing: medium-dark skin tone"},
{u8"man bowing", u8"\U0001F647\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man bowing: dark skin tone", u8"man bowing: dark skin tone"},
{u8"woman bowing", u8"\U0001F647\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman bowing: light skin tone", u8"woman bowing: light skin tone"},
{u8"woman bowing", u8"\U0001F647\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman bowing: medium-light skin tone", u8"woman bowing: medium-light skin tone"},
{u8"woman bowing", u8"\U0001F647\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman bowing: medium skin tone", u8"woman bowing: medium skin tone"},
{u8"woman bowing", u8"\U0001F647\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman bowing: medium-dark skin tone", u8"woman bowing: medium-dark skin tone"},
{u8"woman bowing", u8"\U0001F647\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman bowing: dark skin tone", u8"woman bowing: dark skin tone"},
{u8"person facepalming", u8"\U0001F926\U0001F3FB", u8"face_palm_tone1", u8"person facepalming: light skin tone"},
{u8"person facepalming", u8"\U0001F926\U0001F3FC", u8"face_palm_tone2", u8"person facepalming: medium-light skin tone"},
{u8"person facepalming", u8"\U0001F926\U0001F3FD", u8"face_palm_tone3", u8"person facepalming: medium skin tone"},
{u8"person facepalming", u8"\U0001F926\U0001F3FE", u8"face_palm_tone4", u8"person facepalming: medium-dark skin tone"},
{u8"person facepalming", u8"\U0001F926\U0001F3FF", u8"face_palm_tone5", u8"person facepalming: dark skin tone"},
{u8"man facepalming", u8"\U0001F926\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man facepalming: light skin tone", u8"man facepalming: light skin tone"},
{u8"man facepalming", u8"\U0001F926\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man facepalming: medium-light skin tone", u8"man facepalming: medium-light skin tone"},
{u8"man facepalming", u8"\U0001F926\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man facepalming: medium skin tone", u8"man facepalming: medium skin tone"},
{u8"man facepalming", u8"\U0001F926\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man facepalming: medium-dark skin tone", u8"man facepalming: medium-dark skin tone"},
{u8"man facepalming", u8"\U0001F926\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man facepalming: dark skin tone", u8"man facepalming: dark skin tone"},
{u8"woman facepalming", u8"\U0001F926\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman facepalming: light skin tone", u8"woman facepalming: light skin tone"},
{u8"woman facepalming", u8"\U0001F926\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman facepalming: medium-light skin tone", u8"woman facepalming: medium-light skin tone"},
{u8"woman facepalming", u8"\U0001F926\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman facepalming: medium skin tone", u8"woman facepalming: medium skin tone"},
{u8"woman facepalming", u8"\U0001F926\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman facepalming: medium-dark skin tone", u8"woman facepalming: medium-dark skin tone"},
{u8"woman facepalming", u8"\U0001F926\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman facepalming: dark skin tone", u8"woman facepalming: dark skin tone"},
{u8"person shrugging", u8"\U0001F937\U0001F3FB", u8"shrug_tone1", u8"person shrugging: light skin tone"},
{u8"person shrugging", u8"\U0001F937\U0001F3FC", u8"shrug_tone2", u8"person shrugging: medium-light skin tone"},
{u8"person shrugging", u8"\U0001F937\U0001F3FD", u8"shrug_tone3", u8"person shrugging: medium skin tone"},
{u8"person shrugging", u8"\U0001F937\U0001F3FE", u8"shrug_tone4", u8"person shrugging: medium-dark skin tone"},
{u8"person shrugging", u8"\U0001F937\U0001F3FF", u8"shrug_tone5", u8"person shrugging: dark skin tone"},
{u8"man shrugging", u8"\U0001F937\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man shrugging: light skin tone", u8"man shrugging: light skin tone"},
{u8"man shrugging", u8"\U0001F937\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man shrugging: medium-light skin tone", u8"man shrugging: medium-light skin tone"},
{u8"man shrugging", u8"\U0001F937\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man shrugging: medium skin tone", u8"man shrugging: medium skin tone"},
{u8"man shrugging", u8"\U0001F937\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man shrugging: medium-dark skin tone", u8"man shrugging: medium-dark skin tone"},
{u8"man shrugging", u8"\U0001F937\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man shrugging: dark skin tone", u8"man shrugging: dark skin tone"},
{u8"woman shrugging", u8"\U0001F937\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman shrugging: light skin tone", u8"woman shrugging: light skin tone"},
{u8"woman shrugging", u8"\U0001F937\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman shrugging: medium-light skin tone", u8"woman shrugging: medium-light skin tone"},
{u8"woman shrugging", u8"\U0001F937\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman shrugging: medium skin tone", u8"woman shrugging: medium skin tone"},
{u8"woman shrugging", u8"\U0001F937\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman shrugging: medium-dark skin tone", u8"woman shrugging: medium-dark skin tone"},
{u8"woman shrugging", u8"\U0001F937\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman shrugging: dark skin tone", u8"woman shrugging: dark skin tone"},
{u8"health worker", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002695\U0000FE0F", u8"health worker: light skin tone", u8"health worker: light skin tone"},
{u8"health worker", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002695\U0000FE0F", u8"health worker: medium-light skin tone", u8"health worker: medium-light skin tone"},
{u8"health worker", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002695\U0000FE0F", u8"health worker: medium skin tone", u8"health worker: medium skin tone"},
{u8"health worker", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002695\U0000FE0F", u8"health worker: medium-dark skin tone", u8"health worker: medium-dark skin tone"},
{u8"health worker", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002695\U0000FE0F", u8"health worker: dark skin tone", u8"health worker: dark skin tone"},
{u8"man health worker", u8"\U0001F468\U0001F3FB\U0000200D\U00002695\U0000FE0F", u8"man health worker: light skin tone", u8"man health worker: light skin tone"},
{u8"man health worker", u8"\U0001F468\U0001F3FC\U0000200D\U00002695\U0000FE0F", u8"man health worker: medium-light skin tone", u8"man health worker: medium-light skin tone"},
{u8"man health worker", u8"\U0001F468\U0001F3FD\U0000200D\U00002695\U0000FE0F", u8"man health worker: medium skin tone", u8"man health worker: medium skin tone"},
{u8"man health worker", u8"\U0001F468\U0001F3FE\U0000200D\U00002695\U0000FE0F", u8"man health worker: medium-dark skin tone", u8"man health worker: medium-dark skin tone"},
{u8"man health worker", u8"\U0001F468\U0001F3FF\U0000200D\U00002695\U0000FE0F", u8"man health worker: dark skin tone", u8"man health worker: dark skin tone"},
{u8"woman health worker", u8"\U0001F469\U0001F3FB\U0000200D\U00002695\U0000FE0F", u8"woman health worker: light skin tone", u8"woman health worker: light skin tone"},
{u8"woman health worker", u8"\U0001F469\U0001F3FC\U0000200D\U00002695\U0000FE0F", u8"woman health worker: medium-light skin tone", u8"woman health worker: medium-light skin tone"},
{u8"woman health worker", u8"\U0001F469\U0001F3FD\U0000200D\U00002695\U0000FE0F", u8"woman health worker: medium skin tone", u8"woman health worker: medium skin tone"},
{u8"woman health worker", u8"\U0001F469\U0001F3FE\U0000200D\U00002695\U0000FE0F", u8"woman health worker: medium-dark skin tone", u8"woman health worker: medium-dark skin tone"},
{u8"woman health worker", u8"\U0001F469\U0001F3FF\U0000200D\U00002695\U0000FE0F", u8"woman health worker: dark skin tone", u8"woman health worker: dark skin tone"},
{u8"student", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F393", u8"student: light skin tone", u8"student: light skin tone"},
{u8"student", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F393", u8"student: medium-light skin tone", u8"student: medium-light skin tone"},
{u8"student", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F393", u8"student: medium skin tone", u8"student: medium skin tone"},
{u8"student", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F393", u8"student: medium-dark skin tone", u8"student: medium-dark skin tone"},
{u8"student", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F393", u8"student: dark skin tone", u8"student: dark skin tone"},
{u8"man student", u8"\U0001F468\U0001F3FB\U0000200D\U0001F393", u8"man student: light skin tone", u8"man student: light skin tone"},
{u8"man student", u8"\U0001F468\U0001F3FC\U0000200D\U0001F393", u8"man student: medium-light skin tone", u8"man student: medium-light skin tone"},
{u8"man student", u8"\U0001F468\U0001F3FD\U0000200D\U0001F393", u8"man student: medium skin tone", u8"man student: medium skin tone"},
{u8"man student", u8"\U0001F468\U0001F3FE\U0000200D\U0001F393", u8"man student: medium-dark skin tone", u8"man student: medium-dark skin tone"},
{u8"man student", u8"\U0001F468\U0001F3FF\U0000200D\U0001F393", u8"man student: dark skin tone", u8"man student: dark skin tone"},
{u8"woman student", u8"\U0001F469\U0001F3FB\U0000200D\U0001F393", u8"woman student: light skin tone", u8"woman student: light skin tone"},
{u8"woman student", u8"\U0001F469\U0001F3FC\U0000200D\U0001F393", u8"woman student: medium-light skin tone", u8"woman student: medium-light skin tone"},
{u8"woman student", u8"\U0001F469\U0001F3FD\U0000200D\U0001F393", u8"woman student: medium skin tone", u8"woman student: medium skin tone"},
{u8"woman student", u8"\U0001F469\U0001F3FE\U0000200D\U0001F393", u8"woman student: medium-dark skin tone", u8"woman student: medium-dark skin tone"},
{u8"woman student", u8"\U0001F469\U0001F3FF\U0000200D\U0001F393", u8"woman student: dark skin tone", u8"woman student: dark skin tone"},
{u8"teacher", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F3EB", u8"teacher: light skin tone", u8"teacher: light skin tone"},
{u8"teacher", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F3EB", u8"teacher: medium-light skin tone", u8"teacher: medium-light skin tone"},
{u8"teacher", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F3EB", u8"teacher: medium skin tone", u8"teacher: medium skin tone"},
{u8"teacher", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F3EB", u8"teacher: medium-dark skin tone", u8"teacher: medium-dark skin tone"},
{u8"teacher", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F3EB", u8"teacher: dark skin tone", u8"teacher: dark skin tone"},
{u8"man teacher", u8"\U0001F468\U0001F3FB\U0000200D\U0001F3EB", u8"man teacher: light skin tone", u8"man teacher: light skin tone"},
{u8"man teacher", u8"\U0001F468\U0001F3FC\U0000200D\U0001F3EB", u8"man teacher: medium-light skin tone", u8"man teacher: medium-light skin tone"},
{u8"man teacher", u8"\U0001F468\U0001F3FD\U0000200D\U0001F3EB", u8"man teacher: medium skin tone", u8"man teacher: medium skin tone"},
{u8"man teacher", u8"\U0001F468\U0001F3FE\U0000200D\U0001F3EB", u8"man teacher: medium-dark skin tone", u8"man teacher: medium-dark skin tone"},
{u8"man teacher", u8"\U0001F468\U0001F3FF\U0000200D\U0001F3EB", u8"man teacher: dark skin tone", u8"man teacher: dark skin tone"},
{u8"woman teacher", u8"\U0001F469\U0001F3FB\U0000200D\U0001F3EB", u8"woman teacher: light skin tone", u8"woman teacher: light skin tone"},
{u8"woman teacher", u8"\U0001F469\U0001F3FC\U0000200D\U0001F3EB", u8"woman teacher: medium-light skin tone", u8"woman teacher: medium-light skin tone"},
{u8"woman teacher", u8"\U0001F469\U0001F3FD\U0000200D\U0001F3EB", u8"woman teacher: medium skin tone", u8"woman teacher: medium skin tone"},
{u8"woman teacher", u8"\U0001F469\U0001F3FE\U0000200D\U0001F3EB", u8"woman teacher: medium-dark skin tone", u8"woman teacher: medium-dark skin tone"},
{u8"woman teacher", u8"\U0001F469\U0001F3FF\U0000200D\U0001F3EB", u8"woman teacher: dark skin tone", u8"woman teacher: dark skin tone"},
{u8"judge", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002696\U0000FE0F", u8"judge: light skin tone", u8"judge: light skin tone"},
{u8"judge", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002696\U0000FE0F", u8"judge: medium-light skin tone", u8"judge: medium-light skin tone"},
{u8"judge", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002696\U0000FE0F", u8"judge: medium skin tone", u8"judge: medium skin tone"},
{u8"judge", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002696\U0000FE0F", u8"judge: medium-dark skin tone", u8"judge: medium-dark skin tone"},
{u8"judge", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002696\U0000FE0F", u8"judge: dark skin tone", u8"judge: dark skin tone"},
{u8"man judge", u8"\U0001F468\U0001F3FB\U0000200D\U00002696\U0000FE0F", u8"man judge: light skin tone", u8"man judge: light skin tone"},
{u8"man judge", u8"\U0001F468\U0001F3FC\U0000200D\U00002696\U0000FE0F", u8"man judge: medium-light skin tone", u8"man judge: medium-light skin tone"},
{u8"man judge", u8"\U0001F468\U0001F3FD\U0000200D\U00002696\U0000FE0F", u8"man judge: medium skin tone", u8"man judge: medium skin tone"},
{u8"man judge", u8"\U0001F468\U0001F3FE\U0000200D\U00002696\U0000FE0F", u8"man judge: medium-dark skin tone", u8"man judge: medium-dark skin tone"},
{u8"man judge", u8"\U0001F468\U0001F3FF\U0000200D\U00002696\U0000FE0F", u8"man judge: dark skin tone", u8"man judge: dark skin tone"},
{u8"woman judge", u8"\U0001F469\U0001F3FB\U0000200D\U00002696\U0000FE0F", u8"woman judge: light skin tone", u8"woman judge: light skin tone"},
{u8"woman judge", u8"\U0001F469\U0001F3FC\U0000200D\U00002696\U0000FE0F", u8"woman judge: medium-light skin tone", u8"woman judge: medium-light skin tone"},
{u8"woman judge", u8"\U0001F469\U0001F3FD\U0000200D\U00002696\U0000FE0F", u8"woman judge: medium skin tone", u8"woman judge: medium skin tone"},
{u8"woman judge", u8"\U0001F469\U0001F3FE\U0000200D\U00002696\U0000FE0F", u8"woman judge: medium-dark skin tone", u8"woman judge: medium-dark skin tone"},
{u8"woman judge", u8"\U0001F469\U0001F3FF\U0000200D\U00002696\U0000FE0F", u8"woman judge: dark skin tone", u8"woman judge: dark skin tone"},
{u8"farmer", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F33E", u8"farmer: light skin tone", u8"farmer: light skin tone"},
{u8"farmer", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F33E", u8"farmer: medium-light skin tone", u8"farmer: medium-light skin tone"},
{u8"farmer", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F33E", u8"farmer: medium skin tone", u8"farmer: medium skin tone"},
{u8"farmer", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F33E", u8"farmer: medium-dark skin tone", u8"farmer: medium-dark skin tone"},
{u8"farmer", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F33E", u8"farmer: dark skin tone", u8"farmer: dark skin tone"},
{u8"man farmer", u8"\U0001F468\U0001F3FB\U0000200D\U0001F33E", u8"man farmer: light skin tone", u8"man farmer: light skin tone"},
{u8"man farmer", u8"\U0001F468\U0001F3FC\U0000200D\U0001F33E", u8"man farmer: medium-light skin tone", u8"man farmer: medium-light skin tone"},
{u8"man farmer", u8"\U0001F468\U0001F3FD\U0000200D\U0001F33E", u8"man farmer: medium skin tone", u8"man farmer: medium skin tone"},
{u8"man farmer", u8"\U0001F468\U0001F3FE\U0000200D\U0001F33E", u8"man farmer: medium-dark skin tone", u8"man farmer: medium-dark skin tone"},
{u8"man farmer", u8"\U0001F468\U0001F3FF\U0000200D\U0001F33E", u8"man farmer: dark skin tone", u8"man farmer: dark skin tone"},
{u8"woman farmer", u8"\U0001F469\U0001F3FB\U0000200D\U0001F33E", u8"woman farmer: light skin tone", u8"woman farmer: light skin tone"},
{u8"woman farmer", u8"\U0001F469\U0001F3FC\U0000200D\U0001F33E", u8"woman farmer: medium-light skin tone", u8"woman farmer: medium-light skin tone"},
{u8"woman farmer", u8"\U0001F469\U0001F3FD\U0000200D\U0001F33E", u8"woman farmer: medium skin tone", u8"woman farmer: medium skin tone"},
{u8"woman farmer", u8"\U0001F469\U0001F3FE\U0000200D\U0001F33E", u8"woman farmer: medium-dark skin tone", u8"woman farmer: medium-dark skin tone"},
{u8"woman farmer", u8"\U0001F469\U0001F3FF\U0000200D\U0001F33E", u8"woman farmer: dark skin tone", u8"woman farmer: dark skin tone"},
{u8"cook", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F373", u8"cook: light skin tone", u8"cook: light skin tone"},
{u8"cook", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F373", u8"cook: medium-light skin tone", u8"cook: medium-light skin tone"},
{u8"cook", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F373", u8"cook: medium skin tone", u8"cook: medium skin tone"},
{u8"cook", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F373", u8"cook: medium-dark skin tone", u8"cook: medium-dark skin tone"},
{u8"cook", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F373", u8"cook: dark skin tone", u8"cook: dark skin tone"},
{u8"man cook", u8"\U0001F468\U0001F3FB\U0000200D\U0001F373", u8"man cook: light skin tone", u8"man cook: light skin tone"},
{u8"man cook", u8"\U0001F468\U0001F3FC\U0000200D\U0001F373", u8"man cook: medium-light skin tone", u8"man cook: medium-light skin tone"},
{u8"man cook", u8"\U0001F468\U0001F3FD\U0000200D\U0001F373", u8"man cook: medium skin tone", u8"man cook: medium skin tone"},
{u8"man cook", u8"\U0001F468\U0001F3FE\U0000200D\U0001F373", u8"man cook: medium-dark skin tone", u8"man cook: medium-dark skin tone"},
{u8"man cook", u8"\U0001F468\U0001F3FF\U0000200D\U0001F373", u8"man cook: dark skin tone", u8"man cook: dark skin tone"},
{u8"woman cook", u8"\U0001F469\U0001F3FB\U0000200D\U0001F373", u8"woman cook: light skin tone", u8"woman cook: light skin tone"},
{u8"woman cook", u8"\U0001F469\U0001F3FC\U0000200D\U0001F373", u8"woman cook: medium-light skin tone", u8"woman cook: medium-light skin tone"},
{u8"woman cook", u8"\U0001F469\U0001F3FD\U0000200D\U0001F373", u8"woman cook: medium skin tone", u8"woman cook: medium skin tone"},
{u8"woman cook", u8"\U0001F469\U0001F3FE\U0000200D\U0001F373", u8"woman cook: medium-dark skin tone", u8"woman cook: medium-dark skin tone"},
{u8"woman cook", u8"\U0001F469\U0001F3FF\U0000200D\U0001F373", u8"woman cook: dark skin tone", u8"woman cook: dark skin tone"},
{u8"mechanic", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F527", u8"mechanic: light skin tone", u8"mechanic: light skin tone"},
{u8"mechanic", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F527", u8"mechanic: medium-light skin tone", u8"mechanic: medium-light skin tone"},
{u8"mechanic", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F527", u8"mechanic: medium skin tone", u8"mechanic: medium skin tone"},
{u8"mechanic", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F527", u8"mechanic: medium-dark skin tone", u8"mechanic: medium-dark skin tone"},
{u8"mechanic", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F527", u8"mechanic: dark skin tone", u8"mechanic: dark skin tone"},
{u8"man mechanic", u8"\U0001F468\U0001F3FB\U0000200D\U0001F527", u8"man mechanic: light skin tone", u8"man mechanic: light skin tone"},
{u8"man mechanic", u8"\U0001F468\U0001F3FC\U0000200D\U0001F527", u8"man mechanic: medium-light skin tone", u8"man mechanic: medium-light skin tone"},
{u8"man mechanic", u8"\U0001F468\U0001F3FD\U0000200D\U0001F527", u8"man mechanic: medium skin tone", u8"man mechanic: medium skin tone"},
{u8"man mechanic", u8"\U0001F468\U0001F3FE\U0000200D\U0001F527", u8"man mechanic: medium-dark skin tone", u8"man mechanic: medium-dark skin tone"},
{u8"man mechanic", u8"\U0001F468\U0001F3FF\U0000200D\U0001F527", u8"man mechanic: dark skin tone", u8"man mechanic: dark skin tone"},
{u8"woman mechanic", u8"\U0001F469\U0001F3FB\U0000200D\U0001F527", u8"woman mechanic: light skin tone", u8"woman mechanic: light skin tone"},
{u8"woman mechanic", u8"\U0001F469\U0001F3FC\U0000200D\U0001F527", u8"woman mechanic: medium-light skin tone", u8"woman mechanic: medium-light skin tone"},
{u8"woman mechanic", u8"\U0001F469\U0001F3FD\U0000200D\U0001F527", u8"woman mechanic: medium skin tone", u8"woman mechanic: medium skin tone"},
{u8"woman mechanic", u8"\U0001F469\U0001F3FE\U0000200D\U0001F527", u8"woman mechanic: medium-dark skin tone", u8"woman mechanic: medium-dark skin tone"},
{u8"woman mechanic", u8"\U0001F469\U0001F3FF\U0000200D\U0001F527", u8"woman mechanic: dark skin tone", u8"woman mechanic: dark skin tone"},
{u8"factory worker", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F3ED", u8"factory worker: light skin tone", u8"factory worker: light skin tone"},
{u8"factory worker", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F3ED", u8"factory worker: medium-light skin tone", u8"factory worker: medium-light skin tone"},
{u8"factory worker", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F3ED", u8"factory worker: medium skin tone", u8"factory worker: medium skin tone"},
{u8"factory worker", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F3ED", u8"factory worker: medium-dark skin tone", u8"factory worker: medium-dark skin tone"},
{u8"factory worker", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F3ED", u8"factory worker: dark skin tone", u8"factory worker: dark skin tone"},
{u8"man factory worker", u8"\U0001F468\U0001F3FB\U0000200D\U0001F3ED", u8"man factory worker: light skin tone", u8"man factory worker: light skin tone"},
{u8"man factory worker", u8"\U0001F468\U0001F3FC\U0000200D\U0001F3ED", u8"man factory worker: medium-light skin tone", u8"man factory worker: medium-light skin tone"},
{u8"man factory worker", u8"\U0001F468\U0001F3FD\U0000200D\U0001F3ED", u8"man factory worker: medium skin tone", u8"man factory worker: medium skin tone"},
{u8"man factory worker", u8"\U0001F468\U0001F3FE\U0000200D\U0001F3ED", u8"man factory worker: medium-dark skin tone", u8"man factory worker: medium-dark skin tone"},
{u8"man factory worker", u8"\U0001F468\U0001F3FF\U0000200D\U0001F3ED", u8"man factory worker: dark skin tone", u8"man factory worker: dark skin tone"},
{u8"woman factory worker", u8"\U0001F469\U0001F3FB\U0000200D\U0001F3ED", u8"woman factory worker: light skin tone", u8"woman factory worker: light skin tone"},
{u8"woman factory worker", u8"\U0001F469\U0001F3FC\U0000200D\U0001F3ED", u8"woman factory worker: medium-light skin tone", u8"woman factory worker: medium-light skin tone"},
{u8"woman factory worker", u8"\U0001F469\U0001F3FD\U0000200D\U0001F3ED", u8"woman factory worker: medium skin tone", u8"woman factory worker: medium skin tone"},
{u8"woman factory worker", u8"\U0001F469\U0001F3FE\U0000200D\U0001F3ED", u8"woman factory worker: medium-dark skin tone", u8"woman factory worker: medium-dark skin tone"},
{u8"woman factory worker", u8"\U0001F469\U0001F3FF\U0000200D\U0001F3ED", u8"woman factory worker: dark skin tone", u8"woman factory worker: dark skin tone"},
{u8"office worker", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F4BC", u8"office worker: light skin tone", u8"office worker: light skin tone"},
{u8"office worker", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F4BC", u8"office worker: medium-light skin tone", u8"office worker: medium-light skin tone"},
{u8"office worker", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F4BC", u8"office worker: medium skin tone", u8"office worker: medium skin tone"},
{u8"office worker", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F4BC", u8"office worker: medium-dark skin tone", u8"office worker: medium-dark skin tone"},
{u8"office worker", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F4BC", u8"office worker: dark skin tone", u8"office worker: dark skin tone"},
{u8"man office worker", u8"\U0001F468\U0001F3FB\U0000200D\U0001F4BC", u8"man office worker: light skin tone", u8"man office worker: light skin tone"},
{u8"man office worker", u8"\U0001F468\U0001F3FC\U0000200D\U0001F4BC", u8"man office worker: medium-light skin tone", u8"man office worker: medium-light skin tone"},
{u8"man office worker", u8"\U0001F468\U0001F3FD\U0000200D\U0001F4BC", u8"man office worker: medium skin tone", u8"man office worker: medium skin tone"},
{u8"man office worker", u8"\U0001F468\U0001F3FE\U0000200D\U0001F4BC", u8"man office worker: medium-dark skin tone", u8"man office worker: medium-dark skin tone"},
{u8"man office worker", u8"\U0001F468\U0001F3FF\U0000200D\U0001F4BC", u8"man office worker: dark skin tone", u8"man office worker: dark skin tone"},
{u8"woman office worker", u8"\U0001F469\U0001F3FB\U0000200D\U0001F4BC", u8"woman office worker: light skin tone", u8"woman office worker: light skin tone"},
{u8"woman office worker", u8"\U0001F469\U0001F3FC\U0000200D\U0001F4BC", u8"woman office worker: medium-light skin tone", u8"woman office worker: medium-light skin tone"},
{u8"woman office worker", u8"\U0001F469\U0001F3FD\U0000200D\U0001F4BC", u8"woman office worker: medium skin tone", u8"woman office worker: medium skin tone"},
{u8"woman office worker", u8"\U0001F469\U0001F3FE\U0000200D\U0001F4BC", u8"woman office worker: medium-dark skin tone", u8"woman office worker: medium-dark skin tone"},
{u8"woman office worker", u8"\U0001F469\U0001F3FF\U0000200D\U0001F4BC", u8"woman office worker: dark skin tone", u8"woman office worker: dark skin tone"},
{u8"scientist", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F52C", u8"scientist: light skin tone", u8"scientist: light skin tone"},
{u8"scientist", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F52C", u8"scientist: medium-light skin tone", u8"scientist: medium-light skin tone"},
{u8"scientist", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F52C", u8"scientist: medium skin tone", u8"scientist: medium skin tone"},
{u8"scientist", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F52C", u8"scientist: medium-dark skin tone", u8"scientist: medium-dark skin tone"},
{u8"scientist", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F52C", u8"scientist: dark skin tone", u8"scientist: dark skin tone"},
{u8"man scientist", u8"\U0001F468\U0001F3FB\U0000200D\U0001F52C", u8"man scientist: light skin tone", u8"man scientist: light skin tone"},
{u8"man scientist", u8"\U0001F468\U0001F3FC\U0000200D\U0001F52C", u8"man scientist: medium-light skin tone", u8"man scientist: medium-light skin tone"},
{u8"man scientist", u8"\U0001F468\U0001F3FD\U0000200D\U0001F52C", u8"man scientist: medium skin tone", u8"man scientist: medium skin tone"},
{u8"man scientist", u8"\U0001F468\U0001F3FE\U0000200D\U0001F52C", u8"man scientist: medium-dark skin tone", u8"man scientist: medium-dark skin tone"},
{u8"man scientist", u8"\U0001F468\U0001F3FF\U0000200D\U0001F52C", u8"man scientist: dark skin tone", u8"man scientist: dark skin tone"},
{u8"woman scientist", u8"\U0001F469\U0001F3FB\U0000200D\U0001F52C", u8"woman scientist: light skin tone", u8"woman scientist: light skin tone"},
{u8"woman scientist", u8"\U0001F469\U0001F3FC\U0000200D\U0001F52C", u8"woman scientist: medium-light skin tone", u8"woman scientist: medium-light skin tone"},
{u8"woman scientist", u8"\U0001F469\U0001F3FD\U0000200D\U0001F52C", u8"woman scientist: medium skin tone", u8"woman scientist: medium skin tone"},
{u8"woman scientist", u8"\U0001F469\U0001F3FE\U0000200D\U0001F52C", u8"woman scientist: medium-dark skin tone", u8"woman scientist: medium-dark skin tone"},
{u8"woman scientist", u8"\U0001F469\U0001F3FF\U0000200D\U0001F52C", u8"woman scientist: dark skin tone", u8"woman scientist: dark skin tone"},
{u8"technologist", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F4BB", u8"technologist: light skin tone", u8"technologist: light skin tone"},
{u8"technologist", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F4BB", u8"technologist: medium-light skin tone", u8"technologist: medium-light skin tone"},
{u8"technologist", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F4BB", u8"technologist: medium skin tone", u8"technologist: medium skin tone"},
{u8"technologist", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F4BB", u8"technologist: medium-dark skin tone", u8"technologist: medium-dark skin tone"},
{u8"technologist", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F4BB", u8"technologist: dark skin tone", u8"technologist: dark skin tone"},
{u8"man technologist", u8"\U0001F468\U0001F3FB\U0000200D\U0001F4BB", u8"man technologist: light skin tone", u8"man technologist: light skin tone"},
{u8"man technologist", u8"\U0001F468\U0001F3FC\U0000200D\U0001F4BB", u8"man technologist: medium-light skin tone", u8"man technologist: medium-light skin tone"},
{u8"man technologist", u8"\U0001F468\U0001F3FD\U0000200D\U0001F4BB", u8"man technologist: medium skin tone", u8"man technologist: medium skin tone"},
{u8"man technologist", u8"\U0001F468\U0001F3FE\U0000200D\U0001F4BB", u8"man technologist: medium-dark skin tone", u8"man technologist: medium-dark skin tone"},
{u8"man technologist", u8"\U0001F468\U0001F3FF\U0000200D\U0001F4BB", u8"man technologist: dark skin tone", u8"man technologist: dark skin tone"},
{u8"woman technologist", u8"\U0001F469\U0001F3FB\U0000200D\U0001F4BB", u8"woman technologist: light skin tone", u8"woman technologist: light skin tone"},
{u8"woman technologist", u8"\U0001F469\U0001F3FC\U0000200D\U0001F4BB", u8"woman technologist: medium-light skin tone", u8"woman technologist: medium-light skin tone"},
{u8"woman technologist", u8"\U0001F469\U0001F3FD\U0000200D\U0001F4BB", u8"woman technologist: medium skin tone", u8"woman technologist: medium skin tone"},
{u8"woman technologist", u8"\U0001F469\U0001F3FE\U0000200D\U0001F4BB", u8"woman technologist: medium-dark skin tone", u8"woman technologist: medium-dark skin tone"},
{u8"woman technologist", u8"\U0001F469\U0001F3FF\U0000200D\U0001F4BB", u8"woman technologist: dark skin tone", u8"woman technologist: dark skin tone"},
{u8"singer", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F3A4", u8"singer: light skin tone", u8"singer: light skin tone"},
{u8"singer", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F3A4", u8"singer: medium-light skin tone", u8"singer: medium-light skin tone"},
{u8"singer", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F3A4", u8"singer: medium skin tone", u8"singer: medium skin tone"},
{u8"singer", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F3A4", u8"singer: medium-dark skin tone", u8"singer: medium-dark skin tone"},
{u8"singer", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F3A4", u8"singer: dark skin tone", u8"singer: dark skin tone"},
{u8"man singer", u8"\U0001F468\U0001F3FB\U0000200D\U0001F3A4", u8"man singer: light skin tone", u8"man singer: light skin tone"},
{u8"man singer", u8"\U0001F468\U0001F3FC\U0000200D\U0001F3A4", u8"man singer: medium-light skin tone", u8"man singer: medium-light skin tone"},
{u8"man singer", u8"\U0001F468\U0001F3FD\U0000200D\U0001F3A4", u8"man singer: medium skin tone", u8"man singer: medium skin tone"},
{u8"man singer", u8"\U0001F468\U0001F3FE\U0000200D\U0001F3A4", u8"man singer: medium-dark skin tone", u8"man singer: medium-dark skin tone"},
{u8"man singer", u8"\U0001F468\U0001F3FF\U0000200D\U0001F3A4", u8"man singer: dark skin tone", u8"man singer: dark skin tone"},
{u8"woman singer", u8"\U0001F469\U0001F3FB\U0000200D\U0001F3A4", u8"woman singer: light skin tone", u8"woman singer: light skin tone"},
{u8"woman singer", u8"\U0001F469\U0001F3FC\U0000200D\U0001F3A4", u8"woman singer: medium-light skin tone", u8"woman singer: medium-light skin tone"},
{u8"woman singer", u8"\U0001F469\U0001F3FD\U0000200D\U0001F3A4", u8"woman singer: medium skin tone", u8"woman singer: medium skin tone"},
{u8"woman singer", u8"\U0001F469\U0001F3FE\U0000200D\U0001F3A4", u8"woman singer: medium-dark skin tone", u8"woman singer: medium-dark skin tone"},
{u8"woman singer", u8"\U0001F469\U0001F3FF\U0000200D\U0001F3A4", u8"woman singer: dark skin tone", u8"woman singer: dark skin tone"},
{u8"artist", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F3A8", u8"artist: light skin tone", u8"artist: light skin tone"},
{u8"artist", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F3A8", u8"artist: medium-light skin tone", u8"artist: medium-light skin tone"},
{u8"artist", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F3A8", u8"artist: medium skin tone", u8"artist: medium skin tone"},
{u8"artist", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F3A8", u8"artist: medium-dark skin tone", u8"artist: medium-dark skin tone"},
{u8"artist", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F3A8", u8"artist: dark skin tone", u8"artist: dark skin tone"},
{u8"man artist", u8"\U0001F468\U0001F3FB\U0000200D\U0001F3A8", u8"man artist: light skin tone", u8"man artist: light skin tone"},
{u8"man artist", u8"\U0001F468\U0001F3FC\U0000200D\U0001F3A8", u8"man artist: medium-light skin tone", u8"man artist: medium-light skin tone"},
{u8"man artist", u8"\U0001F468\U0001F3FD\U0000200D\U0001F3A8", u8"man artist: medium skin tone", u8"man artist: medium skin tone"},
{u8"man artist", u8"\U0001F468\U0001F3FE\U0000200D\U0001F3A8", u8"man artist: medium-dark skin tone", u8"man artist: medium-dark skin tone"},
{u8"man artist", u8"\U0001F468\U0001F3FF\U0000200D\U0001F3A8", u8"man artist: dark skin tone", u8"man artist: dark skin tone"},
{u8"woman artist", u8"\U0001F469\U0001F3FB\U0000200D\U0001F3A8", u8"woman artist: light skin tone", u8"woman artist: light skin tone"},
{u8"woman artist", u8"\U0001F469\U0001F3FC\U0000200D\U0001F3A8", u8"woman artist: medium-light skin tone", u8"woman artist: medium-light skin tone"},
{u8"woman artist", u8"\U0001F469\U0001F3FD\U0000200D\U0001F3A8", u8"woman artist: medium skin tone", u8"woman artist: medium skin tone"},
{u8"woman artist", u8"\U0001F469\U0001F3FE\U0000200D\U0001F3A8", u8"woman artist: medium-dark skin tone", u8"woman artist: medium-dark skin tone"},
{u8"woman artist", u8"\U0001F469\U0001F3FF\U0000200D\U0001F3A8", u8"woman artist: dark skin tone", u8"woman artist: dark skin tone"},
{u8"pilot", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002708\U0000FE0F", u8"pilot: light skin tone", u8"pilot: light skin tone"},
{u8"pilot", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002708\U0000FE0F", u8"pilot: medium-light skin tone", u8"pilot: medium-light skin tone"},
{u8"pilot", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002708\U0000FE0F", u8"pilot: medium skin tone", u8"pilot: medium skin tone"},
{u8"pilot", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002708\U0000FE0F", u8"pilot: medium-dark skin tone", u8"pilot: medium-dark skin tone"},
{u8"pilot", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002708\U0000FE0F", u8"pilot: dark skin tone", u8"pilot: dark skin tone"},
{u8"man pilot", u8"\U0001F468\U0001F3FB\U0000200D\U00002708\U0000FE0F", u8"man pilot: light skin tone", u8"man pilot: light skin tone"},
{u8"man pilot", u8"\U0001F468\U0001F3FC\U0000200D\U00002708\U0000FE0F", u8"man pilot: medium-light skin tone", u8"man pilot: medium-light skin tone"},
{u8"man pilot", u8"\U0001F468\U0001F3FD\U0000200D\U00002708\U0000FE0F", u8"man pilot: medium skin tone", u8"man pilot: medium skin tone"},
{u8"man pilot", u8"\U0001F468\U0001F3FE\U0000200D\U00002708\U0000FE0F", u8"man pilot: medium-dark skin tone", u8"man pilot: medium-dark skin tone"},
{u8"man pilot", u8"\U0001F468\U0001F3FF\U0000200D\U00002708\U0000FE0F", u8"man pilot: dark skin tone", u8"man pilot: dark skin tone"},
{u8"woman pilot", u8"\U0001F469\U0001F3FB\U0000200D\U00002708\U0000FE0F", u8"woman pilot: light skin tone", u8"woman pilot: light skin tone"},
{u8"woman pilot", u8"\U0001F469\U0001F3FC\U0000200D\U00002708\U0000FE0F", u8"woman pilot: medium-light skin tone", u8"woman pilot: medium-light skin tone"},
{u8"woman pilot", u8"\U0001F469\U0001F3FD\U0000200D\U00002708\U0000FE0F", u8"woman pilot: medium skin tone", u8"woman pilot: medium skin tone"},
{u8"woman pilot", u8"\U0001F469\U0001F3FE\U0000200D\U00002708\U0000FE0F", u8"woman pilot: medium-dark skin tone", u8"woman pilot: medium-dark skin tone"},
{u8"woman pilot", u8"\U0001F469\U0001F3FF\U0000200D\U00002708\U0000FE0F", u8"woman pilot: dark skin tone", u8"woman pilot: dark skin tone"},
{u8"astronaut", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F680", u8"astronaut: light skin tone", u8"astronaut: light skin tone"},
{u8"astronaut", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F680", u8"astronaut: medium-light skin tone", u8"astronaut: medium-light skin tone"},
{u8"astronaut", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F680", u8"astronaut: medium skin tone", u8"astronaut: medium skin tone"},
{u8"astronaut", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F680", u8"astronaut: medium-dark skin tone", u8"astronaut: medium-dark skin tone"},
{u8"astronaut", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F680", u8"astronaut: dark skin tone", u8"astronaut: dark skin tone"},
{u8"man astronaut", u8"\U0001F468\U0001F3FB\U0000200D\U0001F680", u8"man astronaut: light skin tone", u8"man astronaut: light skin tone"},
{u8"man astronaut", u8"\U0001F468\U0001F3FC\U0000200D\U0001F680", u8"man astronaut: medium-light skin tone", u8"man astronaut: medium-light skin tone"},
{u8"man astronaut", u8"\U0001F468\U0001F3FD\U0000200D\U0001F680", u8"man astronaut: medium skin tone", u8"man astronaut: medium skin tone"},
{u8"man astronaut", u8"\U0001F468\U0001F3FE\U0000200D\U0001F680", u8"man astronaut: medium-dark skin tone", u8"man astronaut: medium-dark skin tone"},
{u8"man astronaut", u8"\U0001F468\U0001F3FF\U0000200D\U0001F680", u8"man astronaut: dark skin tone", u8"man astronaut: dark skin tone"},
{u8"woman astronaut", u8"\U0001F469\U0001F3FB\U0000200D\U0001F680", u8"woman astronaut: light skin tone", u8"woman astronaut: light skin tone"},
{u8"woman astronaut", u8"\U0001F469\U0001F3FC\U0000200D\U0001F680", u8"woman astronaut: medium-light skin tone", u8"woman astronaut: medium-light skin tone"},
{u8"woman astronaut", u8"\U0001F469\U0001F3FD\U0000200D\U0001F680", u8"woman astronaut: medium skin tone", u8"woman astronaut: medium skin tone"},
{u8"woman astronaut", u8"\U0001F469\U0001F3FE\U0000200D\U0001F680", u8"woman astronaut: medium-dark skin tone", u8"woman astronaut: medium-dark skin tone"},
{u8"woman astronaut", u8"\U0001F469\U0001F3FF\U0000200D\U0001F680", u8"woman astronaut: dark skin tone", u8"woman astronaut: dark skin tone"},
{u8"firefighter", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F692", u8"firefighter: light skin tone", u8"firefighter: light skin tone"},
{u8"firefighter", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F692", u8"firefighter: medium-light skin tone", u8"firefighter: medium-light skin tone"},
{u8"firefighter", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F692", u8"firefighter: medium skin tone", u8"firefighter: medium skin tone"},
{u8"firefighter", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F692", u8"firefighter: medium-dark skin tone", u8"firefighter: medium-dark skin tone"},
{u8"firefighter", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F692", u8"firefighter: dark skin tone", u8"firefighter: dark skin tone"},
{u8"man firefighter", u8"\U0001F468\U0001F3FB\U0000200D\U0001F692", u8"man firefighter: light skin tone", u8"man firefighter: light skin tone"},
{u8"man firefighter", u8"\U0001F468\U0001F3FC\U0000200D\U0001F692", u8"man firefighter: medium-light skin tone", u8"man firefighter: medium-light skin tone"},
{u8"man firefighter", u8"\U0001F468\U0001F3FD\U0000200D\U0001F692", u8"man firefighter: medium skin tone", u8"man firefighter: medium skin tone"},
{u8"man firefighter", u8"\U0001F468\U0001F3FE\U0000200D\U0001F692", u8"man firefighter: medium-dark skin tone", u8"man firefighter: medium-dark skin tone"},
{u8"man firefighter", u8"\U0001F468\U0001F3FF\U0000200D\U0001F692", u8"man firefighter: dark skin tone", u8"man firefighter: dark skin tone"},
{u8"woman firefighter", u8"\U0001F469\U0001F3FB\U0000200D\U0001F692", u8"woman firefighter: light skin tone", u8"woman firefighter: light skin tone"},
{u8"woman firefighter", u8"\U0001F469\U0001F3FC\U0000200D\U0001F692", u8"woman firefighter: medium-light skin tone", u8"woman firefighter: medium-light skin tone"},
{u8"woman firefighter", u8"\U0001F469\U0001F3FD\U0000200D\U0001F692", u8"woman firefighter: medium skin tone", u8"woman firefighter: medium skin tone"},
{u8"woman firefighter", u8"\U0001F469\U0001F3FE\U0000200D\U0001F692", u8"woman firefighter: medium-dark skin tone", u8"woman firefighter: medium-dark skin tone"},
{u8"woman firefighter", u8"\U0001F469\U0001F3FF\U0000200D\U0001F692", u8"woman firefighter: dark skin tone", u8"woman firefighter: dark skin tone"},
{u8"police officer", u8"\U0001F46E\U0001F3FB", u8"cop_tone1", u8"police officer: light skin tone"},
{u8"police officer", u8"\U0001F46E\U0001F3FC", u8"cop_tone2", u8"police officer: medium-light skin tone"},
{u8"police officer", u8"\U0001F46E\U0001F3FD", u8"cop_tone3", u8"police officer: medium skin tone"},
{u8"police officer", u8"\U0001F46E\U0001F3FE", u8"cop_tone4", u8"police officer: medium-dark skin tone"},
{u8"police officer", u8"\U0001F46E\U0001F3FF", u8"cop_tone5", u8"police officer: dark skin tone"},
{u8"man police officer", u8"\U0001F46E\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man police officer: light skin tone", u8"man police officer: light skin tone"},
{u8"man police officer", u8"\U0001F46E\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man police officer: medium-light skin tone", u8"man police officer: medium-light skin tone"},
{u8"man police officer", u8"\U0001F46E\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man police officer: medium skin tone", u8"man police officer: medium skin tone"},
{u8"man police officer", u8"\U0001F46E\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man police officer: medium-dark skin tone", u8"man police officer: medium-dark skin tone"},
{u8"man police officer", u8"\U0001F46E\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man police officer: dark skin tone", u8"man police officer: dark skin tone"},
{u8"woman police officer", u8"\U0001F46E\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman police officer: light skin tone", u8"woman police officer: light skin tone"},
{u8"woman police officer", u8"\U0001F46E\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman police officer: medium-light skin tone", u8"woman police officer: medium-light skin tone"},
{u8"woman police officer", u8"\U0001F46E\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman police officer: medium skin tone", u8"woman police officer: medium skin tone"},
{u8"woman police officer", u8"\U0001F46E\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman police officer: medium-dark skin tone", u8"woman police officer: medium-dark skin tone"},
{u8"woman police officer", u8"\U0001F46E\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman police officer: dark skin tone", u8"woman police officer: dark skin tone"},
{u8"detective", u8"\U0001F575\U0001F3FB", u8"spy_tone1", u8"detective: light skin tone"},
{u8"detective", u8"\U0001F575\U0001F3FC", u8"spy_tone2", u8"detective: medium-light skin tone"},
{u8"detective", u8"\U0001F575\U0001F3FD", u8"spy_tone3", u8"detective: medium skin tone"},
{u8"detective", u8"\U0001F575\U0001F3FE", u8"spy_tone4", u8"detective: medium-dark skin tone"},
{u8"detective", u8"\U0001F575\U0001F3FF", u8"spy_tone5", u8"detective: dark skin tone"},
{u8"man detective", u8"\U0001F575\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man detective: light skin tone", u8"man detective: light skin tone"},
{u8"man detective", u8"\U0001F575\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man detective: medium-light skin tone", u8"man detective: medium-light skin tone"},
{u8"man detective", u8"\U0001F575\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man detective: medium skin tone", u8"man detective: medium skin tone"},
{u8"man detective", u8"\U0001F575\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man detective: medium-dark skin tone", u8"man detective: medium-dark skin tone"},
{u8"man detective", u8"\U0001F575\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man detective: dark skin tone", u8"man detective: dark skin tone"},
{u8"woman detective", u8"\U0001F575\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman detective: light skin tone", u8"woman detective: light skin tone"},
{u8"woman detective", u8"\U0001F575\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman detective: medium-light skin tone", u8"woman detective: medium-light skin tone"},
{u8"woman detective", u8"\U0001F575\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman detective: medium skin tone", u8"woman detective: medium skin tone"},
{u8"woman detective", u8"\U0001F575\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman detective: medium-dark skin tone", u8"woman detective: medium-dark skin tone"},
{u8"woman detective", u8"\U0001F575\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman detective: dark skin tone", u8"woman detective: dark skin tone"},
{u8"guard", u8"\U0001F482\U0001F3FB", u8"guardsman_tone1", u8"guard: light skin tone"},
{u8"guard", u8"\U0001F482\U0001F3FC", u8"guardsman_tone2", u8"guard: medium-light skin tone"},
{u8"guard", u8"\U0001F482\U0001F3FD", u8"guardsman_tone3", u8"guard: medium skin tone"},
{u8"guard", u8"\U0001F482\U0001F3FE", u8"guardsman_tone4", u8"guard: medium-dark skin tone"},
{u8"guard", u8"\U0001F482\U0001F3FF", u8"guardsman_tone5", u8"guard: dark skin tone"},
{u8"man guard", u8"\U0001F482\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man guard: light skin tone", u8"man guard: light skin tone"},
{u8"man guard", u8"\U0001F482\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man guard: medium-light skin tone", u8"man guard: medium-light skin tone"},
{u8"man guard", u8"\U0001F482\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man guard: medium skin tone", u8"man guard: medium skin tone"},
{u8"man guard", u8"\U0001F482\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man guard: medium-dark skin tone", u8"man guard: medium-dark skin tone"},
{u8"man guard", u8"\U0001F482\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man guard: dark skin tone", u8"man guard: dark skin tone"},
{u8"woman guard", u8"\U0001F482\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman guard: light skin tone", u8"woman guard: light skin tone"},
{u8"woman guard", u8"\U0001F482\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman guard: medium-light skin tone", u8"woman guard: medium-light skin tone"},
{u8"woman guard", u8"\U0001F482\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman guard: medium skin tone", u8"woman guard: medium skin tone"},
{u8"woman guard", u8"\U0001F482\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman guard: medium-dark skin tone", u8"woman guard: medium-dark skin tone"},
{u8"woman guard", u8"\U0001F482\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman guard: dark skin tone", u8"woman guard: dark skin tone"},
{u8"ninja", u8"\U0001F977\U0001F3FB", u8"ninja: light skin tone", u8"ninja: light skin tone"},
{u8"ninja", u8"\U0001F977\U0001F3FC", u8"ninja: medium-light skin tone", u8"ninja: medium-light skin tone"},
{u8"ninja", u8"\U0001F977\U0001F3FD", u8"ninja: medium skin tone", u8"ninja: medium skin tone"},
{u8"ninja", u8"\U0001F977\U0001F3FE", u8"ninja: medium-dark skin tone", u8"ninja: medium-dark skin tone"},
{u8"ninja", u8"\U0001F977\U0001F3FF", u8"ninja: dark skin tone", u8"ninja: dark skin tone"},
{u8"construction worker", u8"\U0001F477\U0001F3FB", u8"construction_worker_tone1", u8"construction worker: light skin tone"},
{u8"construction worker", u8"\U0001F477\U0001F3FC", u8"construction_worker_tone2", u8"construction worker: medium-light skin tone"},
{u8"construction worker", u8"\U0001F477\U0001F3FD", u8"construction_worker_tone3", u8"construction worker: medium skin tone"},
{u8"construction worker", u8"\U0001F477\U0001F3FE", u8"construction_worker_tone4", u8"construction worker: medium-dark skin tone"},
{u8"construction worker", u8"\U0001F477\U0001F3FF", u8"construction_worker_tone5", u8"construction worker: dark skin tone"},
{u8"man construction worker", u8"\U0001F477\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man construction worker: light skin tone", u8"man construction worker: light skin tone"},
{u8"man construction worker", u8"\U0001F477\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man construction worker: medium-light skin tone", u8"man construction worker: medium-light skin tone"},
{u8"man construction worker", u8"\U0001F477\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man construction worker: medium skin tone", u8"man construction worker: medium skin tone"},
{u8"man construction worker", u8"\U0001F477\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man construction worker: medium-dark skin tone", u8"man construction worker: medium-dark skin tone"},
{u8"man construction worker", u8"\U0001F477\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man construction worker: dark skin tone", u8"man construction worker: dark skin tone"},
{u8"woman construction worker", u8"\U0001F477\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman construction worker: light skin tone", u8"woman construction worker: light skin tone"},
{u8"woman construction worker", u8"\U0001F477\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman construction worker: medium-light skin tone", u8"woman construction worker: medium-light skin tone"},
{u8"woman construction worker", u8"\U0001F477\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman construction worker: medium skin tone", u8"woman construction worker: medium skin tone"},
{u8"woman construction worker", u8"\U0001F477\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman construction worker: medium-dark skin tone", u8"woman construction worker: medium-dark skin tone"},
{u8"woman construction worker", u8"\U0001F477\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman construction worker: dark skin tone", u8"woman construction worker: dark skin tone"},
{u8"person with crown", u8"\U0001FAC5\U0001F3FB", u8"person with crown: light skin tone", u8"person with crown: light skin tone"},
{u8"person with crown", u8"\U0001FAC5\U0001F3FC", u8"person with crown: medium-light skin tone", u8"person with crown: medium-light skin tone"},
{u8"person with crown", u8"\U0001FAC5\U0001F3FD", u8"person with crown: medium skin tone", u8"person with crown: medium skin tone"},
{u8"person with crown", u8"\U0001FAC5\U0001F3FE", u8"person with crown: medium-dark skin tone", u8"person with crown: medium-dark skin tone"},
{u8"person with crown", u8"\U0001FAC5\U0001F3FF", u8"person with crown: dark skin tone", u8"person with crown: dark skin tone"},
{u8"prince", u8"\U0001F934\U0001F3FB", u8"prince_tone1", u8"prince: light skin tone"},
{u8"prince", u8"\U0001F934\U0001F3FC", u8"prince_tone2", u8"prince: medium-light skin tone"},
{u8"prince", u8"\U0001F934\U0001F3FD", u8"prince_tone3", u8"prince: medium skin tone"},
{u8"prince", u8"\U0001F934\U0001F3FE", u8"prince_tone4", u8"prince: medium-dark skin tone"},
{u8"prince", u8"\U0001F934\U0001F3FF", u8"prince_tone5", u8"prince: dark skin tone"},
{u8"princess", u8"\U0001F478\U0001F3FB", u8"princess_tone1", u8"princess: light skin tone"},
{u8"princess", u8"\U0001F478\U0001F3FC", u8"princess_tone2", u8"princess: medium-light skin tone"},
{u8"princess", u8"\U0001F478\U0001F3FD", u8"princess_tone3", u8"princess: medium skin tone"},
{u8"princess", u8"\U0001F478\U0001F3FE", u8"princess_tone4", u8"princess: medium-dark skin tone"},
{u8"princess", u8"\U0001F478\U0001F3FF", u8"princess_tone5", u8"princess: dark skin tone"},
{u8"person wearing turban", u8"\U0001F473\U0001F3FB", u8"man_with_turban_tone1", u8"person wearing turban: light skin tone"},
{u8"person wearing turban", u8"\U0001F473\U0001F3FC", u8"man_with_turban_tone2", u8"person wearing turban: medium-light skin tone"},
{u8"person wearing turban", u8"\U0001F473\U0001F3FD", u8"man_with_turban_tone3", u8"person wearing turban: medium skin tone"},
{u8"person wearing turban", u8"\U0001F473\U0001F3FE", u8"man_with_turban_tone4", u8"person wearing turban: medium-dark skin tone"},
{u8"person wearing turban", u8"\U0001F473\U0001F3FF", u8"man_with_turban_tone5", u8"person wearing turban: dark skin tone"},
{u8"man wearing turban", u8"\U0001F473\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man wearing turban: light skin tone", u8"man wearing turban: light skin tone"},
{u8"man wearing turban", u8"\U0001F473\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man wearing turban: medium-light skin tone", u8"man wearing turban: medium-light skin tone"},
{u8"man wearing turban", u8"\U0001F473\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man wearing turban: medium skin tone", u8"man wearing turban: medium skin tone"},
{u8"man wearing turban", u8"\U0001F473\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man wearing turban: medium-dark skin tone", u8"man wearing turban: medium-dark skin tone"},
{u8"man wearing turban", u8"\U0001F473\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man wearing turban: dark skin tone", u8"man wearing turban: dark skin tone"},
{u8"woman wearing turban", u8"\U0001F473\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman wearing turban: light skin tone", u8"woman wearing turban: light skin tone"},
{u8"woman wearing turban", u8"\U0001F473\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman wearing turban: medium-light skin tone", u8"woman wearing turban: medium-light skin tone"},
{u8"woman wearing turban", u8"\U0001F473\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman wearing turban: medium skin tone", u8"woman wearing turban: medium skin tone"},
{u8"woman wearing turban", u8"\U0001F473\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman wearing turban: medium-dark skin tone", u8"woman wearing turban: medium-dark skin tone"},
{u8"woman wearing turban", u8"\U0001F473\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman wearing turban: dark skin tone", u8"woman wearing turban: dark skin tone"},
{u8"person with skullcap", u8"\U0001F472\U0001F3FB", u8"man_with_gua_pi_mao_tone1", u8"person with skullcap: light skin tone"},
{u8"person with skullcap", u8"\U0001F472\U0001F3FC", u8"man_with_gua_pi_mao_tone2", u8"person with skullcap: medium-light skin tone"},
{u8"person with skullcap", u8"\U0001F472\U0001F3FD", u8"man_with_gua_pi_mao_tone3", u8"person with skullcap: medium skin tone"},
{u8"person with skullcap", u8"\U0001F472\U0001F3FE", u8"man_with_gua_pi_mao_tone4", u8"person with skullcap: medium-dark skin tone"},
{u8"person with skullcap", u8"\U0001F472\U0001F3FF", u8"man_with_gua_pi_mao_tone5", u8"person with skullcap: dark skin tone"},
{u8"woman with headscarf", u8"\U0001F9D5\U0001F3FB", u8"woman_with_headscarf_tone1", u8"woman with headscarf: light skin tone"},
{u8"woman with headscarf", u8"\U0001F9D5\U0001F3FC", u8"woman_with_headscarf_tone2", u8"woman with headscarf: medium-light skin tone"},
{u8"woman with headscarf", u8"\U0001F9D5\U0001F3FD", u8"woman_with_headscarf_tone3", u8"woman with headscarf: medium skin tone"},
{u8"woman with headscarf", u8"\U0001F9D5\U0001F3FE", u8"woman_with_headscarf_tone4", u8"woman with headscarf: medium-dark skin tone"},
{u8"woman with headscarf", u8"\U0001F9D5\U0001F3FF", u8"woman_with_headscarf_tone5", u8"woman with headscarf: dark skin tone"},
{u8"person in tuxedo", u8"\U0001F935\U0001F3FB", u8"man_in_tuxedo_tone1", u8"person in tuxedo: light skin tone"},
{u8"person in tuxedo", u8"\U0001F935\U0001F3FC", u8"man_in_tuxedo_tone2", u8"person in tuxedo: medium-light skin tone"},
{u8"person in tuxedo", u8"\U0001F935\U0001F3FD", u8"man_in_tuxedo_tone3", u8"person in tuxedo: medium skin tone"},
{u8"person in tuxedo", u8"\U0001F935\U0001F3FE", u8"man_in_tuxedo_tone4", u8"person in tuxedo: medium-dark skin tone"},
{u8"person in tuxedo", u8"\U0001F935\U0001F3FF", u8"man_in_tuxedo_tone5", u8"person in tuxedo: dark skin tone"},
{u8"man in tuxedo", u8"\U0001F935\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man in tuxedo: light skin tone", u8"man in tuxedo: light skin tone"},
{u8"man in tuxedo", u8"\U0001F935\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man in tuxedo: medium-light skin tone", u8"man in tuxedo: medium-light skin tone"},
{u8"man in tuxedo", u8"\U0001F935\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man in tuxedo: medium skin tone", u8"man in tuxedo: medium skin tone"},
{u8"man in tuxedo", u8"\U0001F935\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man in tuxedo: medium-dark skin tone", u8"man in tuxedo: medium-dark skin tone"},
{u8"man in tuxedo", u8"\U0001F935\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man in tuxedo: dark skin tone", u8"man in tuxedo: dark skin tone"},
{u8"woman in tuxedo", u8"\U0001F935\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman in tuxedo: light skin tone", u8"woman in tuxedo: light skin tone"},
{u8"woman in tuxedo", u8"\U0001F935\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman in tuxedo: medium-light skin tone", u8"woman in tuxedo: medium-light skin tone"},
{u8"woman in tuxedo", u8"\U0001F935\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman in tuxedo: medium skin tone", u8"woman in tuxedo: medium skin tone"},
{u8"woman in tuxedo", u8"\U0001F935\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman in tuxedo: medium-dark skin tone", u8"woman in tuxedo: medium-dark skin tone"},
{u8"woman in tuxedo", u8"\U0001F935\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman in tuxedo: dark skin tone", u8"woman in tuxedo: dark skin tone"},
{u8"person with veil", u8"\U0001F470\U0001F3FB", u8"bride_with_veil_tone1", u8"person with veil: light skin tone"},
{u8"person with veil", u8"\U0001F470\U0001F3FC", u8"bride_with_veil_tone2", u8"person with veil: medium-light skin tone"},
{u8"person with veil", u8"\U0001F470\U0001F3FD", u8"bride_with_veil_tone3", u8"person with veil: medium skin tone"},
{u8"person with veil", u8"\U0001F470\U0001F3FE", u8"bride_with_veil_tone4", u8"person with veil: medium-dark skin tone"},
{u8"person with veil", u8"\U0001F470\U0001F3FF", u8"bride_with_veil_tone5", u8"person with veil: dark skin tone"},
{u8"man with veil", u8"\U0001F470\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man with veil: light skin tone", u8"man with veil: light skin tone"},
{u8"man with veil", u8"\U0001F470\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man with veil: medium-light skin tone", u8"man with veil: medium-light skin tone"},
{u8"man with veil", u8"\U0001F470\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man with veil: medium skin tone", u8"man with veil: medium skin tone"},
{u8"man with veil", u8"\U0001F470\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man with veil: medium-dark skin tone", u8"man with veil: medium-dark skin tone"},
{u8"man with veil", u8"\U0001F470\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man with veil: dark skin tone", u8"man with veil: dark skin tone"},
{u8"woman with veil", u8"\U0001F470\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman with veil: light skin tone", u8"woman with veil: light skin tone"},
{u8"woman with veil", u8"\U0001F470\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman with veil: medium-light skin tone", u8"woman with veil: medium-light skin tone"},
{u8"woman with veil", u8"\U0001F470\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman with veil: medium skin tone", u8"woman with veil: medium skin tone"},
{u8"woman with veil", u8"\U0001F470\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman with veil: medium-dark skin tone", u8"woman with veil: medium-dark skin tone"},
{u8"woman with veil", u8"\U0001F470\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman with veil: dark skin tone", u8"woman with veil: dark skin tone"},
{u8"pregnant woman", u8"\U0001F930\U0001F3FB", u8"pregnant_woman_tone1", u8"pregnant woman: light skin tone"},
{u8"pregnant woman", u8"\U0001F930\U0001F3FC", u8"pregnant_woman_tone2", u8"pregnant woman: medium-light skin tone"},
{u8"pregnant woman", u8"\U0001F930\U0001F3FD", u8"pregnant_woman_tone3", u8"pregnant woman: medium skin tone"},
{u8"pregnant woman", u8"\U0001F930\U0001F3FE", u8"pregnant_woman_tone4", u8"pregnant woman: medium-dark skin tone"},
{u8"pregnant woman", u8"\U0001F930\U0001F3FF", u8"pregnant_woman_tone5", u8"pregnant woman: dark skin tone"},
{u8"pregnant man", u8"\U0001FAC3\U0001F3FB", u8"pregnant man: light skin tone", u8"pregnant man: light skin tone"},
{u8"pregnant man", u8"\U0001FAC3\U0001F3FC", u8"pregnant man: medium-light skin tone", u8"pregnant man: medium-light skin tone"},
{u8"pregnant man", u8"\U0001FAC3\U0001F3FD", u8"pregnant man: medium skin tone", u8"pregnant man: medium skin tone"},
{u8"pregnant man", u8"\U0001FAC3\U0001F3FE", u8"pregnant man: medium-dark skin tone", u8"pregnant man: medium-dark skin tone"},
{u8"pregnant man", u8"\U0001FAC3\U0001F3FF", u8"pregnant man: dark skin tone", u8"pregnant man: dark skin tone"},
{u8"pregnant person", u8"\U0001FAC4\U0001F3FB", u8"pregnant person: light skin tone", u8"pregnant person: light skin tone"},
{u8"pregnant person", u8"\U0001FAC4\U0001F3FC", u8"pregnant person: medium-light skin tone", u8"pregnant person: medium-light skin tone"},
{u8"pregnant person", u8"\U0001FAC4\U0001F3FD", u8"pregnant person: medium skin tone", u8"pregnant person: medium skin tone"},
{u8"pregnant person", u8"\U0001FAC4\U0001F3FE", u8"pregnant person: medium-dark skin tone", u8"pregnant person: medium-dark skin tone"},
{u8"pregnant person", u8"\U0001FAC4\U0001F3FF", u8"pregnant person: dark skin tone", u8"pregnant person: dark skin tone"},
{u8"breast-feeding", u8"\U0001F931\U0001F3FB", u8"breast_feeding_tone1", u8"breast-feeding: light skin tone"},
{u8"breast-feeding", u8"\U0001F931\U0001F3FC", u8"breast_feeding_tone2", u8"breast-feeding: medium-light skin tone"},
{u8"breast-feeding", u8"\U0001F931\U0001F3FD", u8"breast_feeding_tone3", u8"breast-feeding: medium skin tone"},
{u8"breast-feeding", u8"\U0001F931\U0001F3FE", u8"breast_feeding_tone4", u8"breast-feeding: medium-dark skin tone"},
{u8"breast-feeding", u8"\U0001F931\U0001F3FF", u8"breast_feeding_tone5", u8"breast-feeding: dark skin tone"},
{u8"woman feeding baby", u8"\U0001F469\U0001F3FB\U0000200D\U0001F37C", u8"woman feeding baby: light skin tone", u8"woman feeding baby: light skin tone"},
{u8"woman feeding baby", u8"\U0001F469\U0001F3FC\U0000200D\U0001F37C", u8"woman feeding baby: medium-light skin tone", u8"woman feeding baby: medium-light skin tone"},
{u8"woman feeding baby", u8"\U0001F469\U0001F3FD\U0000200D\U0001F37C", u8"woman feeding baby: medium skin tone", u8"woman feeding baby: medium skin tone"},
{u8"woman feeding baby", u8"\U0001F469\U0001F3FE\U0000200D\U0001F37C", u8"woman feeding baby: medium-dark skin tone", u8"woman feeding baby: medium-dark skin tone"},
{u8"woman feeding baby", u8"\U0001F469\U0001F3FF\U0000200D\U0001F37C", u8"woman feeding baby: dark skin tone", u8"woman feeding baby: dark skin tone"},
{u8"man feeding baby", u8"\U0001F468\U0001F3FB\U0000200D\U0001F37C", u8"man feeding baby: light skin tone", u8"man feeding baby: light skin tone"},
{u8"man feeding baby", u8"\U0001F468\U0001F3FC\U0000200D\U0001F37C", u8"man feeding baby: medium-light skin tone", u8"man feeding baby: medium-light skin tone"},
{u8"man feeding baby", u8"\U0001F468\U0001F3FD\U0000200D\U0001F37C", u8"man feeding baby: medium skin tone", u8"man feeding baby: medium skin tone"},
{u8"man feeding baby", u8"\U0001F468\U0001F3FE\U0000200D\U0001F37C", u8"man feeding baby: medium-dark skin tone", u8"man feeding baby: medium-dark skin tone"},
{u8"man feeding baby", u8"\U0001F468\U0001F3FF\U0000200D\U0001F37C", u8"man feeding baby: dark skin tone", u8"man feeding baby: dark skin tone"},
{u8"person feeding baby", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F37C", u8"person feeding baby: light skin tone", u8"person feeding baby: light skin tone"},
{u8"person feeding baby", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F37C", u8"person feeding baby: medium-light skin tone", u8"person feeding baby: medium-light skin tone"},
{u8"person feeding baby", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F37C", u8"person feeding baby: medium skin tone", u8"person feeding baby: medium skin tone"},
{u8"person feeding baby", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F37C", u8"person feeding baby: medium-dark skin tone", u8"person feeding baby: medium-dark skin tone"},
{u8"person feeding baby", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F37C", u8"person feeding baby: dark skin tone", u8"person feeding baby: dark skin tone"},
{u8"baby angel", u8"\U0001F47C\U0001F3FB", u8"angel_tone1", u8"baby angel: light skin tone"},
{u8"baby angel", u8"\U0001F47C\U0001F3FC", u8"angel_tone2", u8"baby angel: medium-light skin tone"},
{u8"baby angel", u8"\U0001F47C\U0001F3FD", u8"angel_tone3", u8"baby angel: medium skin tone"},
{u8"baby angel", u8"\U0001F47C\U0001F3FE", u8"angel_tone4", u8"baby angel: medium-dark skin tone"},
{u8"baby angel", u8"\U0001F47C\U0001F3FF", u8"angel_tone5", u8"baby angel: dark skin tone"},
{u8"Santa Claus", u8"\U0001F385\U0001F3FB", u8"santa_tone1", u8"Santa Claus: light skin tone"},
{u8"Santa Claus", u8"\U0001F385\U0001F3FC", u8"santa_tone2", u8"Santa Claus: medium-light skin tone"},
{u8"Santa Claus", u8"\U0001F385\U0001F3FD", u8"santa_tone3", u8"Santa Claus: medium skin tone"},
{u8"Santa Claus", u8"\U0001F385\U0001F3FE", u8"santa_tone4", u8"Santa Claus: medium-dark skin tone"},
{u8"Santa Claus", u8"\U0001F385\U0001F3FF", u8"santa_tone5", u8"Santa Claus: dark skin tone"},
{u8"Mrs. Claus", u8"\U0001F936\U0001F3FB", u8"mrs_claus_tone1", u8"Mrs. Claus: light skin tone"},
{u8"Mrs. Claus", u8"\U0001F936\U0001F3FC", u8"mrs_claus_tone2", u8"Mrs. Claus: medium-light skin tone"},
{u8"Mrs. Claus", u8"\U0001F936\U0001F3FD", u8"mrs_claus_tone3", u8"Mrs. Claus: medium skin tone"},
{u8"Mrs. Claus", u8"\U0001F936\U0001F3FE", u8"mrs_claus_tone4", u8"Mrs. Claus: medium-dark skin tone"},
{u8"Mrs. Claus", u8"\U0001F936\U0001F3FF", u8"mrs_claus_tone5", u8"Mrs. Claus: dark skin tone"},
{u8"mx claus", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F384", u8"mx claus: light skin tone", u8"mx claus: light skin tone"},
{u8"mx claus", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F384", u8"mx claus: medium-light skin tone", u8"mx claus: medium-light skin tone"},
{u8"mx claus", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F384", u8"mx claus: medium skin tone", u8"mx claus: medium skin tone"},
{u8"mx claus", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F384", u8"mx claus: medium-dark skin tone", u8"mx claus: medium-dark skin tone"},
{u8"mx claus", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F384", u8"mx claus: dark skin tone", u8"mx claus: dark skin tone"},
{u8"superhero", u8"\U0001F9B8\U0001F3FB", u8"superhero_tone1", u8"superhero: light skin tone"},
{u8"superhero", u8"\U0001F9B8\U0001F3FC", u8"superhero_tone2", u8"superhero: medium-light skin tone"},
{u8"superhero", u8"\U0001F9B8\U0001F3FD", u8"superhero_tone3", u8"superhero: medium skin tone"},
{u8"superhero", u8"\U0001F9B8\U0001F3FE", u8"superhero_tone4", u8"superhero: medium-dark skin tone"},
{u8"superhero", u8"\U0001F9B8\U0001F3FF", u8"superhero_tone5", u8"superhero: dark skin tone"},
{u8"man superhero", u8"\U0001F9B8\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man superhero: light skin tone", u8"man superhero: light skin tone"},
{u8"man superhero", u8"\U0001F9B8\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man superhero: medium-light skin tone", u8"man superhero: medium-light skin tone"},
{u8"man superhero", u8"\U0001F9B8\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man superhero: medium skin tone", u8"man superhero: medium skin tone"},
{u8"man superhero", u8"\U0001F9B8\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man superhero: medium-dark skin tone", u8"man superhero: medium-dark skin tone"},
{u8"man superhero", u8"\U0001F9B8\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man superhero: dark skin tone", u8"man superhero: dark skin tone"},
{u8"woman superhero", u8"\U0001F9B8\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman superhero: light skin tone", u8"woman superhero: light skin tone"},
{u8"woman superhero", u8"\U0001F9B8\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman superhero: medium-light skin tone", u8"woman superhero: medium-light skin tone"},
{u8"woman superhero", u8"\U0001F9B8\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman superhero: medium skin tone", u8"woman superhero: medium skin tone"},
{u8"woman superhero", u8"\U0001F9B8\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman superhero: medium-dark skin tone", u8"woman superhero: medium-dark skin tone"},
{u8"woman superhero", u8"\U0001F9B8\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman superhero: dark skin tone", u8"woman superhero: dark skin tone"},
{u8"supervillain", u8"\U0001F9B9\U0001F3FB", u8"supervillain_tone1", u8"supervillain: light skin tone"},
{u8"supervillain", u8"\U0001F9B9\U0001F3FC", u8"supervillain_tone2", u8"supervillain: medium-light skin tone"},
{u8"supervillain", u8"\U0001F9B9\U0001F3FD", u8"supervillain_tone3", u8"supervillain: medium skin tone"},
{u8"supervillain", u8"\U0001F9B9\U0001F3FE", u8"supervillain_tone4", u8"supervillain: medium-dark skin tone"},
{u8"supervillain", u8"\U0001F9B9\U0001F3FF", u8"supervillain_tone5", u8"supervillain: dark skin tone"},
{u8"man supervillain", u8"\U0001F9B9\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man supervillain: light skin tone", u8"man supervillain: light skin tone"},
{u8"man supervillain", u8"\U0001F9B9\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man supervillain: medium-light skin tone", u8"man supervillain: medium-light skin tone"},
{u8"man supervillain", u8"\U0001F9B9\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man supervillain: medium skin tone", u8"man supervillain: medium skin tone"},
{u8"man supervillain", u8"\U0001F9B9\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man supervillain: medium-dark skin tone", u8"man supervillain: medium-dark skin tone"},
{u8"man supervillain", u8"\U0001F9B9\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man supervillain: dark skin tone", u8"man supervillain: dark skin tone"},
{u8"woman supervillain", u8"\U0001F9B9\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman supervillain: light skin tone", u8"woman supervillain: light skin tone"},
{u8"woman supervillain", u8"\U0001F9B9\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman supervillain: medium-light skin tone", u8"woman supervillain: medium-light skin tone"},
{u8"woman supervillain", u8"\U0001F9B9\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman supervillain: medium skin tone", u8"woman supervillain: medium skin tone"},
{u8"woman supervillain", u8"\U0001F9B9\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman supervillain: medium-dark skin tone", u8"woman supervillain: medium-dark skin tone"},
{u8"woman supervillain", u8"\U0001F9B9\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman supervillain: dark skin tone", u8"woman supervillain: dark skin tone"},
{u8"mage", u8"\U0001F9D9\U0001F3FB", u8"mage_tone1", u8"mage: light skin tone"},
{u8"mage", u8"\U0001F9D9\U0001F3FC", u8"mage_tone2", u8"mage: medium-light skin tone"},
{u8"mage", u8"\U0001F9D9\U0001F3FD", u8"mage_tone3", u8"mage: medium skin tone"},
{u8"mage", u8"\U0001F9D9\U0001F3FE", u8"mage_tone4", u8"mage: medium-dark skin tone"},
{u8"mage", u8"\U0001F9D9\U0001F3FF", u8"mage_tone5", u8"mage: dark skin tone"},
{u8"man mage", u8"\U0001F9D9\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man mage: light skin tone", u8"man mage: light skin tone"},
{u8"man mage", u8"\U0001F9D9\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man mage: medium-light skin tone", u8"man mage: medium-light skin tone"},
{u8"man mage", u8"\U0001F9D9\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man mage: medium skin tone", u8"man mage: medium skin tone"},
{u8"man mage", u8"\U0001F9D9\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man mage: medium-dark skin tone", u8"man mage: medium-dark skin tone"},
{u8"man mage", u8"\U0001F9D9\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man mage: dark skin tone", u8"man mage: dark skin tone"},
{u8"woman mage", u8"\U0001F9D9\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman mage: light skin tone", u8"woman mage: light skin tone"},
{u8"woman mage", u8"\U0001F9D9\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman mage: medium-light skin tone", u8"woman mage: medium-light skin tone"},
{u8"woman mage", u8"\U0001F9D9\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman mage: medium skin tone", u8"woman mage: medium skin tone"},
{u8"woman mage", u8"\U0001F9D9\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman mage: medium-dark skin tone", u8"woman mage: medium-dark skin tone"},
{u8"woman mage", u8"\U0001F9D9\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman mage: dark skin tone", u8"woman mage: dark skin tone"},
{u8"fairy", u8"\U0001F9DA\U0001F3FB", u8"fairy_tone1", u8"fairy: light skin tone"},
{u8"fairy", u8"\U0001F9DA\U0001F3FC", u8"fairy_tone2", u8"fairy: medium-light skin tone"},
{u8"fairy", u8"\U0001F9DA\U0001F3FD", u8"fairy_tone3", u8"fairy: medium skin tone"},
{u8"fairy", u8"\U0001F9DA\U0001F3FE", u8"fairy_tone4", u8"fairy: medium-dark skin tone"},
{u8"fairy", u8"\U0001F9DA\U0001F3FF", u8"fairy_tone5", u8"fairy: dark skin tone"},
{u8"man fairy", u8"\U0001F9DA\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man fairy: light skin tone", u8"man fairy: light skin tone"},
{u8"man fairy", u8"\U0001F9DA\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man fairy: medium-light skin tone", u8"man fairy: medium-light skin tone"},
{u8"man fairy", u8"\U0001F9DA\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man fairy: medium skin tone", u8"man fairy: medium skin tone"},
{u8"man fairy", u8"\U0001F9DA\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man fairy: medium-dark skin tone", u8"man fairy: medium-dark skin tone"},
{u8"man fairy", u8"\U0001F9DA\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man fairy: dark skin tone", u8"man fairy: dark skin tone"},
{u8"woman fairy", u8"\U0001F9DA\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman fairy: light skin tone", u8"woman fairy: light skin tone"},
{u8"woman fairy", u8"\U0001F9DA\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman fairy: medium-light skin tone", u8"woman fairy: medium-light skin tone"},
{u8"woman fairy", u8"\U0001F9DA\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman fairy: medium skin tone", u8"woman fairy: medium skin tone"},
{u8"woman fairy", u8"\U0001F9DA\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman fairy: medium-dark skin tone", u8"woman fairy: medium-dark skin tone"},
{u8"woman fairy", u8"\U0001F9DA\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman fairy: dark skin tone", u8"woman fairy: dark skin tone"},
{u8"vampire", u8"\U0001F9DB\U0001F3FB", u8"vampire_tone1", u8"vampire: light skin tone"},
{u8"vampire", u8"\U0001F9DB\U0001F3FC", u8"vampire_tone2", u8"vampire: medium-light skin tone"},
{u8"vampire", u8"\U0001F9DB\U0001F3FD", u8"vampire_tone3", u8"vampire: medium skin tone"},
{u8"vampire", u8"\U0001F9DB\U0001F3FE", u8"vampire_tone4", u8"vampire: medium-dark skin tone"},
{u8"vampire", u8"\U0001F9DB\U0001F3FF", u8"vampire_tone5", u8"vampire: dark skin tone"},
{u8"man vampire", u8"\U0001F9DB\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man vampire: light skin tone", u8"man vampire: light skin tone"},
{u8"man vampire", u8"\U0001F9DB\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man vampire: medium-light skin tone", u8"man vampire: medium-light skin tone"},
{u8"man vampire", u8"\U0001F9DB\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man vampire: medium skin tone", u8"man vampire: medium skin tone"},
{u8"man vampire", u8"\U0001F9DB\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man vampire: medium-dark skin tone", u8"man vampire: medium-dark skin tone"},
{u8"man vampire", u8"\U0001F9DB\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man vampire: dark skin tone", u8"man vampire: dark skin tone"},
{u8"woman vampire", u8"\U0001F9DB\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman vampire: light skin tone", u8"woman vampire: light skin tone"},
{u8"woman vampire", u8"\U0001F9DB\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman vampire: medium-light skin tone", u8"woman vampire: medium-light skin tone"},
{u8"woman vampire", u8"\U0001F9DB\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman vampire: medium skin tone", u8"woman vampire: medium skin tone"},
{u8"woman vampire", u8"\U0001F9DB\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman vampire: medium-dark skin tone", u8"woman vampire: medium-dark skin tone"},
{u8"woman vampire", u8"\U0001F9DB\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman vampire: dark skin tone", u8"woman vampire: dark skin tone"},
{u8"merperson", u8"\U0001F9DC\U0001F3FB", u8"merperson_tone1", u8"merperson: light skin tone"},
{u8"merperson", u8"\U0001F9DC\U0001F3FC", u8"merperson_tone2", u8"merperson: medium-light skin tone"},
{u8"merperson", u8"\U0001F9DC\U0001F3FD", u8"merperson_tone3", u8"merperson: medium skin tone"},
{u8"merperson", u8"\U0001F9DC\U0001F3FE", u8"merperson_tone4", u8"merperson: medium-dark skin tone"},
{u8"merperson", u8"\U0001F9DC\U0001F3FF", u8"merperson_tone5", u8"merperson: dark skin tone"},
{u8"merman", u8"\U0001F9DC\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"merman: light skin tone", u8"merman: light skin tone"},
{u8"merman", u8"\U0001F9DC\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"merman: medium-light skin tone", u8"merman: medium-light skin tone"},
{u8"merman", u8"\U0001F9DC\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"merman: medium skin tone", u8"merman: medium skin tone"},
{u8"merman", u8"\U0001F9DC\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"merman: medium-dark skin tone", u8"merman: medium-dark skin tone"},
{u8"merman", u8"\U0001F9DC\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"merman: dark skin tone", u8"merman: dark skin tone"},
{u8"mermaid", u8"\U0001F9DC\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"mermaid: light skin tone", u8"mermaid: light skin tone"},
{u8"mermaid", u8"\U0001F9DC\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"mermaid: medium-light skin tone", u8"mermaid: medium-light skin tone"},
{u8"mermaid", u8"\U0001F9DC\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"mermaid: medium skin tone", u8"mermaid: medium skin tone"},
{u8"mermaid", u8"\U0001F9DC\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"mermaid: medium-dark skin tone", u8"mermaid: medium-dark skin tone"},
{u8"mermaid", u8"\U0001F9DC\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"mermaid: dark skin tone", u8"mermaid: dark skin tone"},
{u8"elf", u8"\U0001F9DD\U0001F3FB", u8"elf_tone1", u8"elf: light skin tone"},
{u8"elf", u8"\U0001F9DD\U0001F3FC", u8"elf_tone2", u8"elf: medium-light skin tone"},
{u8"elf", u8"\U0001F9DD\U0001F3FD", u8"elf_tone3", u8"elf: medium skin tone"},
{u8"elf", u8"\U0001F9DD\U0001F3FE", u8"elf_tone4", u8"elf: medium-dark skin tone"},
{u8"elf", u8"\U0001F9DD\U0001F3FF", u8"elf_tone5", u8"elf: dark skin tone"},
{u8"man elf", u8"\U0001F9DD\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man elf: light skin tone", u8"man elf: light skin tone"},
{u8"man elf", u8"\U0001F9DD\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man elf: medium-light skin tone", u8"man elf: medium-light skin tone"},
{u8"man elf", u8"\U0001F9DD\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man elf: medium skin tone", u8"man elf: medium skin tone"},
{u8"man elf", u8"\U0001F9DD\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man elf: medium-dark skin tone", u8"man elf: medium-dark skin tone"},
{u8"man elf", u8"\U0001F9DD\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man elf: dark skin tone", u8"man elf: dark skin tone"},
{u8"woman elf", u8"\U0001F9DD\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman elf: light skin tone", u8"woman elf: light skin tone"},
{u8"woman elf", u8"\U0001F9DD\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman elf: medium-light skin tone", u8"woman elf: medium-light skin tone"},
{u8"woman elf", u8"\U0001F9DD\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman elf: medium skin tone", u8"woman elf: medium skin tone"},
{u8"woman elf", u8"\U0001F9DD\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman elf: medium-dark skin tone", u8"woman elf: medium-dark skin tone"},
{u8"woman elf", u8"\U0001F9DD\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman elf: dark skin tone", u8"woman elf: dark skin tone"},
{u8"person getting massage", u8"\U0001F486\U0001F3FB", u8"massage_tone1", u8"person getting massage: light skin tone"},
{u8"person getting massage", u8"\U0001F486\U0001F3FC", u8"massage_tone2", u8"person getting massage: medium-light skin tone"},
{u8"person getting massage", u8"\U0001F486\U0001F3FD", u8"massage_tone3", u8"person getting massage: medium skin tone"},
{u8"person getting massage", u8"\U0001F486\U0001F3FE", u8"massage_tone4", u8"person getting massage: medium-dark skin tone"},
{u8"person getting massage", u8"\U0001F486\U0001F3FF", u8"massage_tone5", u8"person getting massage: dark skin tone"},
{u8"man getting massage", u8"\U0001F486\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man getting massage: light skin tone", u8"man getting massage: light skin tone"},
{u8"man getting massage", u8"\U0001F486\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man getting massage: medium-light skin tone", u8"man getting massage: medium-light skin tone"},
{u8"man getting massage", u8"\U0001F486\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man getting massage: medium skin tone", u8"man getting massage: medium skin tone"},
{u8"man getting massage", u8"\U0001F486\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man getting massage: medium-dark skin tone", u8"man getting massage: medium-dark skin tone"},
{u8"man getting massage", u8"\U0001F486\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man getting massage: dark skin tone", u8"man getting massage: dark skin tone"},
{u8"woman getting massage", u8"\U0001F486\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman getting massage: light skin tone", u8"woman getting massage: light skin tone"},
{u8"woman getting massage", u8"\U0001F486\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman getting massage: medium-light skin tone", u8"woman getting massage: medium-light skin tone"},
{u8"woman getting massage", u8"\U0001F486\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman getting massage: medium skin tone", u8"woman getting massage: medium skin tone"},
{u8"woman getting massage", u8"\U0001F486\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman getting massage: medium-dark skin tone", u8"woman getting massage: medium-dark skin tone"},
{u8"woman getting massage", u8"\U0001F486\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman getting massage: dark skin tone", u8"woman getting massage: dark skin tone"},
{u8"person getting haircut", u8"\U0001F487\U0001F3FB", u8"haircut_tone1", u8"person getting haircut: light skin tone"},
{u8"person getting haircut", u8"\U0001F487\U0001F3FC", u8"haircut_tone2", u8"person getting haircut: medium-light skin tone"},
{u8"person getting haircut", u8"\U0001F487\U0001F3FD", u8"haircut_tone3", u8"person getting haircut: medium skin tone"},
{u8"person getting haircut", u8"\U0001F487\U0001F3FE", u8"haircut_tone4", u8"person getting haircut: medium-dark skin tone"},
{u8"person getting haircut", u8"\U0001F487\U0001F3FF", u8"haircut_tone5", u8"person getting haircut: dark skin tone"},
{u8"man getting haircut", u8"\U0001F487\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man getting haircut: light skin tone", u8"man getting haircut: light skin tone"},
{u8"man getting haircut", u8"\U0001F487\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man getting haircut: medium-light skin tone", u8"man getting haircut: medium-light skin tone"},
{u8"man getting haircut", u8"\U0001F487\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man getting haircut: medium skin tone", u8"man getting haircut: medium skin tone"},
{u8"man getting haircut", u8"\U0001F487\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man getting haircut: medium-dark skin tone", u8"man getting haircut: medium-dark skin tone"},
{u8"man getting haircut", u8"\U0001F487\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man getting haircut: dark skin tone", u8"man getting haircut: dark skin tone"},
{u8"woman getting haircut", u8"\U0001F487\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman getting haircut: light skin tone", u8"woman getting haircut: light skin tone"},
{u8"woman getting haircut", u8"\U0001F487\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman getting haircut: medium-light skin tone", u8"woman getting haircut: medium-light skin tone"},
{u8"woman getting haircut", u8"\U0001F487\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman getting haircut: medium skin tone", u8"woman getting haircut: medium skin tone"},
{u8"woman getting haircut", u8"\U0001F487\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman getting haircut: medium-dark skin tone", u8"woman getting haircut: medium-dark skin tone"},
{u8"woman getting haircut", u8"\U0001F487\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman getting haircut: dark skin tone", u8"woman getting haircut: dark skin tone"},
{u8"person walking", u8"\U0001F6B6\U0001F3FB", u8"walking_tone1", u8"person walking: light skin tone"},
{u8"person walking", u8"\U0001F6B6\U0001F3FC", u8"walking_tone2", u8"person walking: medium-light skin tone"},
{u8"person walking", u8"\U0001F6B6\U0001F3FD", u8"walking_tone3", u8"person walking: medium skin tone"},
{u8"person walking", u8"\U0001F6B6\U0001F3FE", u8"walking_tone4", u8"person walking: medium-dark skin tone"},
{u8"person walking", u8"\U0001F6B6\U0001F3FF", u8"walking_tone5", u8"person walking: dark skin tone"},
{u8"man walking", u8"\U0001F6B6\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man walking: light skin tone", u8"man walking: light skin tone"},
{u8"man walking", u8"\U0001F6B6\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man walking: medium-light skin tone", u8"man walking: medium-light skin tone"},
{u8"man walking", u8"\U0001F6B6\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man walking: medium skin tone", u8"man walking: medium skin tone"},
{u8"man walking", u8"\U0001F6B6\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man walking: medium-dark skin tone", u8"man walking: medium-dark skin tone"},
{u8"man walking", u8"\U0001F6B6\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man walking: dark skin tone", u8"man walking: dark skin tone"},
{u8"woman walking", u8"\U0001F6B6\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman walking: light skin tone", u8"woman walking: light skin tone"},
{u8"woman walking", u8"\U0001F6B6\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman walking: medium-light skin tone", u8"woman walking: medium-light skin tone"},
{u8"woman walking", u8"\U0001F6B6\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman walking: medium skin tone", u8"woman walking: medium skin tone"},
{u8"woman walking", u8"\U0001F6B6\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman walking: medium-dark skin tone", u8"woman walking: medium-dark skin tone"},
{u8"woman walking", u8"\U0001F6B6\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman walking: dark skin tone", u8"woman walking: dark skin tone"},
{u8"person standing", u8"\U0001F9CD\U0001F3FB", u8"person standing: light skin tone", u8"person standing: light skin tone"},
{u8"person standing", u8"\U0001F9CD\U0001F3FC", u8"person standing: medium-light skin tone", u8"person standing: medium-light skin tone"},
{u8"person standing", u8"\U0001F9CD\U0001F3FD", u8"person standing: medium skin tone", u8"person standing: medium skin tone"},
{u8"person standing", u8"\U0001F9CD\U0001F3FE", u8"person standing: medium-dark skin tone", u8"person standing: medium-dark skin tone"},
{u8"person standing", u8"\U0001F9CD\U0001F3FF", u8"person standing: dark skin tone", u8"person standing: dark skin tone"},
{u8"man standing", u8"\U0001F9CD\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man standing: light skin tone", u8"man standing: light skin tone"},
{u8"man standing", u8"\U0001F9CD\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man standing: medium-light skin tone", u8"man standing: medium-light skin tone"},
{u8"man standing", u8"\U0001F9CD\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man standing: medium skin tone", u8"man standing: medium skin tone"},
{u8"man standing", u8"\U0001F9CD\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man standing: medium-dark skin tone", u8"man standing: medium-dark skin tone"},
{u8"man standing", u8"\U0001F9CD\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man standing: dark skin tone", u8"man standing: dark skin tone"},
{u8"woman standing", u8"\U0001F9CD\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman standing: light skin tone", u8"woman standing: light skin tone"},
{u8"woman standing", u8"\U0001F9CD\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman standing: medium-light skin tone", u8"woman standing: medium-light skin tone"},
{u8"woman standing", u8"\U0001F9CD\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman standing: medium skin tone", u8"woman standing: medium skin tone"},
{u8"woman standing", u8"\U0001F9CD\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman standing: medium-dark skin tone", u8"woman standing: medium-dark skin tone"},
{u8"woman standing", u8"\U0001F9CD\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman standing: dark skin tone", u8"woman standing: dark skin tone"},
{u8"person kneeling", u8"\U0001F9CE\U0001F3FB", u8"person kneeling: light skin tone", u8"person kneeling: light skin tone"},
{u8"person kneeling", u8"\U0001F9CE\U0001F3FC", u8"person kneeling: medium-light skin tone", u8"person kneeling: medium-light skin tone"},
{u8"person kneeling", u8"\U0001F9CE\U0001F3FD", u8"person kneeling: medium skin tone", u8"person kneeling: medium skin tone"},
{u8"person kneeling", u8"\U0001F9CE\U0001F3FE", u8"person kneeling: medium-dark skin tone", u8"person kneeling: medium-dark skin tone"},
{u8"person kneeling", u8"\U0001F9CE\U0001F3FF", u8"person kneeling: dark skin tone", u8"person kneeling: dark skin tone"},
{u8"man kneeling", u8"\U0001F9CE\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man kneeling: light skin tone", u8"man kneeling: light skin tone"},
{u8"man kneeling", u8"\U0001F9CE\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man kneeling: medium-light skin tone", u8"man kneeling: medium-light skin tone"},
{u8"man kneeling", u8"\U0001F9CE\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man kneeling: medium skin tone", u8"man kneeling: medium skin tone"},
{u8"man kneeling", u8"\U0001F9CE\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man kneeling: medium-dark skin tone", u8"man kneeling: medium-dark skin tone"},
{u8"man kneeling", u8"\U0001F9CE\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man kneeling: dark skin tone", u8"man kneeling: dark skin tone"},
{u8"woman kneeling", u8"\U0001F9CE\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman kneeling: light skin tone", u8"woman kneeling: light skin tone"},
{u8"woman kneeling", u8"\U0001F9CE\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman kneeling: medium-light skin tone", u8"woman kneeling: medium-light skin tone"},
{u8"woman kneeling", u8"\U0001F9CE\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman kneeling: medium skin tone", u8"woman kneeling: medium skin tone"},
{u8"woman kneeling", u8"\U0001F9CE\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman kneeling: medium-dark skin tone", u8"woman kneeling: medium-dark skin tone"},
{u8"woman kneeling", u8"\U0001F9CE\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman kneeling: dark skin tone", u8"woman kneeling: dark skin tone"},
{u8"person with white cane", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9AF", u8"person with white cane: light skin tone", u8"person with white cane: light skin tone"},
{u8"person with white cane", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9AF", u8"person with white cane: medium-light skin tone", u8"person with white cane: medium-light skin tone"},
{u8"person with white cane", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9AF", u8"person with white cane: medium skin tone", u8"person with white cane: medium skin tone"},
{u8"person with white cane", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9AF", u8"person with white cane: medium-dark skin tone", u8"person with white cane: medium-dark skin tone"},
{u8"person with white cane", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9AF", u8"person with white cane: dark skin tone", u8"person with white cane: dark skin tone"},
{u8"man with white cane", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9AF", u8"man with white cane: light skin tone", u8"man with white cane: light skin tone"},
{u8"man with white cane", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9AF", u8"man with white cane: medium-light skin tone", u8"man with white cane: medium-light skin tone"},
{u8"man with white cane", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9AF", u8"man with white cane: medium skin tone", u8"man with white cane: medium skin tone"},
{u8"man with white cane", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9AF", u8"man with white cane: medium-dark skin tone", u8"man with white cane: medium-dark skin tone"},
{u8"man with white cane", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9AF", u8"man with white cane: dark skin tone", u8"man with white cane: dark skin tone"},
{u8"woman with white cane", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9AF", u8"woman with white cane: light skin tone", u8"woman with white cane: light skin tone"},
{u8"woman with white cane", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9AF", u8"woman with white cane: medium-light skin tone", u8"woman with white cane: medium-light skin tone"},
{u8"woman with white cane", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9AF", u8"woman with white cane: medium skin tone", u8"woman with white cane: medium skin tone"},
{u8"woman with white cane", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9AF", u8"woman with white cane: medium-dark skin tone", u8"woman with white cane: medium-dark skin tone"},
{u8"woman with white cane", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9AF", u8"woman with white cane: dark skin tone", u8"woman with white cane: dark skin tone"},
{u8"person in motorized wheelchair", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9BC", u8"person in motorized wheelchair: light skin tone", u8"person in motorized wheelchair: light skin tone"},
{u8"person in motorized wheelchair", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9BC", u8"person in motorized wheelchair: medium-light skin tone", u8"person in motorized wheelchair: medium-light skin tone"},
{u8"person in motorized wheelchair", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9BC", u8"person in motorized wheelchair: medium skin tone", u8"person in motorized wheelchair: medium skin tone"},
{u8"person in motorized wheelchair", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9BC", u8"person in motorized wheelchair: medium-dark skin tone", u8"person in motorized wheelchair: medium-dark skin tone"},
{u8"person in motorized wheelchair", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9BC", u8"person in motorized wheelchair: dark skin tone", u8"person in motorized wheelchair: dark skin tone"},
{u8"man in motorized wheelchair", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9BC", u8"man in motorized wheelchair: light skin tone", u8"man in motorized wheelchair: light skin tone"},
{u8"man in motorized wheelchair", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9BC", u8"man in motorized wheelchair: medium-light skin tone", u8"man in motorized wheelchair: medium-light skin tone"},
{u8"man in motorized wheelchair", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9BC", u8"man in motorized wheelchair: medium skin tone", u8"man in motorized wheelchair: medium skin tone"},
{u8"man in motorized wheelchair", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9BC", u8"man in motorized wheelchair: medium-dark skin tone", u8"man in motorized wheelchair: medium-dark skin tone"},
{u8"man in motorized wheelchair", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9BC", u8"man in motorized wheelchair: dark skin tone", u8"man in motorized wheelchair: dark skin tone"},
{u8"woman in motorized wheelchair", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9BC", u8"woman in motorized wheelchair: light skin tone", u8"woman in motorized wheelchair: light skin tone"},
{u8"woman in motorized wheelchair", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9BC", u8"woman in motorized wheelchair: medium-light skin tone", u8"woman in motorized wheelchair: medium-light skin tone"},
{u8"woman in motorized wheelchair", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9BC", u8"woman in motorized wheelchair: medium skin tone", u8"woman in motorized wheelchair: medium skin tone"},
{u8"woman in motorized wheelchair", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9BC", u8"woman in motorized wheelchair: medium-dark skin tone", u8"woman in motorized wheelchair: medium-dark skin tone"},
{u8"woman in motorized wheelchair", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9BC", u8"woman in motorized wheelchair: dark skin tone", u8"woman in motorized wheelchair: dark skin tone"},
{u8"person in manual wheelchair", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F9BD", u8"person in manual wheelchair: light skin tone", u8"person in manual wheelchair: light skin tone"},
{u8"person in manual wheelchair", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F9BD", u8"person in manual wheelchair: medium-light skin tone", u8"person in manual wheelchair: medium-light skin tone"},
{u8"person in manual wheelchair", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F9BD", u8"person in manual wheelchair: medium skin tone", u8"person in manual wheelchair: medium skin tone"},
{u8"person in manual wheelchair", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F9BD", u8"person in manual wheelchair: medium-dark skin tone", u8"person in manual wheelchair: medium-dark skin tone"},
{u8"person in manual wheelchair", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F9BD", u8"person in manual wheelchair: dark skin tone", u8"person in manual wheelchair: dark skin tone"},
{u8"man in manual wheelchair", u8"\U0001F468\U0001F3FB\U0000200D\U0001F9BD", u8"man in manual wheelchair: light skin tone", u8"man in manual wheelchair: light skin tone"},
{u8"man in manual wheelchair", u8"\U0001F468\U0001F3FC\U0000200D\U0001F9BD", u8"man in manual wheelchair: medium-light skin tone", u8"man in manual wheelchair: medium-light skin tone"},
{u8"man in manual wheelchair", u8"\U0001F468\U0001F3FD\U0000200D\U0001F9BD", u8"man in manual wheelchair: medium skin tone", u8"man in manual wheelchair: medium skin tone"},
{u8"man in manual wheelchair", u8"\U0001F468\U0001F3FE\U0000200D\U0001F9BD", u8"man in manual wheelchair: medium-dark skin tone", u8"man in manual wheelchair: medium-dark skin tone"},
{u8"man in manual wheelchair", u8"\U0001F468\U0001F3FF\U0000200D\U0001F9BD", u8"man in manual wheelchair: dark skin tone", u8"man in manual wheelchair: dark skin tone"},
{u8"woman in manual wheelchair", u8"\U0001F469\U0001F3FB\U0000200D\U0001F9BD", u8"woman in manual wheelchair: light skin tone", u8"woman in manual wheelchair: light skin tone"},
{u8"woman in manual wheelchair", u8"\U0001F469\U0001F3FC\U0000200D\U0001F9BD", u8"woman in manual wheelchair: medium-light skin tone", u8"woman in manual wheelchair: medium-light skin tone"},
{u8"woman in manual wheelchair", u8"\U0001F469\U0001F3FD\U0000200D\U0001F9BD", u8"woman in manual wheelchair: medium skin tone", u8"woman in manual wheelchair: medium skin tone"},
{u8"woman in manual wheelchair", u8"\U0001F469\U0001F3FE\U0000200D\U0001F9BD", u8"woman in manual wheelchair: medium-dark skin tone", u8"woman in manual wheelchair: medium-dark skin tone"},
{u8"woman in manual wheelchair", u8"\U0001F469\U0001F3FF\U0000200D\U0001F9BD", u8"woman in manual wheelchair: dark skin tone", u8"woman in manual wheelchair: dark skin tone"},
{u8"person running", u8"\U0001F3C3\U0001F3FB", u8"runner_tone1", u8"person running: light skin tone"},
{u8"person running", u8"\U0001F3C3\U0001F3FC", u8"runner_tone2", u8"person running: medium-light skin tone"},
{u8"person running", u8"\U0001F3C3\U0001F3FD", u8"runner_tone3", u8"person running: medium skin tone"},
{u8"person running", u8"\U0001F3C3\U0001F3FE", u8"runner_tone4", u8"person running: medium-dark skin tone"},
{u8"person running", u8"\U0001F3C3\U0001F3FF", u8"runner_tone5", u8"person running: dark skin tone"},
{u8"man running", u8"\U0001F3C3\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man running: light skin tone", u8"man running: light skin tone"},
{u8"man running", u8"\U0001F3C3\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man running: medium-light skin tone", u8"man running: medium-light skin tone"},
{u8"man running", u8"\U0001F3C3\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man running: medium skin tone", u8"man running: medium skin tone"},
{u8"man running", u8"\U0001F3C3\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man running: medium-dark skin tone", u8"man running: medium-dark skin tone"},
{u8"man running", u8"\U0001F3C3\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man running: dark skin tone", u8"man running: dark skin tone"},
{u8"woman running", u8"\U0001F3C3\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman running: light skin tone", u8"woman running: light skin tone"},
{u8"woman running", u8"\U0001F3C3\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman running: medium-light skin tone", u8"woman running: medium-light skin tone"},
{u8"woman running", u8"\U0001F3C3\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman running: medium skin tone", u8"woman running: medium skin tone"},
{u8"woman running", u8"\U0001F3C3\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman running: medium-dark skin tone", u8"woman running: medium-dark skin tone"},
{u8"woman running", u8"\U0001F3C3\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman running: dark skin tone", u8"woman running: dark skin tone"},
{u8"woman dancing", u8"\U0001F483\U0001F3FB", u8"dancer_tone1", u8"woman dancing: light skin tone"},
{u8"woman dancing", u8"\U0001F483\U0001F3FC", u8"dancer_tone2", u8"woman dancing: medium-light skin tone"},
{u8"woman dancing", u8"\U0001F483\U0001F3FD", u8"dancer_tone3", u8"woman dancing: medium skin tone"},
{u8"woman dancing", u8"\U0001F483\U0001F3FE", u8"dancer_tone4", u8"woman dancing: medium-dark skin tone"},
{u8"woman dancing", u8"\U0001F483\U0001F3FF", u8"dancer_tone5", u8"woman dancing: dark skin tone"},
{u8"man dancing", u8"\U0001F57A\U0001F3FB", u8"man_dancing_tone1", u8"man dancing: light skin tone"},
{u8"man dancing", u8"\U0001F57A\U0001F3FC", u8"man_dancing_tone2", u8"man dancing: medium-light skin tone"},
{u8"man dancing", u8"\U0001F57A\U0001F3FD", u8"man_dancing_tone3", u8"man dancing: medium skin tone"},
{u8"man dancing", u8"\U0001F57A\U0001F3FE", u8"man_dancing_tone4", u8"man dancing: medium-dark skin tone"},
{u8"man dancing", u8"\U0001F57A\U0001F3FF", u8"man_dancing_tone5", u8"man dancing: dark skin tone"},
{u8"person in suit levitating", u8"\U0001F574\U0001F3FB", u8"levitate_tone1", u8"person in suit levitating: light skin tone"},
{u8"person in suit levitating", u8"\U0001F574\U0001F3FC", u8"levitate_tone2", u8"person in suit levitating: medium-light skin tone"},
{u8"person in suit levitating", u8"\U0001F574\U0001F3FD", u8"levitate_tone3", u8"person in suit levitating: medium skin tone"},
{u8"person in suit levitating", u8"\U0001F574\U0001F3FE", u8"levitate_tone4", u8"person in suit levitating: medium-dark skin tone"},
{u8"person in suit levitating", u8"\U0001F574\U0001F3FF", u8"levitate_tone5", u8"person in suit levitating: dark skin tone"},
{u8"person in steamy room", u8"\U0001F9D6\U0001F3FB", u8"person_in_steamy_room_tone1", u8"person in steamy room: light skin tone"},
{u8"person in steamy room", u8"\U0001F9D6\U0001F3FC", u8"person_in_steamy_room_tone2", u8"person in steamy room: medium-light skin tone"},
{u8"person in steamy room", u8"\U0001F9D6\U0001F3FD", u8"person_in_steamy_room_tone3", u8"person in steamy room: medium skin tone"},
{u8"person in steamy room", u8"\U0001F9D6\U0001F3FE", u8"person_in_steamy_room_tone4", u8"person in steamy room: medium-dark skin tone"},
{u8"person in steamy room", u8"\U0001F9D6\U0001F3FF", u8"person_in_steamy_room_tone5", u8"person in steamy room: dark skin tone"},
{u8"man in steamy room", u8"\U0001F9D6\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man in steamy room: light skin tone", u8"man in steamy room: light skin tone"},
{u8"man in steamy room", u8"\U0001F9D6\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man in steamy room: medium-light skin tone", u8"man in steamy room: medium-light skin tone"},
{u8"man in steamy room", u8"\U0001F9D6\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man in steamy room: medium skin tone", u8"man in steamy room: medium skin tone"},
{u8"man in steamy room", u8"\U0001F9D6\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man in steamy room: medium-dark skin tone", u8"man in steamy room: medium-dark skin tone"},
{u8"man in steamy room", u8"\U0001F9D6\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man in steamy room: dark skin tone", u8"man in steamy room: dark skin tone"},
{u8"woman in steamy room", u8"\U0001F9D6\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman in steamy room: light skin tone", u8"woman in steamy room: light skin tone"},
{u8"woman in steamy room", u8"\U0001F9D6\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman in steamy room: medium-light skin tone", u8"woman in steamy room: medium-light skin tone"},
{u8"woman in steamy room", u8"\U0001F9D6\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman in steamy room: medium skin tone", u8"woman in steamy room: medium skin tone"},
{u8"woman in steamy room", u8"\U0001F9D6\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman in steamy room: medium-dark skin tone", u8"woman in steamy room: medium-dark skin tone"},
{u8"woman in steamy room", u8"\U0001F9D6\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman in steamy room: dark skin tone", u8"woman in steamy room: dark skin tone"},
{u8"person climbing", u8"\U0001F9D7\U0001F3FB", u8"person_climbing_tone1", u8"person climbing: light skin tone"},
{u8"person climbing", u8"\U0001F9D7\U0001F3FC", u8"person_climbing_tone2", u8"person climbing: medium-light skin tone"},
{u8"person climbing", u8"\U0001F9D7\U0001F3FD", u8"person_climbing_tone3", u8"person climbing: medium skin tone"},
{u8"person climbing", u8"\U0001F9D7\U0001F3FE", u8"person_climbing_tone4", u8"person climbing: medium-dark skin tone"},
{u8"person climbing", u8"\U0001F9D7\U0001F3FF", u8"person_climbing_tone5", u8"person climbing: dark skin tone"},
{u8"man climbing", u8"\U0001F9D7\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man climbing: light skin tone", u8"man climbing: light skin tone"},
{u8"man climbing", u8"\U0001F9D7\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man climbing: medium-light skin tone", u8"man climbing: medium-light skin tone"},
{u8"man climbing", u8"\U0001F9D7\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man climbing: medium skin tone", u8"man climbing: medium skin tone"},
{u8"man climbing", u8"\U0001F9D7\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man climbing: medium-dark skin tone", u8"man climbing: medium-dark skin tone"},
{u8"man climbing", u8"\U0001F9D7\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man climbing: dark skin tone", u8"man climbing: dark skin tone"},
{u8"woman climbing", u8"\U0001F9D7\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman climbing: light skin tone", u8"woman climbing: light skin tone"},
{u8"woman climbing", u8"\U0001F9D7\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman climbing: medium-light skin tone", u8"woman climbing: medium-light skin tone"},
{u8"woman climbing", u8"\U0001F9D7\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman climbing: medium skin tone", u8"woman climbing: medium skin tone"},
{u8"woman climbing", u8"\U0001F9D7\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman climbing: medium-dark skin tone", u8"woman climbing: medium-dark skin tone"},
{u8"woman climbing", u8"\U0001F9D7\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman climbing: dark skin tone", u8"woman climbing: dark skin tone"},
{u8"horse racing", u8"\U0001F3C7\U0001F3FB", u8"horse_racing_tone1", u8"horse racing: light skin tone"},
{u8"horse racing", u8"\U0001F3C7\U0001F3FC", u8"horse_racing_tone2", u8"horse racing: medium-light skin tone"},
{u8"horse racing", u8"\U0001F3C7\U0001F3FD", u8"horse_racing_tone3", u8"horse racing: medium skin tone"},
{u8"horse racing", u8"\U0001F3C7\U0001F3FE", u8"horse_racing_tone4", u8"horse racing: medium-dark skin tone"},
{u8"horse racing", u8"\U0001F3C7\U0001F3FF", u8"horse_racing_tone5", u8"horse racing: dark skin tone"},
{u8"snowboarder", u8"\U0001F3C2\U0001F3FB", u8"snowboarder_tone1", u8"snowboarder: light skin tone"},
{u8"snowboarder", u8"\U0001F3C2\U0001F3FC", u8"snowboarder_tone2", u8"snowboarder: medium-light skin tone"},
{u8"snowboarder", u8"\U0001F3C2\U0001F3FD", u8"snowboarder_tone3", u8"snowboarder: medium skin tone"},
{u8"snowboarder", u8"\U0001F3C2\U0001F3FE", u8"snowboarder_tone4", u8"snowboarder: medium-dark skin tone"},
{u8"snowboarder", u8"\U0001F3C2\U0001F3FF", u8"snowboarder_tone5", u8"snowboarder: dark skin tone"},
{u8"person golfing", u8"\U0001F3CC\U0001F3FB", u8"person_golfing_tone1", u8"person golfing: light skin tone"},
{u8"person golfing", u8"\U0001F3CC\U0001F3FC", u8"person_golfing_tone2", u8"person golfing: medium-light skin tone"},
{u8"person golfing", u8"\U0001F3CC\U0001F3FD", u8"person_golfing_tone3", u8"person golfing: medium skin tone"},
{u8"person golfing", u8"\U0001F3CC\U0001F3FE", u8"person_golfing_tone4", u8"person golfing: medium-dark skin tone"},
{u8"person golfing", u8"\U0001F3CC\U0001F3FF", u8"person_golfing_tone5", u8"person golfing: dark skin tone"},
{u8"man golfing", u8"\U0001F3CC\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man golfing: light skin tone", u8"man golfing: light skin tone"},
{u8"man golfing", u8"\U0001F3CC\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man golfing: medium-light skin tone", u8"man golfing: medium-light skin tone"},
{u8"man golfing", u8"\U0001F3CC\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man golfing: medium skin tone", u8"man golfing: medium skin tone"},
{u8"man golfing", u8"\U0001F3CC\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man golfing: medium-dark skin tone", u8"man golfing: medium-dark skin tone"},
{u8"man golfing", u8"\U0001F3CC\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man golfing: dark skin tone", u8"man golfing: dark skin tone"},
{u8"woman golfing", u8"\U0001F3CC\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman golfing: light skin tone", u8"woman golfing: light skin tone"},
{u8"woman golfing", u8"\U0001F3CC\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman golfing: medium-light skin tone", u8"woman golfing: medium-light skin tone"},
{u8"woman golfing", u8"\U0001F3CC\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman golfing: medium skin tone", u8"woman golfing: medium skin tone"},
{u8"woman golfing", u8"\U0001F3CC\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman golfing: medium-dark skin tone", u8"woman golfing: medium-dark skin tone"},
{u8"woman golfing", u8"\U0001F3CC\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman golfing: dark skin tone", u8"woman golfing: dark skin tone"},
{u8"person surfing", u8"\U0001F3C4\U0001F3FB", u8"surfer_tone1", u8"person surfing: light skin tone"},
{u8"person surfing", u8"\U0001F3C4\U0001F3FC", u8"surfer_tone2", u8"person surfing: medium-light skin tone"},
{u8"person surfing", u8"\U0001F3C4\U0001F3FD", u8"surfer_tone3", u8"person surfing: medium skin tone"},
{u8"person surfing", u8"\U0001F3C4\U0001F3FE", u8"surfer_tone4", u8"person surfing: medium-dark skin tone"},
{u8"person surfing", u8"\U0001F3C4\U0001F3FF", u8"surfer_tone5", u8"person surfing: dark skin tone"},
{u8"man surfing", u8"\U0001F3C4\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man surfing: light skin tone", u8"man surfing: light skin tone"},
{u8"man surfing", u8"\U0001F3C4\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man surfing: medium-light skin tone", u8"man surfing: medium-light skin tone"},
{u8"man surfing", u8"\U0001F3C4\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man surfing: medium skin tone", u8"man surfing: medium skin tone"},
{u8"man surfing", u8"\U0001F3C4\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man surfing: medium-dark skin tone", u8"man surfing: medium-dark skin tone"},
{u8"man surfing", u8"\U0001F3C4\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man surfing: dark skin tone", u8"man surfing: dark skin tone"},
{u8"woman surfing", u8"\U0001F3C4\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman surfing: light skin tone", u8"woman surfing: light skin tone"},
{u8"woman surfing", u8"\U0001F3C4\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman surfing: medium-light skin tone", u8"woman surfing: medium-light skin tone"},
{u8"woman surfing", u8"\U0001F3C4\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman surfing: medium skin tone", u8"woman surfing: medium skin tone"},
{u8"woman surfing", u8"\U0001F3C4\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman surfing: medium-dark skin tone", u8"woman surfing: medium-dark skin tone"},
{u8"woman surfing", u8"\U0001F3C4\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman surfing: dark skin tone", u8"woman surfing: dark skin tone"},
{u8"person rowing boat", u8"\U0001F6A3\U0001F3FB", u8"rowboat_tone1", u8"person rowing boat: light skin tone"},
{u8"person rowing boat", u8"\U0001F6A3\U0001F3FC", u8"rowboat_tone2", u8"person rowing boat: medium-light skin tone"},
{u8"person rowing boat", u8"\U0001F6A3\U0001F3FD", u8"rowboat_tone3", u8"person rowing boat: medium skin tone"},
{u8"person rowing boat", u8"\U0001F6A3\U0001F3FE", u8"rowboat_tone4", u8"person rowing boat: medium-dark skin tone"},
{u8"person rowing boat", u8"\U0001F6A3\U0001F3FF", u8"rowboat_tone5", u8"person rowing boat: dark skin tone"},
{u8"man rowing boat", u8"\U0001F6A3\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man rowing boat: light skin tone", u8"man rowing boat: light skin tone"},
{u8"man rowing boat", u8"\U0001F6A3\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man rowing boat: medium-light skin tone", u8"man rowing boat: medium-light skin tone"},
{u8"man rowing boat", u8"\U0001F6A3\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man rowing boat: medium skin tone", u8"man rowing boat: medium skin tone"},
{u8"man rowing boat", u8"\U0001F6A3\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man rowing boat: medium-dark skin tone", u8"man rowing boat: medium-dark skin tone"},
{u8"man rowing boat", u8"\U0001F6A3\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man rowing boat: dark skin tone", u8"man rowing boat: dark skin tone"},
{u8"woman rowing boat", u8"\U0001F6A3\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman rowing boat: light skin tone", u8"woman rowing boat: light skin tone"},
{u8"woman rowing boat", u8"\U0001F6A3\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman rowing boat: medium-light skin tone", u8"woman rowing boat: medium-light skin tone"},
{u8"woman rowing boat", u8"\U0001F6A3\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman rowing boat: medium skin tone", u8"woman rowing boat: medium skin tone"},
{u8"woman rowing boat", u8"\U0001F6A3\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman rowing boat: medium-dark skin tone", u8"woman rowing boat: medium-dark skin tone"},
{u8"woman rowing boat", u8"\U0001F6A3\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman rowing boat: dark skin tone", u8"woman rowing boat: dark skin tone"},
{u8"person swimming", u8"\U0001F3CA\U0001F3FB", u8"swimmer_tone1", u8"person swimming: light skin tone"},
{u8"person swimming", u8"\U0001F3CA\U0001F3FC", u8"swimmer_tone2", u8"person swimming: medium-light skin tone"},
{u8"person swimming", u8"\U0001F3CA\U0001F3FD", u8"swimmer_tone3", u8"person swimming: medium skin tone"},
{u8"person swimming", u8"\U0001F3CA\U0001F3FE", u8"swimmer_tone4", u8"person swimming: medium-dark skin tone"},
{u8"person swimming", u8"\U0001F3CA\U0001F3FF", u8"swimmer_tone5", u8"person swimming: dark skin tone"},
{u8"man swimming", u8"\U0001F3CA\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man swimming: light skin tone", u8"man swimming: light skin tone"},
{u8"man swimming", u8"\U0001F3CA\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man swimming: medium-light skin tone", u8"man swimming: medium-light skin tone"},
{u8"man swimming", u8"\U0001F3CA\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man swimming: medium skin tone", u8"man swimming: medium skin tone"},
{u8"man swimming", u8"\U0001F3CA\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man swimming: medium-dark skin tone", u8"man swimming: medium-dark skin tone"},
{u8"man swimming", u8"\U0001F3CA\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man swimming: dark skin tone", u8"man swimming: dark skin tone"},
{u8"woman swimming", u8"\U0001F3CA\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman swimming: light skin tone", u8"woman swimming: light skin tone"},
{u8"woman swimming", u8"\U0001F3CA\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman swimming: medium-light skin tone", u8"woman swimming: medium-light skin tone"},
{u8"woman swimming", u8"\U0001F3CA\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman swimming: medium skin tone", u8"woman swimming: medium skin tone"},
{u8"woman swimming", u8"\U0001F3CA\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman swimming: medium-dark skin tone", u8"woman swimming: medium-dark skin tone"},
{u8"woman swimming", u8"\U0001F3CA\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman swimming: dark skin tone", u8"woman swimming: dark skin tone"},
{u8"person bouncing ball", u8"\U000026F9\U0001F3FB", u8"basketball_player_tone1", u8"person bouncing ball: light skin tone"},
{u8"person bouncing ball", u8"\U000026F9\U0001F3FC", u8"basketball_player_tone2", u8"person bouncing ball: medium-light skin tone"},
{u8"person bouncing ball", u8"\U000026F9\U0001F3FD", u8"basketball_player_tone3", u8"person bouncing ball: medium skin tone"},
{u8"person bouncing ball", u8"\U000026F9\U0001F3FE", u8"basketball_player_tone4", u8"person bouncing ball: medium-dark skin tone"},
{u8"person bouncing ball", u8"\U000026F9\U0001F3FF", u8"basketball_player_tone5", u8"person bouncing ball: dark skin tone"},
{u8"man bouncing ball", u8"\U000026F9\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man bouncing ball: light skin tone", u8"man bouncing ball: light skin tone"},
{u8"man bouncing ball", u8"\U000026F9\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man bouncing ball: medium-light skin tone", u8"man bouncing ball: medium-light skin tone"},
{u8"man bouncing ball", u8"\U000026F9\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man bouncing ball: medium skin tone", u8"man bouncing ball: medium skin tone"},
{u8"man bouncing ball", u8"\U000026F9\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man bouncing ball: medium-dark skin tone", u8"man bouncing ball: medium-dark skin tone"},
{u8"man bouncing ball", u8"\U000026F9\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man bouncing ball: dark skin tone", u8"man bouncing ball: dark skin tone"},
{u8"woman bouncing ball", u8"\U000026F9\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman bouncing ball: light skin tone", u8"woman bouncing ball: light skin tone"},
{u8"woman bouncing ball", u8"\U000026F9\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman bouncing ball: medium-light skin tone", u8"woman bouncing ball: medium-light skin tone"},
{u8"woman bouncing ball", u8"\U000026F9\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman bouncing ball: medium skin tone", u8"woman bouncing ball: medium skin tone"},
{u8"woman bouncing ball", u8"\U000026F9\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman bouncing ball: medium-dark skin tone", u8"woman bouncing ball: medium-dark skin tone"},
{u8"woman bouncing ball", u8"\U000026F9\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman bouncing ball: dark skin tone", u8"woman bouncing ball: dark skin tone"},
{u8"person lifting weights", u8"\U0001F3CB\U0001F3FB", u8"lifter_tone1", u8"person lifting weights: light skin tone"},
{u8"person lifting weights", u8"\U0001F3CB\U0001F3FC", u8"lifter_tone2", u8"person lifting weights: medium-light skin tone"},
{u8"person lifting weights", u8"\U0001F3CB\U0001F3FD", u8"lifter_tone3", u8"person lifting weights: medium skin tone"},
{u8"person lifting weights", u8"\U0001F3CB\U0001F3FE", u8"lifter_tone4", u8"person lifting weights: medium-dark skin tone"},
{u8"person lifting weights", u8"\U0001F3CB\U0001F3FF", u8"lifter_tone5", u8"person lifting weights: dark skin tone"},
{u8"man lifting weights", u8"\U0001F3CB\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man lifting weights: light skin tone", u8"man lifting weights: light skin tone"},
{u8"man lifting weights", u8"\U0001F3CB\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man lifting weights: medium-light skin tone", u8"man lifting weights: medium-light skin tone"},
{u8"man lifting weights", u8"\U0001F3CB\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man lifting weights: medium skin tone", u8"man lifting weights: medium skin tone"},
{u8"man lifting weights", u8"\U0001F3CB\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man lifting weights: medium-dark skin tone", u8"man lifting weights: medium-dark skin tone"},
{u8"man lifting weights", u8"\U0001F3CB\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man lifting weights: dark skin tone", u8"man lifting weights: dark skin tone"},
{u8"woman lifting weights", u8"\U0001F3CB\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman lifting weights: light skin tone", u8"woman lifting weights: light skin tone"},
{u8"woman lifting weights", u8"\U0001F3CB\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman lifting weights: medium-light skin tone", u8"woman lifting weights: medium-light skin tone"},
{u8"woman lifting weights", u8"\U0001F3CB\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman lifting weights: medium skin tone", u8"woman lifting weights: medium skin tone"},
{u8"woman lifting weights", u8"\U0001F3CB\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman lifting weights: medium-dark skin tone", u8"woman lifting weights: medium-dark skin tone"},
{u8"woman lifting weights", u8"\U0001F3CB\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman lifting weights: dark skin tone", u8"woman lifting weights: dark skin tone"},
{u8"person biking", u8"\U0001F6B4\U0001F3FB", u8"bicyclist_tone1", u8"person biking: light skin tone"},
{u8"person biking", u8"\U0001F6B4\U0001F3FC", u8"bicyclist_tone2", u8"person biking: medium-light skin tone"},
{u8"person biking", u8"\U0001F6B4\U0001F3FD", u8"bicyclist_tone3", u8"person biking: medium skin tone"},
{u8"person biking", u8"\U0001F6B4\U0001F3FE", u8"bicyclist_tone4", u8"person biking: medium-dark skin tone"},
{u8"person biking", u8"\U0001F6B4\U0001F3FF", u8"bicyclist_tone5", u8"person biking: dark skin tone"},
{u8"man biking", u8"\U0001F6B4\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man biking: light skin tone", u8"man biking: light skin tone"},
{u8"man biking", u8"\U0001F6B4\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man biking: medium-light skin tone", u8"man biking: medium-light skin tone"},
{u8"man biking", u8"\U0001F6B4\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man biking: medium skin tone", u8"man biking: medium skin tone"},
{u8"man biking", u8"\U0001F6B4\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man biking: medium-dark skin tone", u8"man biking: medium-dark skin tone"},
{u8"man biking", u8"\U0001F6B4\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man biking: dark skin tone", u8"man biking: dark skin tone"},
{u8"woman biking", u8"\U0001F6B4\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman biking: light skin tone", u8"woman biking: light skin tone"},
{u8"woman biking", u8"\U0001F6B4\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman biking: medium-light skin tone", u8"woman biking: medium-light skin tone"},
{u8"woman biking", u8"\U0001F6B4\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman biking: medium skin tone", u8"woman biking: medium skin tone"},
{u8"woman biking", u8"\U0001F6B4\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman biking: medium-dark skin tone", u8"woman biking: medium-dark skin tone"},
{u8"woman biking", u8"\U0001F6B4\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman biking: dark skin tone", u8"woman biking: dark skin tone"},
{u8"person mountain biking", u8"\U0001F6B5\U0001F3FB", u8"mountain_bicyclist_tone1", u8"person mountain biking: light skin tone"},
{u8"person mountain biking", u8"\U0001F6B5\U0001F3FC", u8"mountain_bicyclist_tone2", u8"person mountain biking: medium-light skin tone"},
{u8"person mountain biking", u8"\U0001F6B5\U0001F3FD", u8"mountain_bicyclist_tone3", u8"person mountain biking: medium skin tone"},
{u8"person mountain biking", u8"\U0001F6B5\U0001F3FE", u8"mountain_bicyclist_tone4", u8"person mountain biking: medium-dark skin tone"},
{u8"person mountain biking", u8"\U0001F6B5\U0001F3FF", u8"mountain_bicyclist_tone5", u8"person mountain biking: dark skin tone"},
{u8"man mountain biking", u8"\U0001F6B5\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man mountain biking: light skin tone", u8"man mountain biking: light skin tone"},
{u8"man mountain biking", u8"\U0001F6B5\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man mountain biking: medium-light skin tone", u8"man mountain biking: medium-light skin tone"},
{u8"man mountain biking", u8"\U0001F6B5\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man mountain biking: medium skin tone", u8"man mountain biking: medium skin tone"},
{u8"man mountain biking", u8"\U0001F6B5\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man mountain biking: medium-dark skin tone", u8"man mountain biking: medium-dark skin tone"},
{u8"man mountain biking", u8"\U0001F6B5\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man mountain biking: dark skin tone", u8"man mountain biking: dark skin tone"},
{u8"woman mountain biking", u8"\U0001F6B5\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman mountain biking: light skin tone", u8"woman mountain biking: light skin tone"},
{u8"woman mountain biking", u8"\U0001F6B5\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman mountain biking: medium-light skin tone", u8"woman mountain biking: medium-light skin tone"},
{u8"woman mountain biking", u8"\U0001F6B5\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman mountain biking: medium skin tone", u8"woman mountain biking: medium skin tone"},
{u8"woman mountain biking", u8"\U0001F6B5\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman mountain biking: medium-dark skin tone", u8"woman mountain biking: medium-dark skin tone"},
{u8"woman mountain biking", u8"\U0001F6B5\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman mountain biking: dark skin tone", u8"woman mountain biking: dark skin tone"},
{u8"person cartwheeling", u8"\U0001F938\U0001F3FB", u8"cartwheel_tone1", u8"person cartwheeling: light skin tone"},
{u8"person cartwheeling", u8"\U0001F938\U0001F3FC", u8"cartwheel_tone2", u8"person cartwheeling: medium-light skin tone"},
{u8"person cartwheeling", u8"\U0001F938\U0001F3FD", u8"cartwheel_tone3", u8"person cartwheeling: medium skin tone"},
{u8"person cartwheeling", u8"\U0001F938\U0001F3FE", u8"cartwheel_tone4", u8"person cartwheeling: medium-dark skin tone"},
{u8"person cartwheeling", u8"\U0001F938\U0001F3FF", u8"cartwheel_tone5", u8"person cartwheeling: dark skin tone"},
{u8"man cartwheeling", u8"\U0001F938\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man cartwheeling: light skin tone", u8"man cartwheeling: light skin tone"},
{u8"man cartwheeling", u8"\U0001F938\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man cartwheeling: medium-light skin tone", u8"man cartwheeling: medium-light skin tone"},
{u8"man cartwheeling", u8"\U0001F938\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man cartwheeling: medium skin tone", u8"man cartwheeling: medium skin tone"},
{u8"man cartwheeling", u8"\U0001F938\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man cartwheeling: medium-dark skin tone", u8"man cartwheeling: medium-dark skin tone"},
{u8"man cartwheeling", u8"\U0001F938\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man cartwheeling: dark skin tone", u8"man cartwheeling: dark skin tone"},
{u8"woman cartwheeling", u8"\U0001F938\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman cartwheeling: light skin tone", u8"woman cartwheeling: light skin tone"},
{u8"woman cartwheeling", u8"\U0001F938\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman cartwheeling: medium-light skin tone", u8"woman cartwheeling: medium-light skin tone"},
{u8"woman cartwheeling", u8"\U0001F938\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman cartwheeling: medium skin tone", u8"woman cartwheeling: medium skin tone"},
{u8"woman cartwheeling", u8"\U0001F938\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman cartwheeling: medium-dark skin tone", u8"woman cartwheeling: medium-dark skin tone"},
{u8"woman cartwheeling", u8"\U0001F938\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman cartwheeling: dark skin tone", u8"woman cartwheeling: dark skin tone"},
{u8"person playing water polo", u8"\U0001F93D\U0001F3FB", u8"water_polo_tone1", u8"person playing water polo: light skin tone"},
{u8"person playing water polo", u8"\U0001F93D\U0001F3FC", u8"water_polo_tone2", u8"person playing water polo: medium-light skin tone"},
{u8"person playing water polo", u8"\U0001F93D\U0001F3FD", u8"water_polo_tone3", u8"person playing water polo: medium skin tone"},
{u8"person playing water polo", u8"\U0001F93D\U0001F3FE", u8"water_polo_tone4", u8"person playing water polo: medium-dark skin tone"},
{u8"person playing water polo", u8"\U0001F93D\U0001F3FF", u8"water_polo_tone5", u8"person playing water polo: dark skin tone"},
{u8"man playing water polo", u8"\U0001F93D\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man playing water polo: light skin tone", u8"man playing water polo: light skin tone"},
{u8"man playing water polo", u8"\U0001F93D\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man playing water polo: medium-light skin tone", u8"man playing water polo: medium-light skin tone"},
{u8"man playing water polo", u8"\U0001F93D\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man playing water polo: medium skin tone", u8"man playing water polo: medium skin tone"},
{u8"man playing water polo", u8"\U0001F93D\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man playing water polo: medium-dark skin tone", u8"man playing water polo: medium-dark skin tone"},
{u8"man playing water polo", u8"\U0001F93D\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man playing water polo: dark skin tone", u8"man playing water polo: dark skin tone"},
{u8"woman playing water polo", u8"\U0001F93D\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman playing water polo: light skin tone", u8"woman playing water polo: light skin tone"},
{u8"woman playing water polo", u8"\U0001F93D\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman playing water polo: medium-light skin tone", u8"woman playing water polo: medium-light skin tone"},
{u8"woman playing water polo", u8"\U0001F93D\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman playing water polo: medium skin tone", u8"woman playing water polo: medium skin tone"},
{u8"woman playing water polo", u8"\U0001F93D\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman playing water polo: medium-dark skin tone", u8"woman playing water polo: medium-dark skin tone"},
{u8"woman playing water polo", u8"\U0001F93D\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman playing water polo: dark skin tone", u8"woman playing water polo: dark skin tone"},
{u8"person playing handball", u8"\U0001F93E\U0001F3FB", u8"handball_tone1", u8"person playing handball: light skin tone"},
{u8"person playing handball", u8"\U0001F93E\U0001F3FC", u8"handball_tone2", u8"person playing handball: medium-light skin tone"},
{u8"person playing handball", u8"\U0001F93E\U0001F3FD", u8"handball_tone3", u8"person playing handball: medium skin tone"},
{u8"person playing handball", u8"\U0001F93E\U0001F3FE", u8"handball_tone4", u8"person playing handball: medium-dark skin tone"},
{u8"person playing handball", u8"\U0001F93E\U0001F3FF", u8"handball_tone5", u8"person playing handball: dark skin tone"},
{u8"man playing handball", u8"\U0001F93E\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man playing handball: light skin tone", u8"man playing handball: light skin tone"},
{u8"man playing handball", u8"\U0001F93E\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man playing handball: medium-light skin tone", u8"man playing handball: medium-light skin tone"},
{u8"man playing handball", u8"\U0001F93E\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man playing handball: medium skin tone", u8"man playing handball: medium skin tone"},
{u8"man playing handball", u8"\U0001F93E\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man playing handball: medium-dark skin tone", u8"man playing handball: medium-dark skin tone"},
{u8"man playing handball", u8"\U0001F93E\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man playing handball: dark skin tone", u8"man playing handball: dark skin tone"},
{u8"woman playing handball", u8"\U0001F93E\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman playing handball: light skin tone", u8"woman playing handball: light skin tone"},
{u8"woman playing handball", u8"\U0001F93E\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman playing handball: medium-light skin tone", u8"woman playing handball: medium-light skin tone"},
{u8"woman playing handball", u8"\U0001F93E\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman playing handball: medium skin tone", u8"woman playing handball: medium skin tone"},
{u8"woman playing handball", u8"\U0001F93E\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman playing handball: medium-dark skin tone", u8"woman playing handball: medium-dark skin tone"},
{u8"woman playing handball", u8"\U0001F93E\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman playing handball: dark skin tone", u8"woman playing handball: dark skin tone"},
{u8"person juggling", u8"\U0001F939\U0001F3FB", u8"juggling_tone1", u8"person juggling: light skin tone"},
{u8"person juggling", u8"\U0001F939\U0001F3FC", u8"juggling_tone2", u8"person juggling: medium-light skin tone"},
{u8"person juggling", u8"\U0001F939\U0001F3FD", u8"juggling_tone3", u8"person juggling: medium skin tone"},
{u8"person juggling", u8"\U0001F939\U0001F3FE", u8"juggling_tone4", u8"person juggling: medium-dark skin tone"},
{u8"person juggling", u8"\U0001F939\U0001F3FF", u8"juggling_tone5", u8"person juggling: dark skin tone"},
{u8"man juggling", u8"\U0001F939\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man juggling: light skin tone", u8"man juggling: light skin tone"},
{u8"man juggling", u8"\U0001F939\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man juggling: medium-light skin tone", u8"man juggling: medium-light skin tone"},
{u8"man juggling", u8"\U0001F939\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man juggling: medium skin tone", u8"man juggling: medium skin tone"},
{u8"man juggling", u8"\U0001F939\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man juggling: medium-dark skin tone", u8"man juggling: medium-dark skin tone"},
{u8"man juggling", u8"\U0001F939\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man juggling: dark skin tone", u8"man juggling: dark skin tone"},
{u8"woman juggling", u8"\U0001F939\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman juggling: light skin tone", u8"woman juggling: light skin tone"},
{u8"woman juggling", u8"\U0001F939\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman juggling: medium-light skin tone", u8"woman juggling: medium-light skin tone"},
{u8"woman juggling", u8"\U0001F939\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman juggling: medium skin tone", u8"woman juggling: medium skin tone"},
{u8"woman juggling", u8"\U0001F939\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman juggling: medium-dark skin tone", u8"woman juggling: medium-dark skin tone"},
{u8"woman juggling", u8"\U0001F939\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman juggling: dark skin tone", u8"woman juggling: dark skin tone"},
{u8"person in lotus position", u8"\U0001F9D8\U0001F3FB", u8"person_in_lotus_position_tone1", u8"person in lotus position: light skin tone"},
{u8"person in lotus position", u8"\U0001F9D8\U0001F3FC", u8"person_in_lotus_position_tone2", u8"person in lotus position: medium-light skin tone"},
{u8"person in lotus position", u8"\U0001F9D8\U0001F3FD", u8"person_in_lotus_position_tone3", u8"person in lotus position: medium skin tone"},
{u8"person in lotus position", u8"\U0001F9D8\U0001F3FE", u8"person_in_lotus_position_tone4", u8"person in lotus position: medium-dark skin tone"},
{u8"person in lotus position", u8"\U0001F9D8\U0001F3FF", u8"person_in_lotus_position_tone5", u8"person in lotus position: dark skin tone"},
{u8"man in lotus position", u8"\U0001F9D8\U0001F3FB\U0000200D\U00002642\U0000FE0F", u8"man in lotus position: light skin tone", u8"man in lotus position: light skin tone"},
{u8"man in lotus position", u8"\U0001F9D8\U0001F3FC\U0000200D\U00002642\U0000FE0F", u8"man in lotus position: medium-light skin tone", u8"man in lotus position: medium-light skin tone"},
{u8"man in lotus position", u8"\U0001F9D8\U0001F3FD\U0000200D\U00002642\U0000FE0F", u8"man in lotus position: medium skin tone", u8"man in lotus position: medium skin tone"},
{u8"man in lotus position", u8"\U0001F9D8\U0001F3FE\U0000200D\U00002642\U0000FE0F", u8"man in lotus position: medium-dark skin tone", u8"man in lotus position: medium-dark skin tone"},
{u8"man in lotus position", u8"\U0001F9D8\U0001F3FF\U0000200D\U00002642\U0000FE0F", u8"man in lotus position: dark skin tone", u8"man in lotus position: dark skin tone"},
{u8"woman in lotus position", u8"\U0001F9D8\U0001F3FB\U0000200D\U00002640\U0000FE0F", u8"woman in lotus position: light skin tone", u8"woman in lotus position: light skin tone"},
{u8"woman in lotus position", u8"\U0001F9D8\U0001F3FC\U0000200D\U00002640\U0000FE0F", u8"woman in lotus position: medium-light skin tone", u8"woman in lotus position: medium-light skin tone"},
{u8"woman in lotus position", u8"\U0001F9D8\U0001F3FD\U0000200D\U00002640\U0000FE0F", u8"woman in lotus position: medium skin tone", u8"woman in lotus position: medium skin tone"},
{u8"woman in lotus position", u8"\U0001F9D8\U0001F3FE\U0000200D\U00002640\U0000FE0F", u8"woman in lotus position: medium-dark skin tone", u8"woman in lotus position: medium-dark skin tone"},
{u8"woman in lotus position", u8"\U0001F9D8\U0001F3FF\U0000200D\U00002640\U0000FE0F", u8"woman in lotus position: dark skin tone", u8"woman in lotus position: dark skin tone"},
{u8"person taking bath", u8"\U0001F6C0\U0001F3FB", u8"bath_tone1", u8"person taking bath: light skin tone"},
{u8"person taking bath", u8"\U0001F6C0\U0001F3FC", u8"bath_tone2", u8"person taking bath: medium-light skin tone"},
{u8"person taking bath", u8"\U0001F6C0\U0001F3FD", u8"bath_tone3", u8"person taking bath: medium skin tone"},
{u8"person taking bath", u8"\U0001F6C0\U0001F3FE", u8"bath_tone4", u8"person taking bath: medium-dark skin tone"},
{u8"person taking bath", u8"\U0001F6C0\U0001F3FF", u8"bath_tone5", u8"person taking bath: dark skin tone"},
{u8"person in bed", u8"\U0001F6CC\U0001F3FB", u8"person_in_bed_tone1", u8"person in bed: light skin tone"},
{u8"person in bed", u8"\U0001F6CC\U0001F3FC", u8"person_in_bed_tone2", u8"person in bed: medium-light skin tone"},
{u8"person in bed", u8"\U0001F6CC\U0001F3FD", u8"person_in_bed_tone3", u8"person in bed: medium skin tone"},
{u8"person in bed", u8"\U0001F6CC\U0001F3FE", u8"person_in_bed_tone4", u8"person in bed: medium-dark skin tone"},
{u8"person in bed", u8"\U0001F6CC\U0001F3FF", u8"person_in_bed_tone5", u8"person in bed: dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB", u8"people holding hands: light skin tone", u8"people holding hands: light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC", u8"people holding hands: light skin tone, medium-light skin tone", u8"people holding hands: light skin tone, medium-light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD", u8"people holding hands: light skin tone, medium skin tone", u8"people holding hands: light skin tone, medium skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE", u8"people holding hands: light skin tone, medium-dark skin tone", u8"people holding hands: light skin tone, medium-dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF", u8"people holding hands: light skin tone, dark skin tone", u8"people holding hands: light skin tone, dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB", u8"people holding hands: medium-light skin tone, light skin tone", u8"people holding hands: medium-light skin tone, light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC", u8"people holding hands: medium-light skin tone", u8"people holding hands: medium-light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD", u8"people holding hands: medium-light skin tone, medium skin tone", u8"people holding hands: medium-light skin tone, medium skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE", u8"people holding hands: medium-light skin tone, medium-dark skin tone", u8"people holding hands: medium-light skin tone, medium-dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF", u8"people holding hands: medium-light skin tone, dark skin tone", u8"people holding hands: medium-light skin tone, dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB", u8"people holding hands: medium skin tone, light skin tone", u8"people holding hands: medium skin tone, light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC", u8"people holding hands: medium skin tone, medium-light skin tone", u8"people holding hands: medium skin tone, medium-light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD", u8"people holding hands: medium skin tone", u8"people holding hands: medium skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE", u8"people holding hands: medium skin tone, medium-dark skin tone", u8"people holding hands: medium skin tone, medium-dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF", u8"people holding hands: medium skin tone, dark skin tone", u8"people holding hands: medium skin tone, dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB", u8"people holding hands: medium-dark skin tone, light skin tone", u8"people holding hands: medium-dark skin tone, light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC", u8"people holding hands: medium-dark skin tone, medium-light skin tone", u8"people holding hands: medium-dark skin tone, medium-light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD", u8"people holding hands: medium-dark skin tone, medium skin tone", u8"people holding hands: medium-dark skin tone, medium skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE", u8"people holding hands: medium-dark skin tone", u8"people holding hands: medium-dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF", u8"people holding hands: medium-dark skin tone, dark skin tone", u8"people holding hands: medium-dark skin tone, dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB", u8"people holding hands: dark skin tone, light skin tone", u8"people holding hands: dark skin tone, light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC", u8"people holding hands: dark skin tone, medium-light skin tone", u8"people holding hands: dark skin tone, medium-light skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD", u8"people holding hands: dark skin tone, medium skin tone", u8"people holding hands: dark skin tone, medium skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE", u8"people holding hands: dark skin tone, medium-dark skin tone", u8"people holding hands: dark skin tone, medium-dark skin tone"},
{u8"people holding hands", u8"\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF", u8"people holding hands: dark skin tone", u8"people holding hands: dark skin tone"},
{u8"women holding hands", u8"\U0001F46D\U0001F3FB", u8"women holding hands: light skin tone", u8"women holding hands: light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC", u8"women holding hands: light skin tone, medium-light skin tone", u8"women holding hands: light skin tone, medium-light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD", u8"women holding hands: light skin tone, medium skin tone", u8"women holding hands: light skin tone, medium skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE", u8"women holding hands: light skin tone, medium-dark skin tone", u8"women holding hands: light skin tone, medium-dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF", u8"women holding hands: light skin tone, dark skin tone", u8"women holding hands: light skin tone, dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB", u8"women holding hands: medium-light skin tone, light skin tone", u8"women holding hands: medium-light skin tone, light skin tone"},
{u8"women holding hands", u8"\U0001F46D\U0001F3FC", u8"women holding hands: medium-light skin tone", u8"women holding hands: medium-light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD", u8"women holding hands: medium-light skin tone, medium skin tone", u8"women holding hands: medium-light skin tone, medium skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE", u8"women holding hands: medium-light skin tone, medium-dark skin tone", u8"women holding hands: medium-light skin tone, medium-dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF", u8"women holding hands: medium-light skin tone, dark skin tone", u8"women holding hands: medium-light skin tone, dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB", u8"women holding hands: medium skin tone, light skin tone", u8"women holding hands: medium skin tone, light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC", u8"women holding hands: medium skin tone, medium-light skin tone", u8"women holding hands: medium skin tone, medium-light skin tone"},
{u8"women holding hands", u8"\U0001F46D\U0001F3FD", u8"women holding hands: medium skin tone", u8"women holding hands: medium skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE", u8"women holding hands: medium skin tone, medium-dark skin tone", u8"women holding hands: medium skin tone, medium-dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF", u8"women holding hands: medium skin tone, dark skin tone", u8"women holding hands: medium skin tone, dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB", u8"women holding hands: medium-dark skin tone, light skin tone", u8"women holding hands: medium-dark skin tone, light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC", u8"women holding hands: medium-dark skin tone, medium-light skin tone", u8"women holding hands: medium-dark skin tone, medium-light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD", u8"women holding hands: medium-dark skin tone, medium skin tone", u8"women holding hands: medium-dark skin tone, medium skin tone"},
{u8"women holding hands", u8"\U0001F46D\U0001F3FE", u8"women holding hands: medium-dark skin tone", u8"women holding hands: medium-dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF", u8"women holding hands: medium-dark skin tone, dark skin tone", u8"women holding hands: medium-dark skin tone, dark skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB", u8"women holding hands: dark skin tone, light skin tone", u8"women holding hands: dark skin tone, light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC", u8"women holding hands: dark skin tone, medium-light skin tone", u8"women holding hands: dark skin tone, medium-light skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD", u8"women holding hands: dark skin tone, medium skin tone", u8"women holding hands: dark skin tone, medium skin tone"},
{u8"women holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE", u8"women holding hands: dark skin tone, medium-dark skin tone", u8"women holding hands: dark skin tone, medium-dark skin tone"},
{u8"women holding hands", u8"\U0001F46D\U0001F3FF", u8"women holding hands: dark skin tone", u8"women holding hands: dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F46B\U0001F3FB", u8"woman and man holding hands: light skin tone", u8"woman and man holding hands: light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"woman and man holding hands: light skin tone, medium-light skin tone", u8"woman and man holding hands: light skin tone, medium-light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"woman and man holding hands: light skin tone, medium skin tone", u8"woman and man holding hands: light skin tone, medium skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"woman and man holding hands: light skin tone, medium-dark skin tone", u8"woman and man holding hands: light skin tone, medium-dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"woman and man holding hands: light skin tone, dark skin tone", u8"woman and man holding hands: light skin tone, dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"woman and man holding hands: medium-light skin tone, light skin tone", u8"woman and man holding hands: medium-light skin tone, light skin tone"},
{u8"woman and man holding hands", u8"\U0001F46B\U0001F3FC", u8"woman and man holding hands: medium-light skin tone", u8"woman and man holding hands: medium-light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"woman and man holding hands: medium-light skin tone, medium skin tone", u8"woman and man holding hands: medium-light skin tone, medium skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"woman and man holding hands: medium-light skin tone, medium-dark skin tone", u8"woman and man holding hands: medium-light skin tone, medium-dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"woman and man holding hands: medium-light skin tone, dark skin tone", u8"woman and man holding hands: medium-light skin tone, dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"woman and man holding hands: medium skin tone, light skin tone", u8"woman and man holding hands: medium skin tone, light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"woman and man holding hands: medium skin tone, medium-light skin tone", u8"woman and man holding hands: medium skin tone, medium-light skin tone"},
{u8"woman and man holding hands", u8"\U0001F46B\U0001F3FD", u8"woman and man holding hands: medium skin tone", u8"woman and man holding hands: medium skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"woman and man holding hands: medium skin tone, medium-dark skin tone", u8"woman and man holding hands: medium skin tone, medium-dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"woman and man holding hands: medium skin tone, dark skin tone", u8"woman and man holding hands: medium skin tone, dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"woman and man holding hands: medium-dark skin tone, light skin tone", u8"woman and man holding hands: medium-dark skin tone, light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"woman and man holding hands: medium-dark skin tone, medium-light skin tone", u8"woman and man holding hands: medium-dark skin tone, medium-light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"woman and man holding hands: medium-dark skin tone, medium skin tone", u8"woman and man holding hands: medium-dark skin tone, medium skin tone"},
{u8"woman and man holding hands", u8"\U0001F46B\U0001F3FE", u8"woman and man holding hands: medium-dark skin tone", u8"woman and man holding hands: medium-dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"woman and man holding hands: medium-dark skin tone, dark skin tone", u8"woman and man holding hands: medium-dark skin tone, dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"woman and man holding hands: dark skin tone, light skin tone", u8"woman and man holding hands: dark skin tone, light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"woman and man holding hands: dark skin tone, medium-light skin tone", u8"woman and man holding hands: dark skin tone, medium-light skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"woman and man holding hands: dark skin tone, medium skin tone", u8"woman and man holding hands: dark skin tone, medium skin tone"},
{u8"woman and man holding hands", u8"\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"woman and man holding hands: dark skin tone, medium-dark skin tone", u8"woman and man holding hands: dark skin tone, medium-dark skin tone"},
{u8"woman and man holding hands", u8"\U0001F46B\U0001F3FF", u8"woman and man holding hands: dark skin tone", u8"woman and man holding hands: dark skin tone"},
{u8"men holding hands", u8"\U0001F46C\U0001F3FB", u8"men holding hands: light skin tone", u8"men holding hands: light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"men holding hands: light skin tone, medium-light skin tone", u8"men holding hands: light skin tone, medium-light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"men holding hands: light skin tone, medium skin tone", u8"men holding hands: light skin tone, medium skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"men holding hands: light skin tone, medium-dark skin tone", u8"men holding hands: light skin tone, medium-dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"men holding hands: light skin tone, dark skin tone", u8"men holding hands: light skin tone, dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"men holding hands: medium-light skin tone, light skin tone", u8"men holding hands: medium-light skin tone, light skin tone"},
{u8"men holding hands", u8"\U0001F46C\U0001F3FC", u8"men holding hands: medium-light skin tone", u8"men holding hands: medium-light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"men holding hands: medium-light skin tone, medium skin tone", u8"men holding hands: medium-light skin tone, medium skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"men holding hands: medium-light skin tone, medium-dark skin tone", u8"men holding hands: medium-light skin tone, medium-dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"men holding hands: medium-light skin tone, dark skin tone", u8"men holding hands: medium-light skin tone, dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"men holding hands: medium skin tone, light skin tone", u8"men holding hands: medium skin tone, light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"men holding hands: medium skin tone, medium-light skin tone", u8"men holding hands: medium skin tone, medium-light skin tone"},
{u8"men holding hands", u8"\U0001F46C\U0001F3FD", u8"men holding hands: medium skin tone", u8"men holding hands: medium skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"men holding hands: medium skin tone, medium-dark skin tone", u8"men holding hands: medium skin tone, medium-dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"men holding hands: medium skin tone, dark skin tone", u8"men holding hands: medium skin tone, dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"men holding hands: medium-dark skin tone, light skin tone", u8"men holding hands: medium-dark skin tone, light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"men holding hands: medium-dark skin tone, medium-light skin tone", u8"men holding hands: medium-dark skin tone, medium-light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"men holding hands: medium-dark skin tone, medium skin tone", u8"men holding hands: medium-dark skin tone, medium skin tone"},
{u8"men holding hands", u8"\U0001F46C\U0001F3FE", u8"men holding hands: medium-dark skin tone", u8"men holding hands: medium-dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF", u8"men holding hands: medium-dark skin tone, dark skin tone", u8"men holding hands: medium-dark skin tone, dark skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB", u8"men holding hands: dark skin tone, light skin tone", u8"men holding hands: dark skin tone, light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC", u8"men holding hands: dark skin tone, medium-light skin tone", u8"men holding hands: dark skin tone, medium-light skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD", u8"men holding hands: dark skin tone, medium skin tone", u8"men holding hands: dark skin tone, medium skin tone"},
{u8"men holding hands", u8"\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE", u8"men holding hands: dark skin tone, medium-dark skin tone", u8"men holding hands: dark skin tone, medium-dark skin tone"},
{u8"men holding hands", u8"\U0001F46C\U0001F3FF", u8"men holding hands: dark skin tone", u8"men holding hands: dark skin tone"},
{u8"kiss", u8"\U0001F48F\U0001F3FB", u8"kiss: light skin tone", u8"kiss: light skin tone"},
{u8"kiss", u8"\U0001F48F\U0001F3FC", u8"kiss: medium-light skin tone", u8"kiss: medium-light skin tone"},
{u8"kiss", u8"\U0001F48F\U0001F3FD", u8"kiss: medium skin tone", u8"kiss: medium skin tone"},
{u8"kiss", u8"\U0001F48F\U0001F3FE", u8"kiss: medium-dark skin tone", u8"kiss: medium-dark skin tone"},
{u8"kiss", u8"\U0001F48F\U0001F3FF", u8"kiss: dark skin tone", u8"kiss: dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC", u8"kiss: person, person, light skin tone, medium-light skin tone", u8"kiss: person, person, light skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD", u8"kiss: person, person, light skin tone, medium skin tone", u8"kiss: person, person, light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE", u8"kiss: person, person, light skin tone, medium-dark skin tone", u8"kiss: person, person, light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF", u8"kiss: person, person, light skin tone, dark skin tone", u8"kiss: person, person, light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB", u8"kiss: person, person, medium-light skin tone, light skin tone", u8"kiss: person, person, medium-light skin tone, light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD", u8"kiss: person, person, medium-light skin tone, medium skin tone", u8"kiss: person, person, medium-light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE", u8"kiss: person, person, medium-light skin tone, medium-dark skin tone", u8"kiss: person, person, medium-light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF", u8"kiss: person, person, medium-light skin tone, dark skin tone", u8"kiss: person, person, medium-light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB", u8"kiss: person, person, medium skin tone, light skin tone", u8"kiss: person, person, medium skin tone, light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC", u8"kiss: person, person, medium skin tone, medium-light skin tone", u8"kiss: person, person, medium skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE", u8"kiss: person, person, medium skin tone, medium-dark skin tone", u8"kiss: person, person, medium skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF", u8"kiss: person, person, medium skin tone, dark skin tone", u8"kiss: person, person, medium skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB", u8"kiss: person, person, medium-dark skin tone, light skin tone", u8"kiss: person, person, medium-dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC", u8"kiss: person, person, medium-dark skin tone, medium-light skin tone", u8"kiss: person, person, medium-dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD", u8"kiss: person, person, medium-dark skin tone, medium skin tone", u8"kiss: person, person, medium-dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF", u8"kiss: person, person, medium-dark skin tone, dark skin tone", u8"kiss: person, person, medium-dark skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB", u8"kiss: person, person, dark skin tone, light skin tone", u8"kiss: person, person, dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC", u8"kiss: person, person, dark skin tone, medium-light skin tone", u8"kiss: person, person, dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD", u8"kiss: person, person, dark skin tone, medium skin tone", u8"kiss: person, person, dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE", u8"kiss: person, person, dark skin tone, medium-dark skin tone", u8"kiss: person, person, dark skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: woman, man, light skin tone", u8"kiss: woman, man, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: woman, man, light skin tone, medium-light skin tone", u8"kiss: woman, man, light skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: woman, man, light skin tone, medium skin tone", u8"kiss: woman, man, light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: woman, man, light skin tone, medium-dark skin tone", u8"kiss: woman, man, light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: woman, man, light skin tone, dark skin tone", u8"kiss: woman, man, light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: woman, man, medium-light skin tone, light skin tone", u8"kiss: woman, man, medium-light skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: woman, man, medium-light skin tone", u8"kiss: woman, man, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: woman, man, medium-light skin tone, medium skin tone", u8"kiss: woman, man, medium-light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: woman, man, medium-light skin tone, medium-dark skin tone", u8"kiss: woman, man, medium-light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: woman, man, medium-light skin tone, dark skin tone", u8"kiss: woman, man, medium-light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: woman, man, medium skin tone, light skin tone", u8"kiss: woman, man, medium skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: woman, man, medium skin tone, medium-light skin tone", u8"kiss: woman, man, medium skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: woman, man, medium skin tone", u8"kiss: woman, man, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: woman, man, medium skin tone, medium-dark skin tone", u8"kiss: woman, man, medium skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: woman, man, medium skin tone, dark skin tone", u8"kiss: woman, man, medium skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: woman, man, medium-dark skin tone, light skin tone", u8"kiss: woman, man, medium-dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: woman, man, medium-dark skin tone, medium-light skin tone", u8"kiss: woman, man, medium-dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: woman, man, medium-dark skin tone, medium skin tone", u8"kiss: woman, man, medium-dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: woman, man, medium-dark skin tone", u8"kiss: woman, man, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: woman, man, medium-dark skin tone, dark skin tone", u8"kiss: woman, man, medium-dark skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: woman, man, dark skin tone, light skin tone", u8"kiss: woman, man, dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: woman, man, dark skin tone, medium-light skin tone", u8"kiss: woman, man, dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: woman, man, dark skin tone, medium skin tone", u8"kiss: woman, man, dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: woman, man, dark skin tone, medium-dark skin tone", u8"kiss: woman, man, dark skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: woman, man, dark skin tone", u8"kiss: woman, man, dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: man, man, light skin tone", u8"kiss: man, man, light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: man, man, light skin tone, medium-light skin tone", u8"kiss: man, man, light skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: man, man, light skin tone, medium skin tone", u8"kiss: man, man, light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: man, man, light skin tone, medium-dark skin tone", u8"kiss: man, man, light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: man, man, light skin tone, dark skin tone", u8"kiss: man, man, light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: man, man, medium-light skin tone, light skin tone", u8"kiss: man, man, medium-light skin tone, light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: man, man, medium-light skin tone", u8"kiss: man, man, medium-light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: man, man, medium-light skin tone, medium skin tone", u8"kiss: man, man, medium-light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: man, man, medium-light skin tone, medium-dark skin tone", u8"kiss: man, man, medium-light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: man, man, medium-light skin tone, dark skin tone", u8"kiss: man, man, medium-light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: man, man, medium skin tone, light skin tone", u8"kiss: man, man, medium skin tone, light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: man, man, medium skin tone, medium-light skin tone", u8"kiss: man, man, medium skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: man, man, medium skin tone", u8"kiss: man, man, medium skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: man, man, medium skin tone, medium-dark skin tone", u8"kiss: man, man, medium skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: man, man, medium skin tone, dark skin tone", u8"kiss: man, man, medium skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: man, man, medium-dark skin tone, light skin tone", u8"kiss: man, man, medium-dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: man, man, medium-dark skin tone, medium-light skin tone", u8"kiss: man, man, medium-dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: man, man, medium-dark skin tone, medium skin tone", u8"kiss: man, man, medium-dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: man, man, medium-dark skin tone", u8"kiss: man, man, medium-dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: man, man, medium-dark skin tone, dark skin tone", u8"kiss: man, man, medium-dark skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB", u8"kiss: man, man, dark skin tone, light skin tone", u8"kiss: man, man, dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC", u8"kiss: man, man, dark skin tone, medium-light skin tone", u8"kiss: man, man, dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD", u8"kiss: man, man, dark skin tone, medium skin tone", u8"kiss: man, man, dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE", u8"kiss: man, man, dark skin tone, medium-dark skin tone", u8"kiss: man, man, dark skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF", u8"kiss: man, man, dark skin tone", u8"kiss: man, man, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB", u8"kiss: woman, woman, light skin tone", u8"kiss: woman, woman, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC", u8"kiss: woman, woman, light skin tone, medium-light skin tone", u8"kiss: woman, woman, light skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD", u8"kiss: woman, woman, light skin tone, medium skin tone", u8"kiss: woman, woman, light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE", u8"kiss: woman, woman, light skin tone, medium-dark skin tone", u8"kiss: woman, woman, light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF", u8"kiss: woman, woman, light skin tone, dark skin tone", u8"kiss: woman, woman, light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB", u8"kiss: woman, woman, medium-light skin tone, light skin tone", u8"kiss: woman, woman, medium-light skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC", u8"kiss: woman, woman, medium-light skin tone", u8"kiss: woman, woman, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD", u8"kiss: woman, woman, medium-light skin tone, medium skin tone", u8"kiss: woman, woman, medium-light skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE", u8"kiss: woman, woman, medium-light skin tone, medium-dark skin tone", u8"kiss: woman, woman, medium-light skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF", u8"kiss: woman, woman, medium-light skin tone, dark skin tone", u8"kiss: woman, woman, medium-light skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB", u8"kiss: woman, woman, medium skin tone, light skin tone", u8"kiss: woman, woman, medium skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC", u8"kiss: woman, woman, medium skin tone, medium-light skin tone", u8"kiss: woman, woman, medium skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD", u8"kiss: woman, woman, medium skin tone", u8"kiss: woman, woman, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE", u8"kiss: woman, woman, medium skin tone, medium-dark skin tone", u8"kiss: woman, woman, medium skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF", u8"kiss: woman, woman, medium skin tone, dark skin tone", u8"kiss: woman, woman, medium skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB", u8"kiss: woman, woman, medium-dark skin tone, light skin tone", u8"kiss: woman, woman, medium-dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC", u8"kiss: woman, woman, medium-dark skin tone, medium-light skin tone", u8"kiss: woman, woman, medium-dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD", u8"kiss: woman, woman, medium-dark skin tone, medium skin tone", u8"kiss: woman, woman, medium-dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE", u8"kiss: woman, woman, medium-dark skin tone", u8"kiss: woman, woman, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF", u8"kiss: woman, woman, medium-dark skin tone, dark skin tone", u8"kiss: woman, woman, medium-dark skin tone, dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB", u8"kiss: woman, woman, dark skin tone, light skin tone", u8"kiss: woman, woman, dark skin tone, light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC", u8"kiss: woman, woman, dark skin tone, medium-light skin tone", u8"kiss: woman, woman, dark skin tone, medium-light skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD", u8"kiss: woman, woman, dark skin tone, medium skin tone", u8"kiss: woman, woman, dark skin tone, medium skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE", u8"kiss: woman, woman, dark skin tone, medium-dark skin tone", u8"kiss: woman, woman, dark skin tone, medium-dark skin tone"},
{u8"kiss", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF", u8"kiss: woman, woman, dark skin tone", u8"kiss: woman, woman, dark skin tone"},
{u8"couple with heart", u8"\U0001F491\U0001F3FB", u8"couple with heart: light skin tone", u8"couple with heart: light skin tone"},
{u8"couple with heart", u8"\U0001F491\U0001F3FC", u8"couple with heart: medium-light skin tone", u8"couple with heart: medium-light skin tone"},
{u8"couple with heart", u8"\U0001F491\U0001F3FD", u8"couple with heart: medium skin tone", u8"couple with heart: medium skin tone"},
{u8"couple with heart", u8"\U0001F491\U0001F3FE", u8"couple with heart: medium-dark skin tone", u8"couple with heart: medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F491\U0001F3FF", u8"couple with heart: dark skin tone", u8"couple with heart: dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC", u8"couple with heart: person, person, light skin tone, medium-light skin tone", u8"couple with heart: person, person, light skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD", u8"couple with heart: person, person, light skin tone, medium skin tone", u8"couple with heart: person, person, light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE", u8"couple with heart: person, person, light skin tone, medium-dark skin tone", u8"couple with heart: person, person, light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF", u8"couple with heart: person, person, light skin tone, dark skin tone", u8"couple with heart: person, person, light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB", u8"couple with heart: person, person, medium-light skin tone, light skin tone", u8"couple with heart: person, person, medium-light skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD", u8"couple with heart: person, person, medium-light skin tone, medium skin tone", u8"couple with heart: person, person, medium-light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE", u8"couple with heart: person, person, medium-light skin tone, medium-dark skin tone", u8"couple with heart: person, person, medium-light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF", u8"couple with heart: person, person, medium-light skin tone, dark skin tone", u8"couple with heart: person, person, medium-light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB", u8"couple with heart: person, person, medium skin tone, light skin tone", u8"couple with heart: person, person, medium skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC", u8"couple with heart: person, person, medium skin tone, medium-light skin tone", u8"couple with heart: person, person, medium skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE", u8"couple with heart: person, person, medium skin tone, medium-dark skin tone", u8"couple with heart: person, person, medium skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF", u8"couple with heart: person, person, medium skin tone, dark skin tone", u8"couple with heart: person, person, medium skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB", u8"couple with heart: person, person, medium-dark skin tone, light skin tone", u8"couple with heart: person, person, medium-dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC", u8"couple with heart: person, person, medium-dark skin tone, medium-light skin tone", u8"couple with heart: person, person, medium-dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD", u8"couple with heart: person, person, medium-dark skin tone, medium skin tone", u8"couple with heart: person, person, medium-dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF", u8"couple with heart: person, person, medium-dark skin tone, dark skin tone", u8"couple with heart: person, person, medium-dark skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB", u8"couple with heart: person, person, dark skin tone, light skin tone", u8"couple with heart: person, person, dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC", u8"couple with heart: person, person, dark skin tone, medium-light skin tone", u8"couple with heart: person, person, dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD", u8"couple with heart: person, person, dark skin tone, medium skin tone", u8"couple with heart: person, person, dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE", u8"couple with heart: person, person, dark skin tone, medium-dark skin tone", u8"couple with heart: person, person, dark skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: woman, man, light skin tone", u8"couple with heart: woman, man, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: woman, man, light skin tone, medium-light skin tone", u8"couple with heart: woman, man, light skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: woman, man, light skin tone, medium skin tone", u8"couple with heart: woman, man, light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: woman, man, light skin tone, medium-dark skin tone", u8"couple with heart: woman, man, light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: woman, man, light skin tone, dark skin tone", u8"couple with heart: woman, man, light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: woman, man, medium-light skin tone, light skin tone", u8"couple with heart: woman, man, medium-light skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: woman, man, medium-light skin tone", u8"couple with heart: woman, man, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: woman, man, medium-light skin tone, medium skin tone", u8"couple with heart: woman, man, medium-light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: woman, man, medium-light skin tone, medium-dark skin tone", u8"couple with heart: woman, man, medium-light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: woman, man, medium-light skin tone, dark skin tone", u8"couple with heart: woman, man, medium-light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: woman, man, medium skin tone, light skin tone", u8"couple with heart: woman, man, medium skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: woman, man, medium skin tone, medium-light skin tone", u8"couple with heart: woman, man, medium skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: woman, man, medium skin tone", u8"couple with heart: woman, man, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: woman, man, medium skin tone, medium-dark skin tone", u8"couple with heart: woman, man, medium skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: woman, man, medium skin tone, dark skin tone", u8"couple with heart: woman, man, medium skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: woman, man, medium-dark skin tone, light skin tone", u8"couple with heart: woman, man, medium-dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: woman, man, medium-dark skin tone, medium-light skin tone", u8"couple with heart: woman, man, medium-dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: woman, man, medium-dark skin tone, medium skin tone", u8"couple with heart: woman, man, medium-dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: woman, man, medium-dark skin tone", u8"couple with heart: woman, man, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: woman, man, medium-dark skin tone, dark skin tone", u8"couple with heart: woman, man, medium-dark skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: woman, man, dark skin tone, light skin tone", u8"couple with heart: woman, man, dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: woman, man, dark skin tone, medium-light skin tone", u8"couple with heart: woman, man, dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: woman, man, dark skin tone, medium skin tone", u8"couple with heart: woman, man, dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: woman, man, dark skin tone, medium-dark skin tone", u8"couple with heart: woman, man, dark skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: woman, man, dark skin tone", u8"couple with heart: woman, man, dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: man, man, light skin tone", u8"couple with heart: man, man, light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: man, man, light skin tone, medium-light skin tone", u8"couple with heart: man, man, light skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: man, man, light skin tone, medium skin tone", u8"couple with heart: man, man, light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: man, man, light skin tone, medium-dark skin tone", u8"couple with heart: man, man, light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: man, man, light skin tone, dark skin tone", u8"couple with heart: man, man, light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: man, man, medium-light skin tone, light skin tone", u8"couple with heart: man, man, medium-light skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: man, man, medium-light skin tone", u8"couple with heart: man, man, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: man, man, medium-light skin tone, medium skin tone", u8"couple with heart: man, man, medium-light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: man, man, medium-light skin tone, medium-dark skin tone", u8"couple with heart: man, man, medium-light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: man, man, medium-light skin tone, dark skin tone", u8"couple with heart: man, man, medium-light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: man, man, medium skin tone, light skin tone", u8"couple with heart: man, man, medium skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: man, man, medium skin tone, medium-light skin tone", u8"couple with heart: man, man, medium skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: man, man, medium skin tone", u8"couple with heart: man, man, medium skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: man, man, medium skin tone, medium-dark skin tone", u8"couple with heart: man, man, medium skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: man, man, medium skin tone, dark skin tone", u8"couple with heart: man, man, medium skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: man, man, medium-dark skin tone, light skin tone", u8"couple with heart: man, man, medium-dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: man, man, medium-dark skin tone, medium-light skin tone", u8"couple with heart: man, man, medium-dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: man, man, medium-dark skin tone, medium skin tone", u8"couple with heart: man, man, medium-dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: man, man, medium-dark skin tone", u8"couple with heart: man, man, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: man, man, medium-dark skin tone, dark skin tone", u8"couple with heart: man, man, medium-dark skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB", u8"couple with heart: man, man, dark skin tone, light skin tone", u8"couple with heart: man, man, dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC", u8"couple with heart: man, man, dark skin tone, medium-light skin tone", u8"couple with heart: man, man, dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD", u8"couple with heart: man, man, dark skin tone, medium skin tone", u8"couple with heart: man, man, dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE", u8"couple with heart: man, man, dark skin tone, medium-dark skin tone", u8"couple with heart: man, man, dark skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF", u8"couple with heart: man, man, dark skin tone", u8"couple with heart: man, man, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB", u8"couple with heart: woman, woman, light skin tone", u8"couple with heart: woman, woman, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC", u8"couple with heart: woman, woman, light skin tone, medium-light skin tone", u8"couple with heart: woman, woman, light skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD", u8"couple with heart: woman, woman, light skin tone, medium skin tone", u8"couple with heart: woman, woman, light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE", u8"couple with heart: woman, woman, light skin tone, medium-dark skin tone", u8"couple with heart: woman, woman, light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF", u8"couple with heart: woman, woman, light skin tone, dark skin tone", u8"couple with heart: woman, woman, light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB", u8"couple with heart: woman, woman, medium-light skin tone, light skin tone", u8"couple with heart: woman, woman, medium-light skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC", u8"couple with heart: woman, woman, medium-light skin tone", u8"couple with heart: woman, woman, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD", u8"couple with heart: woman, woman, medium-light skin tone, medium skin tone", u8"couple with heart: woman, woman, medium-light skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE", u8"couple with heart: woman, woman, medium-light skin tone, medium-dark skin tone", u8"couple with heart: woman, woman, medium-light skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF", u8"couple with heart: woman, woman, medium-light skin tone, dark skin tone", u8"couple with heart: woman, woman, medium-light skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB", u8"couple with heart: woman, woman, medium skin tone, light skin tone", u8"couple with heart: woman, woman, medium skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC", u8"couple with heart: woman, woman, medium skin tone, medium-light skin tone", u8"couple with heart: woman, woman, medium skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD", u8"couple with heart: woman, woman, medium skin tone", u8"couple with heart: woman, woman, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE", u8"couple with heart: woman, woman, medium skin tone, medium-dark skin tone", u8"couple with heart: woman, woman, medium skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF", u8"couple with heart: woman, woman, medium skin tone, dark skin tone", u8"couple with heart: woman, woman, medium skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB", u8"couple with heart: woman, woman, medium-dark skin tone, light skin tone", u8"couple with heart: woman, woman, medium-dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC", u8"couple with heart: woman, woman, medium-dark skin tone, medium-light skin tone", u8"couple with heart: woman, woman, medium-dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD", u8"couple with heart: woman, woman, medium-dark skin tone, medium skin tone", u8"couple with heart: woman, woman, medium-dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE", u8"couple with heart: woman, woman, medium-dark skin tone", u8"couple with heart: woman, woman, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF", u8"couple with heart: woman, woman, medium-dark skin tone, dark skin tone", u8"couple with heart: woman, woman, medium-dark skin tone, dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB", u8"couple with heart: woman, woman, dark skin tone, light skin tone", u8"couple with heart: woman, woman, dark skin tone, light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC", u8"couple with heart: woman, woman, dark skin tone, medium-light skin tone", u8"couple with heart: woman, woman, dark skin tone, medium-light skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD", u8"couple with heart: woman, woman, dark skin tone, medium skin tone", u8"couple with heart: woman, woman, dark skin tone, medium skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE", u8"couple with heart: woman, woman, dark skin tone, medium-dark skin tone", u8"couple with heart: woman, woman, dark skin tone, medium-dark skin tone"},
{u8"couple with heart", u8"\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF", u8"couple with heart: woman, woman, dark skin tone", u8"couple with heart: woman, woman, dark skin tone"},
{u8"light skin tone", u8"\U0001F3FB", u8"tone1", u8"light skin tone"},
{u8"medium-light skin tone", u8"\U0001F3FC", u8"tone2", u8"medium-light skin tone"},
{u8"medium skin tone", u8"\U0001F3FD", u8"tone3", u8"medium skin tone"},
{u8"medium-dark skin tone", u8"\U0001F3FE", u8"tone4", u8"medium-dark skin tone"},
{u8"dark skin tone", u8"\U0001F3FF", u8"tone5", u8"dark skin tone"},
